Package: unitcm
Version: 0.1.2
Title: Client for the 'UniTCM' Traditional Chinese Medicine Platform
Description: Provides functions to query the 'UniTCM' API
    (<https://unitcm.qfxulab.com>), covering herb exploration,
    compound/ADMET (Absorption, Distribution, Metabolism, Excretion, and
    Toxicity) data, disease-formula associations, Traditional Chinese
    Medicine (TCM) ontology, transcriptomics, and gene-disease analysis
    (MIDAS, Mining Integrated Disease Association Sources).
Authors@R: person("Xiao", "Zheng", email = "12519088@zju.edu.cn",
    role = c("aut", "cre"), comment = c(ORCID = "0009-0000-3918-5725"))
Author: Xiao Zheng [aut, cre] (ORCID: <https://orcid.org/0009-0000-3918-5725>)
Maintainer: Xiao Zheng <12519088@zju.edu.cn>
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.1.0)
Imports: httr2 (>= 1.0.0), tibble (>= 3.0.0), dplyr (>= 1.1.0), rlang
        (>= 1.0.0), cli, glue
Suggests: igraph (>= 1.4.0), tidygraph, ggplot2 (>= 3.4.0), ggraph,
        keyring, testthat (>= 3.0.0), httptest2, withr, knitr,
        rmarkdown, pkgdown
URL: https://zx122ty.github.io/UniTCM_R_Package/,
        https://github.com/zx122ty/UniTCM_R_Package
BugReports: https://github.com/zx122ty/UniTCM_R_Package/issues
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-21 02:10:50 UTC; since
Repository: CRAN
Date/Publication: 2026-05-28 11:10:08 UTC
Built: R 4.5.2; ; 2026-05-28 12:39:26 UTC; unix

# Generated by roxygen2: do not edit by hand

export(aggregated_target2np)
export(as_igraph)
export(as_tidygraph)
export(autocomplete_disease)
export(batch_target2np)
export(build_formula_herb_network)
export(build_hct_network)
export(clear_api_key)
export(clear_unitcm_token)
export(convert_gene_ids)
export(detect_communities)
export(export_compound_module)
export(export_compounds)
export(export_datasets)
export(export_herb_compounds)
export(export_herbs)
export(export_ontology)
export(fetch_compound_facets)
export(fetch_dataset_facets)
export(fetch_dataset_stats)
export(fetch_disease_tree)
export(fetch_herb_facets)
export(fetch_home_stats)
export(fetch_latest_submissions)
export(fetch_mechanism_filters)
export(fetch_midas_sources)
export(fetch_midas_stats)
export(fetch_netvis_stats)
export(fetch_omics_type_stats)
export(fetch_ontology_stats)
export(fetch_ontology_tree)
export(fetch_target2np_filters)
export(fetch_target2np_stats)
export(fetch_tcm_classification_stats)
export(fetch_transcriptome_filters)
export(fetch_transcriptome_stats)
export(find_path)
export(flatten_response)
export(get_analysis_data)
export(get_analysis_modules)
export(get_api_key)
export(get_base_url)
export(get_compound)
export(get_compound_admet)
export(get_compound_herbs)
export(get_compound_targets)
export(get_dataset)
export(get_formula)
export(get_formula_doses)
export(get_herb)
export(get_herb_compounds)
export(get_mechanism)
export(get_neighbors)
export(get_node_detail)
export(get_node_metrics)
export(get_ontology_ancestors)
export(get_ontology_by_level)
export(get_ontology_children)
export(get_ontology_descendants)
export(get_ontology_entity)
export(get_similar_datasets)
export(get_subgraph)
export(get_target2np)
export(get_term)
export(get_transcriptome)
export(get_unitcm_token)
export(list_book_sources)
export(list_dosage_forms)
export(list_ontology_categories)
export(list_origin_sources)
export(list_term_categories)
export(list_term_sources)
export(plot_compound_radar)
export(plot_enrichment)
export(plot_network)
export(query_disease_enrichment)
export(query_disease_genes)
export(query_disease_intersection)
export(query_gene_diseases)
export(query_source_comparison)
export(search_compounds)
export(search_datasets)
export(search_formulas)
export(search_herbs)
export(search_mechanisms)
export(search_netvis)
export(search_ontology)
export(search_ontology_mapping)
export(search_target2np)
export(search_target2np_drugclip)
export(search_target2np_sea)
export(search_terms)
export(search_transcriptomes)
export(set_api_key)
export(set_base_url)
export(set_unitcm_token)
export(target2np_multi_source_summary)
export(unitcm_cache_clear)
importFrom(cli,cli_inform)
importFrom(cli,cli_progress_bar)
importFrom(cli,cli_progress_done)
importFrom(cli,cli_progress_update)
importFrom(dplyr,bind_rows)
importFrom(glue,glue)
importFrom(httr2,req_auth_bearer_token)
importFrom(httr2,req_body_json)
importFrom(httr2,req_error)
importFrom(httr2,req_method)
importFrom(httr2,req_perform)
importFrom(httr2,req_retry)
importFrom(httr2,req_throttle)
importFrom(httr2,req_url_path_append)
importFrom(httr2,req_url_query)
importFrom(httr2,req_user_agent)
importFrom(httr2,request)
importFrom(httr2,resp_body_json)
importFrom(httr2,resp_status)
importFrom(rlang,"%||%")
importFrom(rlang,.data)
importFrom(rlang,abort)
importFrom(rlang,arg_match)
importFrom(rlang,check_installed)
importFrom(rlang,inform)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)

# unitcm  <img src="inst/icon.png" align="right" height="139" alt="" />

<!-- badges: start -->
[![R-CMD-check](https://github.com/zx122ty/UniTCM_R_Package/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/zx122ty/UniTCM_R_Package/actions/workflows/R-CMD-check.yaml)
[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)
<!-- badges: end -->

**unitcm** is an R client for the [UniTCM](https://unitcm.qfxulab.com/) Traditional
Chinese Medicine multi-omics platform. It provides tidy, paginated access to
herbs, compounds, disease-formula associations, TCM ontology, and gene-disease
analysis (MIDAS).

## Installation

```r
# install.packages("pak")
pak::pkg_install("zx122ty/UniTCM_R_Package")
```

## Quick Start

```r
library(unitcm)

# Search herbs
search_herbs(q = "ginseng")

# Get compound targets (DrugCLIP deep learning predictions)
get_compound_targets("1")

# Gene-disease enrichment
query_disease_enrichment(c("TP53", "BRCA1", "EGFR", "VEGFA"))
```

## Modules

| Module | Functions | Description |
|--------|-----------|-------------|
| Herb Explorer | `search_herbs()`, `get_herb()` | 800+ herbs with properties and compounds |
| Ingredient Explorer | `search_compounds()`, `get_compound_admet()` | 50,000+ compounds with ADMET and targets |
| Disease-Formula Atlas | `search_formulas()`, `get_formula_doses()` | 259,000+ formula-disease associations |
| TCM Ontology | `search_ontology()`, `export_ontology()` | Hierarchical TCM knowledge system |
| MIDAS | `query_gene_diseases()`, `query_disease_enrichment()` | Multi-source gene-disease analysis |

## Documentation

- [Getting Started](https://zx122ty.github.io/UniTCM_R_Package/articles/unitcm.html)
- [Database Queries](https://zx122ty.github.io/UniTCM_R_Package/articles/database-queries.html)
- [Network Pharmacology Workflow](https://zx122ty.github.io/UniTCM_R_Package/articles/network-pharmacology.html)
- [Gene-Disease Analysis](https://zx122ty.github.io/UniTCM_R_Package/articles/gene-disease-analysis.html)
- [Function Reference](https://zx122ty.github.io/UniTCM_R_Package/reference/index.html)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## ----setup--------------------------------------------------------------------
# library(unitcm)
# library(dplyr)

## ----select-formula-----------------------------------------------------------
# formulas <- search_formulas(q = "insomnia", page_size = 5)
# formulas
# 
# # Get full details for the first formula
# formula <- get_formula(formulas$order[1])
# formula$formula_name
# formula$composition

## ----get-doses----------------------------------------------------------------
# doses <- get_formula_doses(formulas$order[1])
# doses
# 
# # Extract herb names
# herb_names <- doses$herb_name
# herb_names

## ----find-herbs---------------------------------------------------------------
# herb_data <- lapply(herb_names, function(name) {
#   result <- search_herbs(q = name, page_size = 1)
#   if (nrow(result) > 0) result[1, ]
# })
# herb_data <- bind_rows(herb_data)
# herb_data

## ----get-compounds------------------------------------------------------------
# all_compounds <- lapply(herb_data$unitcm_herb_id, function(hid) {
#   comps <- get_herb_compounds(hid, all_pages = TRUE)
#   if (nrow(comps) > 0) comps$herb_id <- hid
#   comps
# })
# all_compounds <- bind_rows(all_compounds)
# 
# # Unique compounds
# unique_compounds <- distinct(all_compounds, unitcm_ingredient_id,
#                              .keep_all = TRUE)
# cat(nrow(unique_compounds), "unique compounds found\n")

## ----get-targets--------------------------------------------------------------
# # Get targets for top compounds (limit to first 10 for speed)
# top_compounds <- head(unique_compounds, 10)
# 
# all_targets <- lapply(top_compounds$unitcm_ingredient_id, function(cid) {
#   targets <- get_compound_targets(cid, method = "drugclip")
#   if (nrow(targets) > 0) targets$compound_id <- cid
#   targets
# })
# all_targets <- bind_rows(all_targets)
# cat(length(unique(all_targets$gene_symbol)), "unique targets found\n")

## ----build-network------------------------------------------------------------
# library(igraph)
# 
# # Herb-Compound edges
# hc_edges <- all_compounds |>
#   select(from = herb_id, to = unitcm_ingredient_id) |>
#   distinct()
# 
# # Compound-Target edges
# ct_edges <- all_targets |>
#   select(from = compound_id, to = gene_symbol) |>
#   distinct()
# 
# # Combine edges
# edges <- bind_rows(hc_edges, ct_edges)
# 
# # Build graph
# g <- graph_from_data_frame(edges, directed = FALSE)
# 
# # Add node type attribute
# V(g)$type <- case_when(
#   V(g)$name %in% herb_data$unitcm_herb_id ~ "herb",
#   V(g)$name %in% unique_compounds$unitcm_ingredient_id ~ "compound",
#   TRUE ~ "target"
# )
# 
# cat("Network:", vcount(g), "nodes,", ecount(g), "edges\n")

## ----plot-network, fig.width = 10, fig.height = 8-----------------------------
# library(ggraph)
# 
# ggraph(g, layout = "fr") +
#   geom_edge_link(alpha = 0.3) +
#   geom_node_point(aes(color = type, size = type)) +
#   scale_color_manual(values = c(herb = "#e74c3c", compound = "#3498db",
#                                 target = "#2ecc71")) +
#   scale_size_manual(values = c(herb = 6, compound = 3, target = 2)) +
#   geom_node_text(
#     aes(label = ifelse(type == "herb", name, "")),
#     repel = TRUE, size = 3
#   ) +
#   theme_void() +
#   labs(title = paste("H-C-T Network:", formula$formula_name))

## ----enrichment---------------------------------------------------------------
# target_genes <- unique(all_targets$gene_symbol)
# enrichment <- query_disease_enrichment(
#   target_genes,
#   p_value_cutoff = 0.05,
#   correction_method = "fdr"
# )
# head(enrichment, 10)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## ----setup--------------------------------------------------------------------
# library(unitcm)

## ----herbs-search-------------------------------------------------------------
# # Text search
# herbs <- search_herbs(q = "ginseng")
# 
# # Multi-value faceted filters
# herbs <- search_herbs(
#   flavors = c("sweet", "bitter"),
#   properties = "warm",
#   toxicity = "non-toxic"
# )
# 
# # View available filter values
# facets <- fetch_herb_facets()
# facets$toxicity
# facets$meridians

## ----herbs-detail-------------------------------------------------------------
# herb <- get_herb("UNITCM_H001")
# herb$herb_english_name
# herb$efficacy
# 
# # Get compounds found in this herb
# compounds <- get_herb_compounds("UNITCM_H001", all_pages = TRUE)
# head(compounds)

## ----herbs-export-------------------------------------------------------------
# export_herbs(q = "ginseng", file = "ginseng_herbs.csv")
# export_herb_compounds("UNITCM_H001", file = "ginseng_compounds.csv")

## ----compounds-search---------------------------------------------------------
# # Drug-like compounds with molecular weight 200-500
# compounds <- search_compounds(
#   mw_min = 200, mw_max = 500,
#   lipinski = "pass",
#   is_drug = TRUE
# )
# 
# # Get facet statistics
# facets <- fetch_compound_facets()
# facets$mw_range

## ----compounds-detail---------------------------------------------------------
# # Full compound record
# compound <- get_compound("UNITCM_I00001")
# compound$component_name
# compound$xref$pubchem_cid
# 
# # ADMET predictions (~90 endpoints)
# admet <- get_compound_admet("UNITCM_I00001")
# admet$caco2_permeability
# admet$hia
# 
# # Predicted targets (DrugCLIP deep learning)
# targets_dc <- get_compound_targets("UNITCM_I00001", method = "drugclip")
# 
# # ChEMBL similarity-based targets
# targets_ch <- get_compound_targets("UNITCM_I00001", method = "chembl")
# 
# # Both sources combined
# targets_all <- get_compound_targets("UNITCM_I00001", method = "both")
# table(targets_all$source)

## ----formulas-search----------------------------------------------------------
# # Search by text
# formulas <- search_formulas(q = "insomnia")
# 
# # Filter by ICD-11 classification
# formulas <- search_formulas(
#   level1 = "Neoplasms",
#   mapping_confidence = c("high", "medium")
# )
# 
# # Browse the disease classification tree
# tree <- fetch_disease_tree()
# 
# # Available filter options
# list_book_sources()
# list_origin_sources()
# list_dosage_forms()

## ----formulas-detail----------------------------------------------------------
# formula <- get_formula(1)
# formula$formula_name
# formula$efficacy
# 
# # Herb composition and dosage
# doses <- get_formula_doses(1)
# doses

## ----ontology-search----------------------------------------------------------
# # Full-text search
# results <- search_ontology("Qi stagnation")
# 
# # Get entity with relations
# entity <- get_ontology_entity("TCM_0001")
# entity$ancestors
# entity$children
# entity$external_mappings
# 
# # Navigate the hierarchy
# children <- get_ontology_children("TCM_0001")
# ancestors <- get_ontology_ancestors("TCM_0001")
# descendants <- get_ontology_descendants("TCM_0001", max_level = 2)

## ----ontology-tree------------------------------------------------------------
# # Fetch tree at depth 2
# tree <- fetch_ontology_tree(depth = 2)
# 
# # Statistics
# stats <- fetch_ontology_stats()
# stats$total_entities
# 
# # Top-level categories
# categories <- list_ontology_categories()

## ----ontology-mapping---------------------------------------------------------
# # Find TCM entities mapped to a MeSH term
# mapped <- search_ontology_mapping("MeSH", "D008516")
# 
# # Export ontology
# export_ontology("csv", file = "tcm_ontology.csv")
# export_ontology("json", depth = 3, file = "tcm_ontology.json")

## ----midas-basic--------------------------------------------------------------
# # Which diseases are associated with these genes?
# g2d <- query_gene_diseases(c("TP53", "BRCA1", "EGFR"))
# g2d
# attr(g2d, "gene_mapping")
# 
# # Which genes are associated with breast cancer?
# d2g <- query_disease_genes("breast cancer")
# d2g
# attr(d2g, "matched_diseases")

## ----midas-enrichment---------------------------------------------------------
# gene_list <- c("TP53", "BRCA1", "EGFR", "VEGFA", "MYC", "KRAS")
# enrichment <- query_disease_enrichment(
#   gene_list,
#   p_value_cutoff = 0.05,
#   correction_method = "fdr"
# )
# enrichment
# attr(enrichment, "total_significant")

## ----midas-utils--------------------------------------------------------------
# # Convert mixed gene identifiers
# convert_gene_ids(c("TP53", "7157", "ENSG00000141510"))
# 
# # Autocomplete disease names
# autocomplete_disease("breast")
# 
# # Available data sources
# fetch_midas_sources()
# 
# # Database statistics
# fetch_midas_stats()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## ----setup--------------------------------------------------------------------
# library(unitcm)
# library(dplyr)

## ----sources------------------------------------------------------------------
# sources <- fetch_midas_sources()
# sources
# 
# stats <- fetch_midas_stats()
# cat(stats$total_associations, "associations across",
#     stats$total_genes, "genes and",
#     stats$total_diseases, "diseases\n")

## ----gene-to-disease----------------------------------------------------------
# genes <- c("TP53", "BRCA1", "EGFR", "VEGFA", "MYC")
# 
# results <- query_gene_diseases(
#   genes,
#   min_sources = 2,
#   scoring_method = "max"
# )
# 
# head(results, 10)
# 
# # Gene ID resolution mapping
# attr(results, "gene_mapping")

## ----disease-to-gene----------------------------------------------------------
# results <- query_disease_genes(
#   "breast cancer",
#   min_sources = 3,
#   page_size = 50
# )
# 
# head(results, 10)
# 
# # Which diseases were matched?
# attr(results, "matched_diseases")

## ----enrichment---------------------------------------------------------------
# gene_list <- c("TP53", "BRCA1", "EGFR", "VEGFA", "MYC", "KRAS",
#                "AKT1", "PIK3CA", "PTEN", "RB1")
# 
# enrichment <- query_disease_enrichment(
#   gene_list,
#   p_value_cutoff = 0.05,
#   correction_method = "fdr",
#   min_hit_count = 3
# )
# 
# cat(attr(enrichment, "total_significant"), "significant diseases from",
#     attr(enrichment, "total_tested"), "tested\n")
# 
# head(enrichment, 10)

## ----convert------------------------------------------------------------------
# mixed_ids <- c("TP53", "7157", "ENSG00000141510", "BRCA1")
# converted <- convert_gene_ids(mixed_ids)
# converted

## ----source-comparison--------------------------------------------------------
# comparison <- query_source_comparison(
#   c("TP53", "BRCA1", "EGFR"),
#   mode = "union"
# )
# 
# # Genes covered by each source
# lapply(comparison$sets, length)
# 
# # Exclusive to each source
# comparison$exclusives

## ----disease-intersection-----------------------------------------------------
# intersection <- query_disease_intersection(
#   c("breast cancer", "lung cancer", "colorectal cancer")
# )
# 
# cat(intersection$total_intersection_genes, "genes shared across all diseases\n")
# head(intersection$targets)

## ----autocomplete-------------------------------------------------------------
# autocomplete_disease("diabet")
# autocomplete_disease("breast")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## ----setup--------------------------------------------------------------------
# library(unitcm)

## ----search-basic-------------------------------------------------------------
# # Free-text search across all fields
# hits <- search_target2np(search = "quercetin")
# hits
# 
# # Exact-match by gene symbol
# tp53 <- search_target2np(
#   search = "TP53",
#   search_field = "gene_symbol",
#   search_mode = "exact"
# )
# tp53
# attr(tp53, "total")

## ----search-filters-----------------------------------------------------------
# # Combine filters
# results <- search_target2np(
#   search = "curcumin",
#   search_field = "compound_name",
#   source_db = "BindingDB",
#   target_organism = "Homo sapiens",
#   activity_type = "IC50"
# )
# results

## ----filter-options-----------------------------------------------------------
# # What filter values exist?
# opts <- fetch_target2np_filters()
# opts$source_db
# opts$target_organism
# opts$activity_type
# 
# # Global database statistics
# stats <- fetch_target2np_stats()
# stats$total_records
# stats$source_db_distribution

## ----detail-------------------------------------------------------------------
# detail <- get_target2np(1)
# detail$compound_name
# detail$gene_symbol
# detail$activity_value
# detail$activity_units
# detail$pmid

## ----batch--------------------------------------------------------------------
# batch <- batch_target2np(c("TP53", "BRCA1", "EGFR", "VEGFA"))
# batch
# attr(batch, "queries_matched")
# attr(batch, "queries_not_found")
# 
# # UniProt-based batch
# batch_up <- batch_target2np(
#   c("P04637", "P38398"),
#   id_type = "uniprot_id"
# )

## ----drugclip-----------------------------------------------------------------
# # High-confidence predictions for quercetin
# dc_high <- search_target2np_drugclip(
#   search = "quercetin",
#   search_field = "compound_name",
#   confidence = "high"
# )
# dc_high
# 
# # Score-based filtering
# dc <- search_target2np_drugclip(
#   search = "EGFR",
#   search_field = "gene_symbol",
#   min_score = 0.7
# )
# dc

## ----sea----------------------------------------------------------------------
# # High-confidence SEA predictions
# sea_high <- search_target2np_sea(
#   search = "quercetin",
#   search_field = "compound_name",
#   confidence = "high"
# )
# sea_high
# 
# # Filter by adjusted p-value
# sea <- search_target2np_sea(
#   search = "TP53",
#   search_field = "gene_symbol",
#   max_pvalue = 0.01
# )
# sea

## ----multi-source-------------------------------------------------------------
# summary <- target2np_multi_source_summary(
#   search = "TP53",
#   search_field = "gene_symbol"
# )
# 
# # How many results per source?
# summary$source_counts
# 
# # Target overlap across data sources
# summary$target_overlap
# 
# # Confidence-level distribution for each source
# summary$confidence_distribution
# 
# # Compound-target pairs found in >= 2 sources
# summary$cross_validated
# 
# # Natural-language interpretation
# cat(summary$suggestion_text)

## ----aggregated---------------------------------------------------------------
# # Pairs seen in >= 3 databases
# agg <- aggregated_target2np(
#   search = "quercetin",
#   min_sources = 3
# )
# agg
# 
# # Include DrugCLIP/SEA prediction counts as additional sources
# agg_pred <- aggregated_target2np(
#   search = "quercetin",
#   min_sources = 2,
#   include_predictions = TRUE
# )
# agg_pred

## ----workflow-----------------------------------------------------------------
# # 1. Check experimental evidence
# exp <- search_target2np(
#   search = "quercetin",
#   search_field = "compound_name",
#   search_mode = "fuzzy",
#   all_pages = TRUE
# )
# nrow(exp)
# 
# # 2. Get multi-source summary in one call
# ms <- target2np_multi_source_summary(
#   search = "quercetin",
#   search_field = "compound_name",
#   search_mode = "fuzzy"
# )
# ms$source_counts
# ms$cross_validated
# 
# # 3. Batch-check the top cross-validated targets
# top_genes <- unique(vapply(
#   ms$cross_validated, `[[`, character(1), "gene_symbol"
# ))
# if (length(top_genes) > 0) {
#   batch_detail <- batch_target2np(top_genes)
#   batch_detail
# }

#' Get NetVis network statistics
#'
#' @return A named list with node counts (`formula`, `herb`, `compound`,
#'   `target`, `disease`) and `edges` sub-list.
#' @export
#' @examples
#' \dontrun{
#' fetch_netvis_stats()
#' }
fetch_netvis_stats <- function() {
  unitcm_request("/netvis/stats")
}

#' Search NetVis nodes
#'
#' @param q Search query.
#' @param type Node type filter: `"all"` (default), `"formula"`, `"herb"`,
#'   `"compound"`, `"target"`, or `"disease"`.
#' @param limit Maximum results (default 20).
#' @return A [tibble::tibble()] with columns: `id`, `type`, `label`,
#'   `label_cn`, `degree`.
#' @export
#' @examples
#' \dontrun{
#' search_netvis("ginseng", type = "herb")
#' }
search_netvis <- function(q, type = "all", limit = 20L) {
  resp <- unitcm_request("/netvis/search",
    query = list(q = q, type = type, limit = limit))
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }
  tibble::tibble()
}

#' Get neighbors of a node
#'
#' @param node_id Node ID (e.g. `"H:UNITCM_H001"`).
#' @param depth Neighbor depth (1--3, default 1).
#' @param limit Maximum neighbors (max 200, default 50).
#' @param node_types Comma-separated node types to include.
#' @return A named list with `$nodes` ([tibble::tibble()]), `$edges`
#'   ([tibble::tibble()]), and `$has_more`.
#' @export
#' @examples
#' \dontrun{
#' get_neighbors("H:UNITCM_H001", depth = 1)
#' }
get_neighbors <- function(node_id, depth = 1L, limit = 50L,
                          node_types = NULL) {
  resp <- unitcm_request(paste0("/netvis/neighbors/", normalize_node_id(node_id)),
    query = list(depth = depth, limit = limit, node_types = node_types))
  parse_graph_response(resp)
}

#' Get subgraph for a set of nodes
#'
#' @param node_ids Character vector of node IDs (max 50).
#' @param limit Maximum edges (default 200).
#' @return A named list with `$nodes` ([tibble::tibble()]), `$edges`
#'   ([tibble::tibble()]), and `$has_more`.
#' @export
#' @examples
#' \dontrun{
#' get_subgraph(c("H:UNITCM_H001", "C:UNITCM_I00001"))
#' }
get_subgraph <- function(node_ids, limit = 200L) {
  resp <- unitcm_request("/netvis/subgraph",
    query = list(nodes = paste(normalize_node_id(node_ids), collapse = ","), limit = limit))
  parse_graph_response(resp)
}

#' Find shortest path between two nodes
#'
#' @param source Source node ID.
#' @param target Target node ID.
#' @param max_depth Maximum path depth (max 8, default 4).
#' @return A named list with `$nodes` ([tibble::tibble()]) and `$edges`
#'   ([tibble::tibble()]).
#' @export
#' @examples
#' \dontrun{
#' find_path("H:UNITCM_H001", "T:TP53")
#' }
find_path <- function(source, target, max_depth = 4L) {
  resp <- unitcm_request("/netvis/path",
    query = list(source = normalize_node_id(source),
                 target = normalize_node_id(target),
                 max_depth = max_depth))
  parse_graph_response(resp)
}

#' Get node detail
#'
#' @param node_id Node ID.
#' @return A named list with fields: `id`, `type`, `label`, `label_cn`,
#'   `properties`, `detail_url`.
#' @export
#' @examples
#' \dontrun{
#' get_node_detail("H:UNITCM_H001")
#' }
get_node_detail <- function(node_id) {
  unitcm_request(paste0("/netvis/node/", normalize_node_id(node_id)))
}

#' Get node metrics
#'
#' @param node_id Node ID.
#' @return A named list with fields: `node_id`, `degree`, `neighbor_types`.
#' @export
#' @examples
#' \dontrun{
#' get_node_metrics("H:UNITCM_H001")
#' }
get_node_metrics <- function(node_id) {
  unitcm_request(paste0("/netvis/metrics/", normalize_node_id(node_id)))
}

#' Detect communities in a graph
#'
#' POST a graph (nodes + edges) to the server for community detection.
#'
#' @param nodes Character vector of node IDs.
#' @param edges A data frame or list of edges, each with `source` and
#'   `target` fields.
#' @return A [tibble::tibble()] with columns `node_id` and `community_id`.
#' @export
#' @examples
#' \dontrun{
#' detect_communities(
#'   nodes = c("A", "B", "C"),
#'   edges = data.frame(source = c("A", "B"), target = c("B", "C"))
#' )
#' }
detect_communities <- function(nodes, edges) {
  if (is.data.frame(edges)) {
    edge_list <- lapply(seq_len(nrow(edges)), function(i) {
      list(source = edges$source[i], target = edges$target[i])
    })
  } else {
    edge_list <- edges
  }

  resp <- unitcm_request("/netvis/communities", method = "POST",
    body = list(nodes = as.list(nodes), edges = edge_list))

  comms <- resp[["communities"]]
  if (is.null(comms)) return(tibble::tibble(node_id = character(), community_id = integer()))

  tibble::tibble(
    node_id = names(comms),
    community_id = as.integer(unlist(comms))
  )
}

#' Parse a graph response into tibbles
#'
#' @param resp API response list with `nodes` and `edges`.
#' @return A named list with `$nodes` and `$edges` as tibbles, plus any
#'   extra fields (e.g. `$has_more`).
#' @noRd
parse_graph_response <- function(resp) {
  result <- list()

  nodes <- resp[["nodes"]]
  if (is.data.frame(nodes)) {
    result$nodes <- tibble::as_tibble(nodes)
  } else if (is.list(nodes) && length(nodes) > 0L) {
    result$nodes <- tryCatch(tibble::as_tibble(nodes),
      error = function(e) tibble::tibble())
  } else {
    result$nodes <- tibble::tibble()
  }

  edges <- resp[["edges"]]
  if (is.data.frame(edges)) {
    result$edges <- tibble::as_tibble(edges)
  } else if (is.list(edges) && length(edges) > 0L) {
    result$edges <- tryCatch(tibble::as_tibble(edges),
      error = function(e) tibble::tibble())
  } else {
    result$edges <- tibble::tibble()
  }

  extra_fields <- setdiff(names(resp), c("nodes", "edges"))
  for (f in extra_fields) {
    result[[f]] <- resp[[f]]
  }

  result
}

test_that("search_netvis returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(id = "H:UNITCM_H001", type = "herb",
                 label = "Ginseng", degree = 50L,
                 stringsAsFactors = FALSE)
    }
  )
  result <- search_netvis("ginseng")
  expect_s3_class(result, "tbl_df")
})

test_that("get_neighbors returns graph structure", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(
        nodes = data.frame(id = c("H:1", "C:1"), type = c("herb", "compound"),
                           stringsAsFactors = FALSE),
        edges = data.frame(source = "H:1", target = "C:1",
                           stringsAsFactors = FALSE),
        has_more = list(compound = FALSE)
      )
    }
  )
  result <- get_neighbors("H:1")
  expect_s3_class(result$nodes, "tbl_df")
  expect_s3_class(result$edges, "tbl_df")
  expect_equal(nrow(result$nodes), 2L)
})

test_that("detect_communities returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, method, body, ...) {
      list(communities = list(A = 1L, B = 1L, C = 2L), num_communities = 2L)
    }
  )
  result <- detect_communities(
    c("A", "B", "C"),
    data.frame(source = c("A", "B"), target = c("B", "C"))
  )
  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 3L)
  expect_true("community_id" %in% names(result))
})

test_that("get_node_detail returns a list", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(id = "H:1", type = "herb", label = "Ginseng")
    }
  )
  result <- get_node_detail("H:1")
  expect_type(result, "list")
})

test_that("fetch_netvis_stats returns a list", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(formula = 100L, herb = 500L, compound = 5000L)
    }
  )
  result <- fetch_netvis_stats()
  expect_type(result, "list")
})

# Package-level environment for session state
.unitcm_env <- new.env(parent = emptyenv())
.unitcm_env$base_url <- NULL
.unitcm_env$token <- NULL
.unitcm_env$api_key <- NULL

#' Set the UniTCM API base URL
#'
#' @param url A character string. The base URL for the UniTCM API,
#'   e.g. `"https://unitcm.qfxulab.com/api/v1"`.
#' @return Invisible previous URL value.
#' @export
#' @examples
#' \dontrun{
#' set_base_url("https://unitcm.qfxulab.com/api/v1")
#' }
set_base_url <- function(url) {
  old <- .unitcm_env$base_url
  .unitcm_env$base_url <- url
  cli::cli_inform("UniTCM base URL set to {.url {url}}")
  invisible(old)
}

#' Get the UniTCM API base URL
#'
#' Checks in order: (1) session value set via [set_base_url()],
#' (2) option `unitcm.base_url`, (3) environment variable `UNITCM_BASE_URL`,
#' (4) hardcoded default.
#'
#' @return A character string.
#' @export
#' @examples
#' \dontrun{
#' get_base_url()
#' }
get_base_url <- function() {
  url <- .unitcm_env$base_url
  if (!is.null(url) && nzchar(url)) return(url)

  url <- getOption("unitcm.base_url")
  if (!is.null(url) && nzchar(url)) return(url)

  url <- Sys.getenv("UNITCM_BASE_URL", unset = "")
  if (nzchar(url)) return(url)

  "https://unitcm.qfxulab.com/api/v1"
}

#' Set a UniTCM API token
#'
#' Stores the token in session memory. Optionally also stores it in the
#' system keyring (requires the \pkg{keyring} package).
#'
#' @param token A character string. The bearer token.
#' @param keyring Logical. If `TRUE`, also store in system keyring.
#' @return Invisible `NULL`.
#' @export
#' @examples
#' \dontrun{
#' set_unitcm_token("my-secret-token")
#' }
set_unitcm_token <- function(token, keyring = FALSE) {
  .unitcm_env$token <- token
  if (isTRUE(keyring)) {
    check_pkg("keyring", reason = "to store token in system keyring")
    keyring::key_set_with_value("unitcm", username = "token", password = token)
    cli::cli_inform("Token stored in session and system keyring.")
  } else {
    cli::cli_inform("Token stored in session.")
  }
  invisible(NULL)
}

#' Get the UniTCM API token
#'
#' Checks in order: (1) session value set via [set_unitcm_token()],
#' (2) environment variable `UNITCM_TOKEN`, (3) system keyring.
#'
#' @return A character string, or `NULL` if no token is found.
#' @export
#' @examples
#' \dontrun{
#' get_unitcm_token()
#' }
get_unitcm_token <- function() {
  token <- .unitcm_env$token
  if (!is.null(token) && nzchar(token)) return(token)

  env_token <- Sys.getenv("UNITCM_TOKEN", unset = "")
  if (nzchar(env_token)) return(env_token)

  if (requireNamespace("keyring", quietly = TRUE)) {
    kr_token <- tryCatch(
      keyring::key_get("unitcm", username = "token"),
      error = function(e) NULL
    )
    if (!is.null(kr_token) && nzchar(kr_token)) return(kr_token)
  }

  NULL
}

#' Clear the UniTCM API token
#'
#' Removes the token from session memory and optionally from the system
#' keyring.
#'
#' @param keyring Logical. If `TRUE`, also remove from system keyring.
#' @return Invisible `NULL`.
#' @export
#' @examples
#' \dontrun{
#' clear_unitcm_token()
#' }
clear_unitcm_token <- function(keyring = FALSE) {
  .unitcm_env$token <- NULL
  if (isTRUE(keyring) && requireNamespace("keyring", quietly = TRUE)) {
    tryCatch(
      keyring::key_delete("unitcm", username = "token"),
      error = function(e) NULL
    )
    cli::cli_inform("Token cleared from session and system keyring.")
  } else {
    cli::cli_inform("Token cleared from session.")
  }
  invisible(NULL)
}


#' Set a UniTCM API Key
#'
#' Stores the API key in session memory. Optionally also stores it in the
#' system keyring (requires the \pkg{keyring} package).
#'
#' @param api_key A character string. The API key (starts with `unitcm_`).
#' @param keyring Logical. If `TRUE`, also store in system keyring.
#' @return Invisible `NULL`.
#' @export
#' @examples
#' \dontrun{
#' set_api_key("unitcm_your_key_here")
#' }
set_api_key <- function(api_key, keyring = FALSE) {
  .unitcm_env$api_key <- api_key
  if (isTRUE(keyring)) {
    check_pkg("keyring", reason = "to store API key in system keyring")
    keyring::key_set_with_value("unitcm", username = "api_key", password = api_key)
    cli::cli_inform("API key stored in session and system keyring.")
  } else {
    cli::cli_inform("API key stored in session.")
  }
  invisible(NULL)
}

#' Get the UniTCM API Key
#'
#' Checks in order: (1) session value set via [set_api_key()],
#' (2) environment variable `UNITCM_API_KEY`, (3) system keyring.
#'
#' @return A character string, or `NULL` if no API key is found.
#' @export
#' @examples
#' \dontrun{
#' get_api_key()
#' }
get_api_key <- function() {
  key <- .unitcm_env$api_key
  if (!is.null(key) && nzchar(key)) return(key)

  env_key <- Sys.getenv("UNITCM_API_KEY", unset = "")
  if (nzchar(env_key)) return(env_key)

  if (requireNamespace("keyring", quietly = TRUE)) {
    kr_key <- tryCatch(
      keyring::key_get("unitcm", username = "api_key"),
      error = function(e) NULL
    )
    if (!is.null(kr_key) && nzchar(kr_key)) return(kr_key)
  }

  NULL
}

#' Clear the UniTCM API Key
#'
#' Removes the API key from session memory and optionally from the system
#' keyring.
#'
#' @param keyring Logical. If `TRUE`, also remove from system keyring.
#' @return Invisible `NULL`.
#' @export
#' @examples
#' \dontrun{
#' clear_api_key()
#' }
clear_api_key <- function(keyring = FALSE) {
  .unitcm_env$api_key <- NULL
  if (isTRUE(keyring) && requireNamespace("keyring", quietly = TRUE)) {
    tryCatch(
      keyring::key_delete("unitcm", username = "api_key"),
      error = function(e) NULL
    )
    cli::cli_inform("API key cleared from session and system keyring.")
  } else {
    cli::cli_inform("API key cleared from session.")
  }
  invisible(NULL)
}

test_that("set/get/clear token lifecycle works", {
  withr::local_envvar(UNITCM_TOKEN = "")
  clear_unitcm_token()

  expect_null(get_unitcm_token())

  set_unitcm_token("test-token-123")
  expect_equal(get_unitcm_token(), "test-token-123")

  clear_unitcm_token()
  expect_null(get_unitcm_token())
})

test_that("get_unitcm_token falls back to env var", {
  clear_unitcm_token()
  withr::local_envvar(UNITCM_TOKEN = "env-token-456")
  expect_equal(get_unitcm_token(), "env-token-456")
})

test_that("session token takes priority over env var", {
  withr::local_envvar(UNITCM_TOKEN = "env-token")
  set_unitcm_token("session-token")
  expect_equal(get_unitcm_token(), "session-token")
  clear_unitcm_token()
})

test_that("set/get base URL works", {
  old <- get_base_url()
  set_base_url("https://custom.example.com/api")
  expect_equal(get_base_url(), "https://custom.example.com/api")
  set_base_url(old)
})

test_that("get_base_url returns default when unset", {
  .unitcm_env <- unitcm:::.unitcm_env
  old <- .unitcm_env$base_url
  .unitcm_env$base_url <- NULL
  withr::local_envvar(UNITCM_BASE_URL = "")
  withr::local_options(unitcm.base_url = NULL)
  expect_equal(get_base_url(), "https://unitcm.qfxulab.com/api/v1")
  .unitcm_env$base_url <- old
})

#' Search TCM bilingual corpus terms
#'
#' Query the TCM Bilingual Corpus with optional text search and filters.
#'
#' @param q Optional search query string.
#' @param sources Data source filter (character vector, comma-collapsed).
#' @param category Category filter (character vector, comma-collapsed).
#' @param page Page number (default 1).
#' @param page_size Results per page (default 20, max 100).
#' @param all_pages If `TRUE`, fetch all pages via auto-pagination.
#' @return A [tibble::tibble()] of terms with attribute `"total"`.
#' @export
#' @examples
#' \dontrun{
#' search_terms(q = "ginseng")
#' }
search_terms <- function(q = NULL, sources = NULL, category = NULL,
                         page = 1L, page_size = 20L, all_pages = FALSE) {
  query <- list(
    q = q,
    sources = collapse_param(sources, ","),
    category = collapse_param(category, ",")
  )

  if (isTRUE(all_pages)) {
    return(unitcm_paginate("/tcm-term", query = query))
  }

  query[["page"]] <- page
  query[["page_size"]] <- page_size
  resp <- unitcm_request("/tcm-term", query = query)

  items <- resp[["items"]] %||% list()
  result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
  attr(result, "total") <- resp[["total"]] %||% 0L
  result
}

#' Get a single term by ID
#'
#' Retrieve full detail for one term from the TCM Bilingual Corpus.
#'
#' @param term_id The term ID.
#' @return A named list with fields including `chinese_name`, `pinyin`,
#'   `english_name`, `latin_name`, `description_english`, etc.
#' @export
#' @examples
#' \dontrun{
#' get_term("TCM_T001")
#' }
get_term <- function(term_id) {
  unitcm_request(paste0("/tcm-term/", term_id))
}

#' List term sources
#'
#' @return A [tibble::tibble()] with columns `value`, `label`, `count`.
#' @export
#' @examples
#' \dontrun{
#' list_term_sources()
#' }
list_term_sources <- function() {
  resp <- unitcm_request("/tcm-term/sources")
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }
  tibble::tibble()
}

#' List term categories
#'
#' @return A [tibble::tibble()] with columns `value`, `label`, `count`.
#' @export
#' @examples
#' \dontrun{
#' list_term_categories()
#' }
list_term_categories <- function() {
  resp <- unitcm_request("/tcm-term/categories")
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }
  tibble::tibble()
}

test_that("search_terms returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(
        items = data.frame(id = "T1", chinese_name = "\u4eba\u53c2",
                           english_name = "Ginseng",
                           stringsAsFactors = FALSE),
        total = 1L
      )
    }
  )

  result <- search_terms(q = "ginseng")
  expect_s3_class(result, "tbl_df")
  expect_equal(attr(result, "total"), 1L)
})

test_that("get_term returns a list", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(id = "T1", chinese_name = "\u4eba\u53c2", english_name = "Ginseng")
    }
  )
  result <- get_term("T1")
  expect_type(result, "list")
})

test_that("list_term_sources returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(value = "src1", label = "Source 1", count = 10L,
                 stringsAsFactors = FALSE)
    }
  )
  result <- list_term_sources()
  expect_s3_class(result, "tbl_df")
})

test_that("list_term_categories returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(value = "cat1", label = "Category 1", count = 5L,
                 stringsAsFactors = FALSE)
    }
  )
  result <- list_term_categories()
  expect_s3_class(result, "tbl_df")
})

#' Make a request to the UniTCM API
#'
#' Low-level function that builds and performs an HTTP request to the
#' UniTCM API. All higher-level functions in this package use this
#' internally.
#'
#' @param path API path appended to the base URL, e.g. `"/herbs"`.
#' @param method HTTP method, either `"GET"` or `"POST"`.
#' @param query Named list of query parameters. `NULL` values are removed.
#' @param body A list to send as JSON body (POST requests only).
#' @param base_url Override the base URL for this request.
#' @return Parsed JSON response as an R list (vectors simplified).
#' @noRd
unitcm_request <- function(path, method = "GET", query = list(),
                           body = NULL, base_url = NULL) {
  base <- base_url %||% get_base_url()

  req <- httr2::request(base) |>
    httr2::req_url_path_append(path) |>
    httr2::req_user_agent("unitcm R package (https://github.com/zx122ty/UniTCM_R_Package)") |>
    httr2::req_retry(max_tries = 3L, is_transient = is_transient_status) |>
    httr2::req_throttle(rate = 80 / 60) |>
    httr2::req_error(body = unitcm_error_body)

  api_key <- get_api_key()
  if (!is.null(api_key)) {
    req <- req |> httr2::req_headers(`X-API-Key` = api_key)
  } else {
    token <- get_unitcm_token()
    if (!is.null(token)) {
      req <- req |> httr2::req_auth_bearer_token(token)
    }
  }

  query <- Filter(Negate(is.null), query)
  if (length(query) > 0L) {
    req <- do.call(httr2::req_url_query, c(list(req), query))
  }

  if (method == "POST") {
    req <- req |> httr2::req_method("POST")
    if (!is.null(body)) {
      req <- req |> httr2::req_body_json(body)
    }
  }

  resp <- httr2::req_perform(req)

  remaining <- httr2::resp_header(resp, "X-RateLimit-Remaining")
  if (!is.null(remaining) && as.integer(remaining) < 5000) {
    warning(sprintf("[UniTCM] API quota remaining: %s. Visit your profile to refresh the key.", remaining),
            call. = FALSE)
  }

  httr2::resp_body_json(resp, simplifyVector = TRUE)
}

#' Download raw bytes from the API (for CSV/file exports)
#'
#' @inheritParams unitcm_request
#' @param file Path to write the downloaded file.
#' @return Invisible file path.
#' @noRd
unitcm_download <- function(path, query = list(), file, base_url = NULL) {
  base <- base_url %||% get_base_url()

  req <- httr2::request(base) |>
    httr2::req_url_path_append(path) |>
    httr2::req_user_agent("unitcm R package (https://github.com/zx122ty/UniTCM_R_Package)") |>
    httr2::req_retry(max_tries = 3L, is_transient = is_transient_status) |>
    httr2::req_error(body = unitcm_error_body)

  api_key <- get_api_key()
  if (!is.null(api_key)) {
    req <- req |> httr2::req_headers(`X-API-Key` = api_key)
  } else {
    token <- get_unitcm_token()
    if (!is.null(token)) {
      req <- req |> httr2::req_auth_bearer_token(token)
    }
  }

  query <- Filter(Negate(is.null), query)
  if (length(query) > 0L) {
    req <- do.call(httr2::req_url_query, c(list(req), query))
  }

  httr2::req_perform(req, path = file)
  invisible(file)
}

#' Check if an HTTP status is transient (for retry logic)
#' @noRd
is_transient_status <- function(resp) {
  httr2::resp_status(resp) %in% c(429L, 503L)
}

test_that("unitcm_request builds correct URL with query params", {
  local_mocked_bindings(
    req_perform = function(req, ...) {
      # Capture the built request for inspection
      structure(list(url = req$url, headers = req$headers, body = req$body),
                class = "httr2_response")
    },
    resp_body_json = function(resp, ...) list(items = list(), total = 0L),
    resp_status = function(resp) 200L,
    .package = "httr2"
  )

  result <- unitcm_request("/herbs", query = list(q = "ginseng", page = 1L))
  expect_type(result, "list")
})

test_that("NULL query params are filtered out", {
  query <- list(q = "test", page = NULL, empty = NULL, size = 20)
  filtered <- Filter(Negate(is.null), query)
  expect_equal(length(filtered), 2L)
  expect_equal(filtered$q, "test")
  expect_equal(filtered$size, 20)
})

test_that("is_transient_status identifies transient codes", {
  mock_resp <- function(status) {
    structure(list(status_code = status), class = "httr2_response")
  }
  local_mocked_bindings(
    resp_status = function(resp) resp$status_code,
    .package = "httr2"
  )

  expect_true(unitcm:::is_transient_status(mock_resp(429L)))
  expect_true(unitcm:::is_transient_status(mock_resp(503L)))
  expect_false(unitcm:::is_transient_status(mock_resp(200L)))
  expect_false(unitcm:::is_transient_status(mock_resp(404L)))
})

test_that("unitcm_error_body extracts detail from JSON", {
  mock_resp <- structure(
    list(status_code = 404L, body = charToRaw('{"detail":"Herb not found"}')),
    class = "httr2_response"
  )
  local_mocked_bindings(
    resp_status = function(resp) resp$status_code,
    resp_body_json = function(resp, ...) list(detail = "Herb not found"),
    .package = "httr2"
  )

  msg <- unitcm_error_body(mock_resp)
  expect_match(msg, "not found")
  expect_match(msg, "Herb not found")
})

test_that("unitcm_error_body handles 429 and 500", {
  local_mocked_bindings(
    resp_body_json = function(resp, ...) list(),
    .package = "httr2"
  )

  mock_429 <- structure(list(status_code = 429L), class = "httr2_response")
  local_mocked_bindings(resp_status = function(resp) 429L, .package = "httr2")
  expect_match(unitcm_error_body(mock_429), "Rate limit")

  local_mocked_bindings(resp_status = function(resp) 500L, .package = "httr2")
  mock_500 <- structure(list(status_code = 500L), class = "httr2_response")
  expect_match(unitcm_error_body(mock_500), "server error")
})

#' Search terms molecular mechanisms
#'
#' Query the Terms Molecular Mechanisms database with optional filters.
#'
#' @param search Optional text search query.
#' @param category Category filter.
#' @param omics_type Omics type filter.
#' @param evidence_level Evidence level filter.
#' @param confidence_level Confidence level filter.
#' @param study_type Study type filter.
#' @param species Species filter.
#' @param page Page number (default 1).
#' @param page_size Results per page (default 20).
#' @param all_pages If `TRUE`, fetch all pages via auto-pagination.
#' @return A [tibble::tibble()] of mechanism terms with attribute `"total"`.
#' @export
#' @examples
#' \dontrun{
#' search_mechanisms(search = "Qi deficiency")
#' }
search_mechanisms <- function(search = NULL, category = NULL,
                              omics_type = NULL, evidence_level = NULL,
                              confidence_level = NULL, study_type = NULL,
                              species = NULL, page = 1L, page_size = 20L,
                              all_pages = FALSE) {
  query <- list(
    search = search,
    category = category,
    omics_type = omics_type,
    evidence_level = evidence_level,
    confidence_level = confidence_level,
    study_type = study_type,
    species = species
  )

  if (isTRUE(all_pages)) {
    return(unitcm_paginate("/terms-molecular-mechanisms/", query = query))
  }

  query[["page"]] <- page
  query[["page_size"]] <- page_size
  resp <- unitcm_request("/terms-molecular-mechanisms/", query = query)

  items <- resp[["items"]] %||% list()
  result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
  attr(result, "total") <- resp[["total"]] %||% 0L
  result
}

#' Get a single mechanism term by ID
#'
#' Retrieve full detail for one term including nested arrays of biomarkers,
#' pathways, gene targets, metabolites, etc.
#'
#' @param term_id The mechanism term ID.
#' @return A named list with ~50 fields. Nested arrays (e.g. `biomarkers`,
#'   `signaling_pathways`, `gene_targets`) are returned as-is (lists).
#' @export
#' @examples
#' \dontrun{
#' get_mechanism("TMM001")
#' }
get_mechanism <- function(term_id) {
  unitcm_request(paste0("/terms-molecular-mechanisms/", term_id, "/"))
}

#' Get mechanism filter options
#'
#' Fetches all 6 filter option endpoints and returns them as a named list
#' of tibbles.
#'
#' @return A named list with elements: `categories`, `omics_types`,
#'   `evidence_levels`, `confidence_levels`, `study_types`, `species`.
#'   Each is a [tibble::tibble()] with columns `value`, `label`, `count`.
#' @export
#' @examples
#' \dontrun{
#' filters <- fetch_mechanism_filters()
#' filters$categories
#' }
fetch_mechanism_filters <- function() {
  endpoints <- list(
    categories = "/terms-molecular-mechanisms/filters/categories/",
    omics_types = "/terms-molecular-mechanisms/filters/omics-types/",
    evidence_levels = "/terms-molecular-mechanisms/filters/evidence-levels/",
    confidence_levels = "/terms-molecular-mechanisms/filters/confidence-levels/",
    study_types = "/terms-molecular-mechanisms/filters/study-types/",
    species = "/terms-molecular-mechanisms/filters/species/"
  )

  lapply(endpoints, function(path) {
    resp <- unitcm_request(path)
    if (is.data.frame(resp)) return(tibble::as_tibble(resp))
    if (is.list(resp) && length(resp) > 0L) {
      return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
    }
    tibble::tibble()
  })
}

test_that("search_mechanisms returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(
        items = data.frame(tcm_term_id = "TMM001",
                           tcm_term_en = "Qi deficiency",
                           stringsAsFactors = FALSE),
        total = 1L
      )
    }
  )
  result <- search_mechanisms(search = "Qi")
  expect_s3_class(result, "tbl_df")
})

test_that("get_mechanism returns a list", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(tcm_term_id = "TMM001", biomarkers = list("IL-6", "TNF-alpha"))
    }
  )
  result <- get_mechanism("TMM001")
  expect_type(result, "list")
  expect_type(result$biomarkers, "list")
})

test_that("fetch_mechanism_filters returns named list of tibbles", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(value = "cat1", label = "Category 1", count = 5L,
                 stringsAsFactors = FALSE)
    }
  )
  result <- fetch_mechanism_filters()
  expect_type(result, "list")
  expect_true(all(c("categories", "omics_types", "species") %in% names(result)))
})

#' @keywords internal
"_PACKAGE"

#' @importFrom tibble tibble as_tibble
#' @importFrom rlang %||% abort arg_match check_installed inform .data
#' @importFrom dplyr bind_rows
#' @importFrom cli cli_inform cli_progress_bar cli_progress_update cli_progress_done
#' @importFrom glue glue
#' @importFrom httr2 request
#' @importFrom httr2 req_url_path_append
#' @importFrom httr2 req_user_agent
#' @importFrom httr2 req_retry
#' @importFrom httr2 req_throttle
#' @importFrom httr2 req_error
#' @importFrom httr2 req_auth_bearer_token
#' @importFrom httr2 req_url_query
#' @importFrom httr2 req_method
#' @importFrom httr2 req_body_json
#' @importFrom httr2 req_perform
#' @importFrom httr2 resp_body_json
#' @importFrom httr2 resp_status
NULL

#' Flatten a nested API response to a tibble
#'
#' Recursively flattens a nested list into a single-row tibble. Nested lists
#' that cannot be further flattened are kept as list-columns.
#'
#' @param x A named list from an API response.
#' @return A single-row [tibble::tibble()].
#' @export
#' @examples
#' \dontrun{
#' herb <- get_herb("UNITCM_H001")
#' flatten_response(herb)
#' }
flatten_response <- function(x) {
  if (!is.list(x) || is.null(names(x))) {
    rlang::abort("`x` must be a named list.")
  }

  flat <- list()
  for (nm in names(x)) {
    val <- x[[nm]]
    if (is.null(val)) {
      flat[[nm]] <- NA
    } else if (is.atomic(val) && length(val) == 1L) {
      flat[[nm]] <- val
    } else {
      flat[[nm]] <- list(val)
    }
  }
  tibble::as_tibble(flat)
}

#' Clear unitcm cache
#'
#' Clears any memoized API results. Currently a no-op placeholder for
#' future caching support.
#'
#' @return Invisible `NULL`.
#' @export
unitcm_cache_clear <- function() {
  cli::cli_inform("Cache cleared (no active cache in current version).")
  invisible(NULL)
}

#' Check that a suggested package is installed
#'
#' @param pkg Package name as a string.
#' @param reason Why the package is needed.
#' @return Invisible `TRUE` if installed.
#' @noRd
check_pkg <- function(pkg, reason = NULL) {
  rlang::check_installed(pkg, reason = reason)
}

#' Normalize a UniTCM entity ID to its numeric form
#'
#' The UniTCM API expects numeric IDs (e.g. `1`), while users typically
#' refer to entities by their prefixed display form (e.g. `"UNITCM_H001"`,
#' `"UNITCM_I00001"`). This helper extracts the first integer it finds in
#' the input and returns it as a zero-stripped string. Pure integers and
#' integer-like strings pass through unchanged.
#'
#' @param x A character or numeric ID. Accepts forms like `"UNITCM_H001"`,
#'   `"UNITCM_I00001"`, `"1"`, or `1L`.
#' @return A character string of digits (no leading zeros).
#' @noRd
normalize_id <- function(x) {
  if (is.null(x) || length(x) == 0L || (length(x) == 1L && is.na(x))) {
    rlang::abort("ID cannot be NULL, NA, or empty.")
  }
  if (length(x) > 1L) {
    return(vapply(x, normalize_id, character(1L), USE.NAMES = FALSE))
  }
  if (is.numeric(x)) {
    return(as.character(as.integer(x)))
  }
  s <- as.character(x)
  m <- regmatches(s, regexpr("\\d+", s))
  if (length(m) == 0L || !nzchar(m)) {
    rlang::abort(sprintf("No numeric ID found in '%s'.", s))
  }
  as.character(as.integer(m))
}

#' Normalize a NetVis typed node ID
#'
#' NetVis node IDs are composite: a single-letter type prefix (`H` herb,
#' `C` compound/ingredient, `T` target, `F` formula, `D` disease) plus a
#' colon plus the entity ID. This helper accepts either the display form
#' (`"H:UNITCM_H001"`) or the numeric form (`"H:1"`) and always returns
#' the numeric form expected by the API. Targets and other non-prefixed
#' entities (e.g. `"T:TP53"`) pass through unchanged.
#'
#' @param x A character node ID, e.g. `"H:UNITCM_H001"` or `"H:1"`.
#' @return A character node ID with numeric suffix where applicable.
#' @noRd
normalize_node_id <- function(x) {
  if (is.null(x) || length(x) == 0L || (length(x) == 1L && is.na(x))) {
    rlang::abort("Node ID cannot be NULL, NA, or empty.")
  }
  if (length(x) > 1L) {
    return(vapply(x, normalize_node_id, character(1L), USE.NAMES = FALSE))
  }
  s <- as.character(x)
  parts <- strsplit(s, ":", fixed = TRUE)[[1L]]
  if (length(parts) != 2L) return(s)
  type <- parts[[1L]]
  suffix <- parts[[2L]]
  if (grepl("^UNITCM_", suffix, ignore.case = TRUE)) {
    suffix <- normalize_id(suffix)
  }
  paste0(type, ":", suffix)
}

#' Collapse a character vector for query parameters
#'
#' @param x A character vector or `NULL`.
#' @param sep Separator string (default `","`).
#' @return A single string, or `NULL` if input is `NULL`.
#' @noRd
collapse_param <- function(x, sep = ",") {
  if (is.null(x)) return(NULL)
  paste(x, collapse = sep)
}

test_that("%||% returns non-NULL value", {
  expect_equal(1 %||% 2, 1)
  expect_equal(NULL %||% 2, 2)
  expect_equal(NULL %||% NULL, NULL)
})

test_that("flatten_response converts named list to tibble", {
  x <- list(id = "H001", name = "Ginseng", score = 0.95)
  result <- flatten_response(x)
  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 1L)
  expect_equal(result$id, "H001")
  expect_equal(result$name, "Ginseng")
})

test_that("flatten_response handles nested lists as list-columns", {
  x <- list(id = "H001", aliases = c("a", "b", "c"))
  result <- flatten_response(x)
  expect_s3_class(result, "tbl_df")
  expect_true(is.list(result$aliases))
})

test_that("flatten_response handles NULL values as NA", {
  x <- list(id = "H001", missing = NULL)
  result <- flatten_response(x)
  expect_true(is.na(result$missing))
})

test_that("flatten_response rejects non-named lists", {
  expect_error(flatten_response(list(1, 2, 3)), "named list")
  expect_error(flatten_response("string"), "named list")
})

test_that("collapse_param works correctly", {
  expect_null(collapse_param(NULL))
  expect_equal(collapse_param(c("a", "b"), ","), "a,b")
  expect_equal(collapse_param(c("x", "y"), ";"), "x;y")
  expect_equal(collapse_param("single", ","), "single")
})

test_that("unitcm_cache_clear runs without error", {
  expect_message(unitcm_cache_clear(), "Cache cleared")
})

#' Query gene-to-disease associations (MIDAS)
#'
#' Given a list of gene symbols or IDs, find associated diseases across
#' multiple evidence sources.
#'
#' @param gene_list Character vector of gene identifiers.
#' @param gene_id_type ID type: `"symbol"` (default), `"entrez"`, or
#'   `"ensembl"`.
#' @param sources Character vector of source databases to query (or `NULL`
#'   for all).
#' @param min_sources Minimum number of sources supporting an association
#'   (default 1).
#' @param min_score Minimum association score (default 0).
#' @param evidence_types Character vector of evidence types to filter by.
#' @param scoring_method Scoring method: `"max"` (default) or `"mean"`.
#' @param page Page number (default 1).
#' @param page_size Results per page (default 20).
#' @return A [tibble::tibble()] of gene-disease associations with attribute
#'   `"gene_mapping"` containing the gene ID resolution mapping.
#' @export
#' @examples
#' \dontrun{
#' query_gene_diseases(c("TP53", "BRCA1"))
#' }
query_gene_diseases <- function(gene_list, gene_id_type = "symbol",
                                sources = NULL, min_sources = 1L,
                                min_score = 0, evidence_types = NULL,
                                scoring_method = "max",
                                page = 1L, page_size = 20L) {
  body <- list(
    gene_list = as.list(gene_list),
    gene_id_type = gene_id_type,
    min_sources = min_sources,
    min_score = min_score,
    scoring_method = scoring_method,
    page = page,
    page_size = page_size
  )
  if (!is.null(sources)) body[["sources"]] <- as.list(sources)
  if (!is.null(evidence_types)) body[["evidence_types"]] <- as.list(evidence_types)

  resp <- unitcm_request("/tools/midas/gene-to-disease", method = "POST",
                         body = body)

  items <- resp[["items"]] %||% list()
  result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
  attr(result, "total") <- resp[["total"]] %||% 0L
  attr(result, "gene_mapping") <- resp[["gene_mapping"]]
  result
}

#' Query disease-to-gene associations (MIDAS)
#'
#' Given a disease query, find associated genes across multiple evidence
#' sources.
#'
#' @param disease_query Disease name or ID.
#' @param disease_id_type ID type: `"name"` (default) or `"icd11"`.
#' @param sources Character vector of source databases (or `NULL` for all).
#' @param min_sources Minimum supporting sources (default 1).
#' @param scoring_method Scoring method: `"max"` (default) or `"mean"`.
#' @param page Page number (default 1).
#' @param page_size Results per page (default 20).
#' @return A [tibble::tibble()] of disease-gene associations with attribute
#'   `"matched_diseases"`.
#' @export
#' @examples
#' \dontrun{
#' query_disease_genes("breast cancer")
#' }
query_disease_genes <- function(disease_query, disease_id_type = "name",
                                sources = NULL, min_sources = 1L,
                                scoring_method = "max",
                                page = 1L, page_size = 20L) {
  body <- list(
    disease_query = disease_query,
    disease_id_type = disease_id_type,
    min_sources = min_sources,
    scoring_method = scoring_method,
    page = page,
    page_size = page_size
  )
  if (!is.null(sources)) body[["sources"]] <- as.list(sources)

  resp <- unitcm_request("/tools/midas/disease-to-gene", method = "POST",
                         body = body)

  items <- resp[["items"]] %||% list()
  result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
  attr(result, "total") <- resp[["total"]] %||% 0L
  attr(result, "matched_diseases") <- resp[["matched_diseases"]]
  result
}

#' Convert gene identifiers (MIDAS)
#'
#' Convert a mixed list of gene identifiers (symbols, Entrez IDs, Ensembl
#' IDs) to a standardized mapping.
#'
#' @param identifiers Character vector of gene identifiers.
#' @return A [tibble::tibble()] with columns including match status.
#' @export
#' @examples
#' \dontrun{
#' convert_gene_ids(c("TP53", "7157", "ENSG00000141510"))
#' }
convert_gene_ids <- function(identifiers) {
  resp <- unitcm_request(
    "/tools/midas/convert-gene-ids",
    method = "POST",
    body = list(identifiers = as.list(identifiers))
  )

  results <- resp[["results"]] %||% list()
  if (length(results) > 0L) tibble::as_tibble(results) else tibble::tibble()
}

#' Disease enrichment analysis (MIDAS)
#'
#' Perform Fisher's exact test enrichment analysis to identify diseases
#' significantly associated with a gene list.
#'
#' @param gene_list Character vector of gene identifiers.
#' @param gene_id_type ID type: `"symbol"` (default), `"entrez"`, or
#'   `"ensembl"`.
#' @param sources Character vector of source databases (or `NULL`).
#' @param min_sources Minimum supporting sources (default 1).
#' @param background_gene_count Background gene count (default 20000).
#' @param p_value_cutoff P-value significance cutoff (default 0.05).
#' @param correction_method P-value correction: `"fdr"` (default),
#'   `"bonferroni"`, `"holm"`, or `"none"`.
#' @param min_hit_count Minimum gene hits per disease (default 2).
#' @param page Page number (default 1).
#' @param page_size Results per page (default 20).
#' @return A [tibble::tibble()] of enrichment results with attributes
#'   `"total_significant"`, `"total_tested"`, and `"input_gene_count"`.
#' @export
#' @examples
#' \dontrun{
#' query_disease_enrichment(c("TP53", "BRCA1", "EGFR", "VEGFA"))
#' }
query_disease_enrichment <- function(gene_list, gene_id_type = "symbol",
                                     sources = NULL, min_sources = 1L,
                                     background_gene_count = 20000L,
                                     p_value_cutoff = 0.05,
                                     correction_method = "fdr",
                                     min_hit_count = 2L,
                                     page = 1L, page_size = 20L) {
  body <- list(
    gene_list = as.list(gene_list),
    gene_id_type = gene_id_type,
    min_sources = min_sources,
    background_gene_count = background_gene_count,
    p_value_cutoff = p_value_cutoff,
    correction_method = correction_method,
    min_hit_count = min_hit_count,
    page = page,
    page_size = page_size
  )
  if (!is.null(sources)) body[["sources"]] <- as.list(sources)

  resp <- unitcm_request("/tools/midas/disease-enrichment", method = "POST",
                         body = body)

  items <- resp[["items"]] %||% list()
  result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
  attr(result, "total_significant") <- resp[["total_significant"]]
  attr(result, "total_tested") <- resp[["total_tested"]]
  attr(result, "input_gene_count") <- resp[["input_gene_count"]]
  result
}

#' Compare gene-disease sources (MIDAS)
#'
#' Compare how different evidence sources cover a gene list, producing
#' Venn-diagram-ready set data.
#'
#' @param gene_list Character vector of gene identifiers.
#' @param sources Character vector of source databases (or `NULL`).
#' @param mode Comparison mode: `"union"` (default) or `"intersection"`.
#' @return A named list with elements: `$mode`, `$sources`, `$sets`
#'   (named list of gene vectors), `$intersections`, `$exclusives`,
#'   `$genes_used`.
#' @export
#' @examples
#' \dontrun{
#' query_source_comparison(c("TP53", "BRCA1"), mode = "union")
#' }
query_source_comparison <- function(gene_list, sources = NULL,
                                    mode = c("union", "intersection")) {
  mode <- rlang::arg_match(mode)
  body <- list(
    gene_list = as.list(gene_list),
    mode = mode
  )
  if (!is.null(sources)) body[["sources"]] <- as.list(sources)

  unitcm_request("/tools/midas/source-comparison", method = "POST",
                 body = body)
}

#' Find disease intersection (MIDAS)
#'
#' Find genes shared across multiple diseases.
#'
#' @param disease_queries Character vector of disease names/IDs.
#' @param sources Character vector of source databases (or `NULL`).
#' @return A named list with elements: `$diseases`, `$per_source`,
#'   `$targets`, `$total_intersection_genes`.
#' @export
#' @examples
#' \dontrun{
#' query_disease_intersection(c("breast cancer", "lung cancer"))
#' }
query_disease_intersection <- function(disease_queries, sources = NULL) {
  body <- list(disease_queries = as.list(disease_queries))
  if (!is.null(sources)) body[["sources"]] <- as.list(sources)

  unitcm_request("/tools/midas/disease-intersection", method = "POST",
                 body = body)
}

#' Autocomplete disease names (MIDAS)
#'
#' Search for disease names with autocomplete. Query must be at least 2
#' characters.
#'
#' @param q Search query (minimum 2 characters).
#' @return A [tibble::tibble()] with columns: `disease_name`, `disease_id`,
#'   `gene_count`.
#' @export
#' @examples
#' \dontrun{
#' autocomplete_disease("breast")
#' }
autocomplete_disease <- function(q) {
  if (nchar(q) < 2L) {
    rlang::abort("Query `q` must be at least 2 characters.")
  }
  resp <- unitcm_request("/tools/midas/autocomplete-disease",
                         query = list(q = q))
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }
  tibble::tibble()
}

#' Get MIDAS data sources
#'
#' List all available gene-disease association databases.
#'
#' @return A [tibble::tibble()] with columns: `key`, `label`, `has_score`,
#'   `weight`, `row_count`.
#' @export
#' @examples
#' \dontrun{
#' fetch_midas_sources()
#' }
fetch_midas_sources <- function() {
  resp <- unitcm_request("/tools/midas/sources")
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }
  tibble::tibble()
}

#' Get MIDAS statistics
#'
#' @return A named list with fields: `total_associations`, `total_genes`,
#'   `total_diseases`, `sources`.
#' @export
#' @examples
#' \dontrun{
#' fetch_midas_stats()
#' }
fetch_midas_stats <- function() {
  unitcm_request("/tools/midas/statistics")
}

test_that("query_gene_diseases returns tibble with gene_mapping attr", {
  local_mocked_bindings(
    unitcm_request = function(path, method, body, ...) {
      list(
        items = data.frame(
          gene_symbol = "TP53",
          disease_name = "Breast cancer",
          score = 0.95,
          stringsAsFactors = FALSE
        ),
        total = 1L,
        gene_mapping = list(TP53 = list(entrez_id = "7157"))
      )
    }
  )

  result <- query_gene_diseases(c("TP53"))
  expect_s3_class(result, "tbl_df")
  expect_equal(attr(result, "total"), 1L)
  expect_type(attr(result, "gene_mapping"), "list")
})

test_that("query_disease_genes returns tibble with matched_diseases", {
  local_mocked_bindings(
    unitcm_request = function(path, method, body, ...) {
      list(
        items = data.frame(
          gene_symbol = "BRCA1",
          score = 0.9,
          stringsAsFactors = FALSE
        ),
        total = 1L,
        matched_diseases = list(list(disease_name = "breast cancer"))
      )
    }
  )

  result <- query_disease_genes("breast cancer")
  expect_s3_class(result, "tbl_df")
  expect_type(attr(result, "matched_diseases"), "list")
})

test_that("convert_gene_ids returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, method, body, ...) {
      list(
        results = data.frame(
          input = c("TP53", "7157"),
          matched_id = c("7157", "7157"),
          status = c("matched", "matched"),
          stringsAsFactors = FALSE
        ),
        total = 2L
      )
    }
  )

  result <- convert_gene_ids(c("TP53", "7157"))
  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 2L)
})

test_that("query_disease_enrichment returns tibble with attrs", {
  local_mocked_bindings(
    unitcm_request = function(path, method, body, ...) {
      list(
        items = data.frame(
          disease_name = "Breast cancer",
          p_value = 0.001,
          stringsAsFactors = FALSE
        ),
        total_significant = 5L,
        total_tested = 100L,
        input_gene_count = 10L
      )
    }
  )

  result <- query_disease_enrichment(c("TP53", "BRCA1"))
  expect_s3_class(result, "tbl_df")
  expect_equal(attr(result, "total_significant"), 5L)
  expect_equal(attr(result, "total_tested"), 100L)
  expect_equal(attr(result, "input_gene_count"), 10L)
})

test_that("autocomplete_disease rejects short queries", {
  expect_error(autocomplete_disease("a"), "at least 2 characters")
})

test_that("autocomplete_disease returns tibble for valid query", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(
        disease_name = "breast cancer",
        disease_id = "D001",
        gene_count = 500L,
        stringsAsFactors = FALSE
      )
    }
  )

  result <- autocomplete_disease("breast")
  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 1L)
})

test_that("query_source_comparison validates mode", {
  expect_error(
    query_source_comparison(c("TP53"), mode = "invalid"),
    "must be one of"
  )
})

test_that("fetch_midas_sources returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(
        key = c("disgenet", "opentargets"),
        label = c("DisGeNET", "OpenTargets"),
        row_count = c(1000000L, 500000L),
        stringsAsFactors = FALSE
      )
    }
  )

  result <- fetch_midas_sources()
  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 2L)
})

test_that("fetch_midas_stats returns a list", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(total_associations = 5000000L, total_genes = 20000L,
           total_diseases = 15000L)
    }
  )

  result <- fetch_midas_stats()
  expect_type(result, "list")
  expect_equal(result$total_associations, 5000000L)
})

#' Plot a network graph
#'
#' Visualize an `igraph` or `tbl_graph` object using `ggraph`.
#'
#' @param graph An `igraph` or `tidygraph::tbl_graph` object.
#' @param layout Layout algorithm (default `"fr"` for Fruchterman-Reingold).
#'   See [ggraph::create_layout()] for options.
#' @param color_by Vertex attribute name to map to node color (default
#'   `"type"`).
#' @param node_size Base node size (default 3).
#' @param edge_alpha Edge transparency (default 0.3).
#' @param show_labels Whether to label nodes (default `TRUE` for nodes
#'   with degree >= 3, or all if graph has <= 50 nodes).
#' @return A `ggplot` object.
#' @export
#' @examples
#' \dontrun{
#' g <- build_hct_network("UNITCM_H001")
#' plot_network(g)
#' }
plot_network <- function(graph, layout = "fr", color_by = "type",
                         node_size = 3, edge_alpha = 0.3,
                         show_labels = NULL) {
  check_pkg("ggraph", reason = "to plot networks")
  check_pkg("ggplot2", reason = "to plot networks")

  n <- igraph::vcount(graph)
  if (is.null(show_labels)) {
    show_labels <- n <= 50L
  }

  p <- ggraph::ggraph(graph, layout = layout) +
    ggraph::geom_edge_link(alpha = edge_alpha) +
    ggplot2::theme_void()

  if (color_by %in% igraph::vertex_attr_names(graph)) {
    p <- p + ggraph::geom_node_point(
      ggplot2::aes(color = .data[[color_by]]),
      size = node_size
    )
  } else {
    p <- p + ggraph::geom_node_point(size = node_size)
  }

  if (isTRUE(show_labels)) {
    label_attr <- if ("label" %in% igraph::vertex_attr_names(graph)) {
      "label"
    } else {
      "name"
    }
    p <- p + ggraph::geom_node_text(
      ggplot2::aes(label = .data[[label_attr]]),
      repel = TRUE, size = 2.5
    )
  }

  p
}

#' Plot enrichment results
#'
#' Create a dot plot or bar plot from enrichment analysis results
#' (e.g. from [query_disease_enrichment()]).
#'
#' @param enrichment_result A [tibble::tibble()] from an enrichment
#'   analysis. Must contain columns mappable to term name and p-value.
#' @param type Plot type: `"dotplot"` (default) or `"barplot"`.
#' @param top_n Number of top terms to show (default 20).
#' @param name_col Column name for term labels (auto-detected if `NULL`).
#' @param pvalue_col Column name for p-values (auto-detected if `NULL`).
#' @param count_col Column name for gene counts (auto-detected if `NULL`).
#' @return A `ggplot` object.
#' @export
#' @examples
#' \dontrun{
#' enrich <- query_disease_enrichment(c("TP53", "BRCA1", "EGFR"))
#' plot_enrichment(enrich)
#' }
plot_enrichment <- function(enrichment_result,
                            type = c("dotplot", "barplot"),
                            top_n = 20L,
                            name_col = NULL, pvalue_col = NULL,
                            count_col = NULL) {
  check_pkg("ggplot2", reason = "to plot enrichment results")
  type <- rlang::arg_match(type)

  cols <- names(enrichment_result)

  if (is.null(name_col)) {
    name_col <- intersect(
      c("disease_name", "term_name", "name", "description", "pathway"),
      cols
    )[1L]
  }
  if (is.null(pvalue_col)) {
    pvalue_col <- intersect(
      c("p_value_adjusted", "p_adjusted", "padj", "fdr", "p_value", "pvalue"),
      cols
    )[1L]
  }
  if (is.null(count_col)) {
    count_col <- intersect(
      c("hit_count", "gene_count", "count", "n_genes", "hits"),
      cols
    )[1L]
  }

  if (is.na(name_col) || is.na(pvalue_col)) {
    rlang::abort("Could not auto-detect column names. Specify `name_col` and `pvalue_col`.")
  }

  df <- utils::head(enrichment_result, top_n)
  df[["..name.."]] <- df[[name_col]]
  df[["..pval.."]] <- df[[pvalue_col]]
  df[["..neg_log_p.."]] <- -log10(df[["..pval.."]])

  df[["..name.."]] <- factor(df[["..name.."]],
    levels = rev(df[["..name.."]]))

  if (type == "barplot") {
    p <- ggplot2::ggplot(df, ggplot2::aes(x = .data[["..neg_log_p.."]],
                                          y = .data[["..name.."]])) +
      ggplot2::geom_col(fill = "#3498db") +
      ggplot2::labs(x = expression(-log[10](p)), y = NULL)
  } else {
    if (!is.na(count_col) && count_col %in% names(df)) {
      df[["..count.."]] <- df[[count_col]]
      p <- ggplot2::ggplot(df, ggplot2::aes(
        x = .data[["..neg_log_p.."]],
        y = .data[["..name.."]],
        size = .data[["..count.."]],
        color = .data[["..pval.."]]
      )) +
        ggplot2::geom_point() +
        ggplot2::scale_color_gradient(low = "red", high = "blue",
          name = "p-value") +
        ggplot2::labs(x = expression(-log[10](p)), y = NULL, size = "Count")
    } else {
      p <- ggplot2::ggplot(df, ggplot2::aes(
        x = .data[["..neg_log_p.."]],
        y = .data[["..name.."]],
        color = .data[["..pval.."]]
      )) +
        ggplot2::geom_point(size = 4) +
        ggplot2::scale_color_gradient(low = "red", high = "blue",
          name = "p-value") +
        ggplot2::labs(x = expression(-log[10](p)), y = NULL)
    }
  }

  p + ggplot2::theme_minimal()
}

#' Plot compound physicochemical radar chart
#'
#' Create a radar/spider chart of normalized physicochemical properties
#' for a compound.
#'
#' @param compound_id The UniTCM ingredient ID.
#' @param properties Character vector of property names to plot (default:
#'   key drug-likeness properties).
#' @return A `ggplot` object.
#' @export
#' @examples
#' \dontrun{
#' plot_compound_radar("UNITCM_I00001")
#' }
plot_compound_radar <- function(compound_id,
                                properties = c("mw", "clogp", "tpsa",
                                               "hbd", "hba", "qed_score")) {
  check_pkg("ggplot2", reason = "to plot radar charts")

  compound <- get_compound(compound_id)

  ref_max <- c(mw = 500, clogp = 5, tpsa = 140, hbd = 5, hba = 10,
               rotatable_bonds = 10, ring_count = 6, aromatic_ring_count = 4,
               fsp3 = 1, qed_score = 1, sa_score = 10,
               np_likeness_score = 5, lipinski_violations = 5)

  vals <- vapply(properties, function(p) {
    v <- compound[[p]]
    if (is.null(v) || is.na(v)) return(0)
    mx <- ref_max[p]
    if (is.na(mx)) mx <- max(abs(v), 1)
    min(abs(as.numeric(v)) / mx, 1)
  }, numeric(1L))

  n <- length(properties)
  angles <- seq(0, 2 * pi, length.out = n + 1L)[-(n + 1L)]

  df <- data.frame(
    property = properties,
    value = vals,
    x = vals * cos(angles),
    y = vals * sin(angles),
    stringsAsFactors = FALSE
  )
  df_closed <- rbind(df, df[1L, ])

  ggplot2::ggplot(df_closed, ggplot2::aes(x = .data[["x"]],
                                           y = .data[["y"]])) +
    ggplot2::geom_polygon(fill = "#3498db", alpha = 0.3) +
    ggplot2::geom_path(color = "#2c3e50") +
    ggplot2::geom_point(data = df, color = "#e74c3c", size = 3) +
    ggplot2::geom_text(data = df,
      ggplot2::aes(
        x = 1.15 * cos(angles),
        y = 1.15 * sin(angles),
        label = .data[["property"]]
      ), size = 3) +
    ggplot2::coord_fixed() +
    ggplot2::theme_void() +
    ggplot2::labs(title = paste("Compound:", compound_id))
}

#' Search transcriptome datasets
#'
#' Query the TCM Transcriptome Hub. This endpoint uses Style-B pagination
#' (`count`/`results` instead of `total`/`items`).
#'
#' @param search Optional text search query.
#' @param tcm_classification TCM classification filter.
#' @param organism Organism filter.
#' @param model_type Model type filter.
#' @param experiment_type Experiment type filter.
#' @param disease_classification Disease classification filter.
#' @param cell_line Cell line filter.
#' @param comparison_type Comparison type filter.
#' @param confidence Confidence filter.
#' @param sequence_type Sequence type filter.
#' @param page Page number (default 1).
#' @param page_size Results per page (default 20).
#' @param all_pages If `TRUE`, fetch all pages via auto-pagination.
#' @return A [tibble::tibble()] of datasets with attribute `"total"`.
#' @export
#' @examples
#' \dontrun{
#' search_transcriptomes(search = "ginseng")
#' }
search_transcriptomes <- function(search = NULL, tcm_classification = NULL,
                                  organism = NULL, model_type = NULL,
                                  experiment_type = NULL,
                                  disease_classification = NULL,
                                  cell_line = NULL, comparison_type = NULL,
                                  confidence = NULL, sequence_type = NULL,
                                  page = 1L, page_size = 20L,
                                  all_pages = FALSE) {
  query <- list(
    search = search,
    tcm_classification = tcm_classification,
    organism = organism,
    model_type = model_type,
    experiment_type = experiment_type,
    disease_classification = disease_classification,
    cell_line = cell_line,
    comparison_type = comparison_type,
    confidence = confidence,
    sequence_type = sequence_type
  )

  if (isTRUE(all_pages)) {
    return(unitcm_paginate("/transcriptome-hub/", query = query))
  }

  query[["page"]] <- page
  query[["page_size"]] <- page_size
  resp <- unitcm_request("/transcriptome-hub/", query = query)

  items <- resp[["results"]] %||% resp[["items"]] %||% list()
  result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
  attr(result, "total") <- resp[["count"]] %||% resp[["total"]] %||% 0L
  result
}

#' Get a single transcriptome dataset
#'
#' @param dataset_id The dataset ID (e.g. `"TCMtrans00001"`).
#' @return A named list with 35+ fields.
#' @export
#' @examples
#' \dontrun{
#' get_transcriptome("TCMtrans00001")
#' }
get_transcriptome <- function(dataset_id) {
  unitcm_request(paste0("/transcriptome-hub/", dataset_id, "/"))
}

#' Get transcriptome filter options
#'
#' @return A named list of character vectors for each filter field.
#' @export
#' @examples
#' \dontrun{
#' filters <- fetch_transcriptome_filters()
#' filters$organism
#' }
fetch_transcriptome_filters <- function() {
  unitcm_request("/transcriptome-hub/filter-options/")
}

#' Get Transcriptome Hub statistics
#'
#' @return A named list with fields: `total_datasets`, `total_organisms`,
#'   `total_tcm_entities`, `total_analysis_modules`, plus distribution data.
#' @export
#' @examples
#' \dontrun{
#' fetch_transcriptome_stats()
#' }
fetch_transcriptome_stats <- function() {
  unitcm_request("/transcriptome-hub/statistics/")
}

#' List available analysis modules for a dataset
#'
#' @param dataset_id The dataset ID.
#' @return A character vector of available module names.
#' @export
#' @examples
#' \dontrun{
#' get_analysis_modules("TCMtrans00001")
#' }
get_analysis_modules <- function(dataset_id) {
  resp <- unitcm_request(paste0("/transcriptome-hub/", dataset_id, "/analysis/"))
  resp[["available_modules"]] %||% character(0L)
}

#' Get analysis data for a transcriptome dataset
#'
#' Retrieve data for a specific analysis module. Return type varies by
#' module.
#'
#' @param dataset_id The dataset ID.
#' @param module Analysis module name. One of: `"meta"`, `"expression"`,
#'   `"deg"`, `"go"`, `"kegg"`, `"gsea"`, `"ppi"`, `"immune"`, `"tf"`,
#'   `"pca"`, `"qc"`.
#' @param gene Optional gene filter (for expression module only).
#' @return A [tibble::tibble()] for tabular modules (deg, go, kegg, gsea,
#'   immune, tf), or a named list for structured modules (meta, expression,
#'   ppi, pca, qc).
#' @export
#' @examples
#' \dontrun{
#' get_analysis_data("TCMtrans00001", "deg")
#' get_analysis_data("TCMtrans00001", "expression", gene = "TP53")
#' }
get_analysis_data <- function(dataset_id, module, gene = NULL) {
  valid_modules <- c("meta", "expression", "deg", "go", "kegg", "gsea",
                     "ppi", "immune", "tf", "pca", "qc")
  module <- rlang::arg_match(module, values = valid_modules)

  query <- list(gene = gene)
  resp <- unitcm_request(
    paste0("/transcriptome-hub/", dataset_id, "/analysis/", module, "/"),
    query = query
  )

  tabular_modules <- c("deg", "go", "kegg", "gsea", "immune", "tf")
  if (module %in% tabular_modules) {
    if (is.data.frame(resp)) return(tibble::as_tibble(resp))
    if (is.list(resp) && length(resp) > 0L) {
      return(tryCatch(tibble::as_tibble(resp), error = function(e) resp))
    }
    return(tibble::tibble())
  }

  resp
}

test_that("search_transcriptomes handles Style B pagination", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(
        results = data.frame(id = "TCMtrans00001",
                             stringsAsFactors = FALSE),
        count = 1L
      )
    }
  )
  result <- search_transcriptomes(search = "ginseng")
  expect_s3_class(result, "tbl_df")
  expect_equal(attr(result, "total"), 1L)
})

test_that("get_transcriptome returns a list", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(id = "TCMtrans00001", organism = "Homo sapiens")
    }
  )
  result <- get_transcriptome("TCMtrans00001")
  expect_type(result, "list")
})

test_that("get_analysis_modules returns character vector", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(available_modules = c("deg", "go", "kegg"), total = 3L)
    }
  )
  result <- get_analysis_modules("TCMtrans00001")
  expect_type(result, "character")
  expect_equal(length(result), 3L)
})

test_that("get_analysis_data validates module arg", {
  expect_error(get_analysis_data("TCMtrans00001", "invalid"), "must be one of")
})

test_that("get_analysis_data returns tibble for tabular modules", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(gene = "TP53", log2FC = 2.5, padj = 0.01,
                 stringsAsFactors = FALSE)
    }
  )
  result <- get_analysis_data("TCMtrans00001", "deg")
  expect_s3_class(result, "tbl_df")
})

test_that("fetch_transcriptome_stats returns a list", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(total_datasets = 100L, total_organisms = 5L)
    }
  )
  result <- fetch_transcriptome_stats()
  expect_type(result, "list")
})

#' Search formulas in the Disease-Formula Atlas
#'
#' Query the Disease-Formula Atlas with optional text search and ICD-11
#' disease classification filters. Multi-value parameters accept character
#' vectors and are collapsed to comma-separated strings internally.
#'
#' @param q Optional search query string.
#' @param level1,level2,level3,level4 ICD-11 disease classification levels.
#' @param book_sources Book source filter (character vector).
#' @param origin_sources Origin source filter (character vector).
#' @param dosage_forms Dosage form filter (character vector).
#' @param mapping_confidence Mapping confidence filter (character vector).
#' @param page Page number (default 1).
#' @param page_size Results per page (default 20, max 100).
#' @param all_pages If `TRUE`, fetch all pages via auto-pagination.
#' @return A [tibble::tibble()] of formulas with attribute `"total"`.
#' @export
#' @examples
#' \dontrun{
#' search_formulas(q = "insomnia")
#' search_formulas(level1 = "Neoplasms", mapping_confidence = "high")
#' }
search_formulas <- function(q = NULL, level1 = NULL, level2 = NULL,
                            level3 = NULL, level4 = NULL,
                            book_sources = NULL, origin_sources = NULL,
                            dosage_forms = NULL, mapping_confidence = NULL,
                            page = 1L, page_size = 20L,
                            all_pages = FALSE) {
  query <- list(
    q = q,
    level1 = level1,
    level2 = level2,
    level3 = level3,
    level4 = level4,
    book_sources = collapse_param(book_sources, ","),
    origin_sources = collapse_param(origin_sources, ","),
    dosage_forms = collapse_param(dosage_forms, ","),
    mapping_confidence = collapse_param(mapping_confidence, ",")
  )

  if (isTRUE(all_pages)) {
    return(unitcm_paginate("/disease-formula", query = query))
  }

  query[["page"]] <- page
  query[["page_size"]] <- page_size
  resp <- unitcm_request("/disease-formula", query = query)

  items <- resp[["items"]] %||% list()
  result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
  attr(result, "total") <- resp[["total"]] %||% 0L
  result
}

#' Get a single formula by order ID
#'
#' Retrieve full detail for one formula from the Disease-Formula Atlas.
#'
#' @param order_id The formula order ID (integer or character).
#' @return A named list with 30+ fields.
#' @export
#' @examples
#' \dontrun{
#' get_formula(1)
#' }
get_formula <- function(order_id) {
  unitcm_request(paste0("/disease-formula/", order_id))
}

#' Get herb doses for a formula
#'
#' Retrieve the composition and dosage information for a specific formula.
#'
#' @param order_id The formula order ID.
#' @return A [tibble::tibble()] with columns: `id`, `herb_name`,
#'   `original_dose`, `composition_ratio`, `modern_dose_g`,
#'   `clinical_ref_dose_g`, `dynasty`, `notes`, `herb_ids`.
#' @export
#' @examples
#' \dontrun{
#' get_formula_doses(1)
#' }
get_formula_doses <- function(order_id) {
  resp <- unitcm_request(paste0("/disease-formula/", order_id, "/doses"))
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }
  tibble::tibble()
}

#' Get the ICD-11 disease classification tree
#'
#' Returns the full 4-level ICD-11 disease classification tree used by the
#' Disease-Formula Atlas.
#'
#' @return A recursive nested list with structure
#'   `list(label, count, children = list(...))`.
#' @export
#' @examples
#' \dontrun{
#' tree <- fetch_disease_tree()
#' names(tree[[1]])
#' }
fetch_disease_tree <- function() {
  unitcm_request("/disease-formula/disease-tree")
}

#' List book sources
#'
#' @return A [tibble::tibble()] with columns `value`, `label`, `count`.
#' @export
#' @examples
#' \dontrun{
#' list_book_sources()
#' }
list_book_sources <- function() {
  resp <- unitcm_request("/disease-formula/book-sources")
  tibble::as_tibble(resp)
}

#' List origin sources
#'
#' Returns the top 50 formula origin sources by frequency.
#'
#' @return A [tibble::tibble()] with columns `value`, `label`, `count`.
#' @export
#' @examples
#' \dontrun{
#' list_origin_sources()
#' }
list_origin_sources <- function() {
  resp <- unitcm_request("/disease-formula/origin-sources")
  tibble::as_tibble(resp)
}

#' List dosage forms
#'
#' Returns the top 50 dosage forms by frequency.
#'
#' @return A [tibble::tibble()] with columns `value`, `label`, `count`.
#' @export
#' @examples
#' \dontrun{
#' list_dosage_forms()
#' }
list_dosage_forms <- function() {
  resp <- unitcm_request("/disease-formula/dosage-forms")
  tibble::as_tibble(resp)
}

test_that("search_formulas returns tibble with total", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(
        items = data.frame(
          order = 1L,
          formula_name = "Liu Wei Di Huang Wan",
          disease_name = "Insomnia",
          stringsAsFactors = FALSE
        ),
        total = 200L
      )
    }
  )

  result <- search_formulas(q = "insomnia")
  expect_s3_class(result, "tbl_df")
  expect_equal(attr(result, "total"), 200L)
})

test_that("search_formulas collapses comma-sep params", {
  captured_query <- NULL
  local_mocked_bindings(
    unitcm_request = function(path, query = list(), ...) {
      captured_query <<- query
      list(items = list(), total = 0L)
    }
  )

  search_formulas(book_sources = c("A", "B"), mapping_confidence = c("high", "medium"))
  expect_equal(captured_query$book_sources, "A,B")
  expect_equal(captured_query$mapping_confidence, "high,medium")
})

test_that("get_formula returns a list", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(order = 1L, formula_name = "Test Formula", efficacy = "Tonify Qi")
    }
  )

  result <- get_formula(1)
  expect_type(result, "list")
  expect_equal(result$formula_name, "Test Formula")
})

test_that("get_formula_doses returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(
        herb_name = c("Ren Shen", "Huang Qi"),
        original_dose = c("10g", "15g"),
        stringsAsFactors = FALSE
      )
    }
  )

  result <- get_formula_doses(1)
  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 2L)
})

test_that("fetch_disease_tree returns nested list", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(
        list(label = "Neoplasms", count = 100L, children = list(
          list(label = "Breast cancer", count = 20L, children = list())
        ))
      )
    }
  )

  result <- fetch_disease_tree()
  expect_type(result, "list")
  expect_equal(result[[1]]$label, "Neoplasms")
})

test_that("list_book_sources returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(value = "Ben Cao", label = "Ben Cao", count = 50L,
                 stringsAsFactors = FALSE)
    }
  )

  result <- list_book_sources()
  expect_s3_class(result, "tbl_df")
})

#' Search TCMomics datasets
#'
#' Query the TCMomics multi-omics database with optional text search and
#' faceted filters.
#'
#' @param q Optional search query string.
#' @param tcm TCM classification filter.
#' @param omics Omics type filter.
#' @param source Source type filter.
#' @param organism Organism filter.
#' @param tissue Tissue filter.
#' @param disease Disease filter.
#' @param repo Repository filter.
#' @param year_min Minimum publication year.
#' @param year_max Maximum publication year.
#' @param sort Sort field: `"relevance"`, `"date_desc"`, `"views_desc"`, or
#'   `"downloads_desc"`.
#' @param search_mode Search mode: `"fuzzy"` (default) or `"exact"`.
#' @param page Page number (default 1).
#' @param page_size Results per page (default 20).
#' @param all_pages If `TRUE`, fetch all pages via auto-pagination.
#' @return A [tibble::tibble()] of datasets with attribute `"total"`.
#' @export
#' @examples
#' \dontrun{
#' search_datasets(q = "ginseng", omics = "Transcriptomics")
#' }
search_datasets <- function(q = NULL, tcm = NULL, omics = NULL,
                            source = NULL, organism = NULL,
                            tissue = NULL, disease = NULL,
                            repo = NULL, year_min = NULL, year_max = NULL,
                            sort = NULL, search_mode = NULL,
                            page = 1L, page_size = 20L,
                            all_pages = FALSE) {
  query <- list(
    q = q, tcm = tcm, omics = omics, source = source,
    organism = organism, tissue = tissue, disease = disease,
    repo = repo, year_min = year_min, year_max = year_max,
    sort = sort, search_mode = search_mode
  )

  path <- if (!is.null(search_mode)) "/browse/general_search" else "/browse"

  if (isTRUE(all_pages)) {
    return(unitcm_paginate(path, query = query))
  }

  query[["page"]] <- page
  query[["page_size"]] <- page_size
  resp <- unitcm_request(path, query = query)

  items <- resp[["items"]] %||% list()
  result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
  attr(result, "total") <- resp[["total"]] %||% 0L
  result
}

#' Get a single dataset by submission ID
#'
#' Retrieve full detail including nested persons, publications, grants,
#' and data files.
#'
#' @param submission_id The submission ID (e.g. `"TMA2025001"`).
#' @return A named list with nested sub-lists for `persons`, `publications`,
#'   `grants`, and `data_files`.
#' @export
#' @examples
#' \dontrun{
#' get_dataset("TMA2025001")
#' }
get_dataset <- function(submission_id) {
  unitcm_request(paste0("/browse/detailPage/", submission_id))
}

#' Get similar datasets
#'
#' Find datasets similar to a given submission based on content similarity.
#'
#' @param submission_id The submission ID.
#' @return A [tibble::tibble()] with columns: `submission_id`,
#'   `project_title`, `TCM_classification`, `similarity_score`.
#' @export
#' @examples
#' \dontrun{
#' get_similar_datasets("TMA2025001")
#' }
get_similar_datasets <- function(submission_id) {
  resp <- unitcm_request(paste0("/browse/similar-datasets/", submission_id))
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }
  tibble::tibble()
}

#' Get dataset facets
#'
#' Returns available filter values and their counts for the TCMomics
#' database.
#'
#' @return A named list of tibbles for each facet field.
#' @export
#' @examples
#' \dontrun{
#' facets <- fetch_dataset_facets()
#' facets$omics_type
#' }
fetch_dataset_facets <- function() {
  resp <- unitcm_request("/browse/facets")
  lapply(resp, function(x) {
    if (is.data.frame(x)) {
      tibble::as_tibble(x)
    } else if (is.list(x) && length(x) > 0L) {
      tryCatch(tibble::as_tibble(x), error = function(e) x)
    } else {
      x
    }
  })
}

#' Get TCMomics database statistics
#'
#' @return A named list with fields: `total_datasets`, `total_downloads`,
#'   `omics_types_count`, `unique_organisms`.
#' @export
#' @examples
#' \dontrun{
#' fetch_dataset_stats()
#' }
fetch_dataset_stats <- function() {
  unitcm_request("/browse/stats")
}

#' Export datasets to CSV
#'
#' Download a CSV export of datasets matching the given filters.
#'
#' @inheritParams search_datasets
#' @param file Output file path (default `"datasets_export.csv"`).
#' @return Invisible file path.
#' @export
#' @examples
#' \dontrun{
#' export_datasets(omics = "Transcriptomics", file = "transcriptomics.csv")
#' }
export_datasets <- function(q = NULL, tcm = NULL, omics = NULL,
                            source = NULL, organism = NULL,
                            tissue = NULL, disease = NULL,
                            repo = NULL, year_min = NULL, year_max = NULL,
                            sort = NULL, file = "datasets_export.csv") {
  query <- list(
    q = q, tcm = tcm, omics = omics, source = source,
    organism = organism, tissue = tissue, disease = disease,
    repo = repo, year_min = year_min, year_max = year_max,
    sort = sort
  )
  unitcm_download("/browse/export", query = query, file = file)
}

test_that("search_datasets returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(
        items = data.frame(submission_id = "TMA2025001",
                           project_title = "Test",
                           stringsAsFactors = FALSE),
        total = 1L
      )
    }
  )
  result <- search_datasets(q = "ginseng")
  expect_s3_class(result, "tbl_df")
  expect_equal(attr(result, "total"), 1L)
})

test_that("search_datasets uses general_search when search_mode set", {
  captured_path <- NULL
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      captured_path <<- path
      list(items = list(), total = 0L)
    }
  )
  search_datasets(q = "test", search_mode = "exact")
  expect_equal(captured_path, "/browse/general_search")
})

test_that("get_dataset returns a list", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(submission_id = "TMA2025001", persons = list(), publications = list())
    }
  )
  result <- get_dataset("TMA2025001")
  expect_type(result, "list")
})

test_that("get_similar_datasets returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(submission_id = "TMA2025002",
                 similarity_score = 0.85,
                 stringsAsFactors = FALSE)
    }
  )
  result <- get_similar_datasets("TMA2025001")
  expect_s3_class(result, "tbl_df")
})

test_that("fetch_dataset_stats returns a list", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(total_datasets = 50L, total_downloads = 1000L)
    }
  )
  result <- fetch_dataset_stats()
  expect_type(result, "list")
})

#' Search compounds in the Ingredient Explorer
#'
#' Query the UniTCM Ingredient Explorer with optional text search and
#' physicochemical property filters.
#'
#' @param q Search query (name, SMILES, formula, or CAS number).
#' @param mw_min,mw_max Molecular weight range.
#' @param clogp_min,clogp_max CLogP range.
#' @param tpsa_min,tpsa_max Topological polar surface area range.
#' @param qed_min,qed_max QED score range.
#' @param ring_count_min,ring_count_max Ring count range.
#' @param lipinski Lipinski rule filter (character vector, comma-collapsed).
#' @param is_drug Approved drug filter (logical or `NULL`).
#' @param sort Sort field (e.g. `"mw"`, `"-mw"`).
#' @param page Page number (default 1).
#' @param page_size Results per page (default 20, max 200).
#' @param all_pages If `TRUE`, fetch all pages.
#' @return A [tibble::tibble()] of compounds with attribute `"total"`.
#' @export
#' @examples
#' \dontrun{
#' search_compounds(q = "quercetin")
#' search_compounds(mw_min = 200, mw_max = 500, lipinski = "pass")
#' }
search_compounds <- function(q = NULL, mw_min = NULL, mw_max = NULL,
                             clogp_min = NULL, clogp_max = NULL,
                             tpsa_min = NULL, tpsa_max = NULL,
                             qed_min = NULL, qed_max = NULL,
                             ring_count_min = NULL, ring_count_max = NULL,
                             lipinski = NULL, is_drug = NULL,
                             sort = NULL, page = 1L, page_size = 20L,
                             all_pages = FALSE) {
  query <- list(
    q = q,
    mw_min = mw_min, mw_max = mw_max,
    clogp_min = clogp_min, clogp_max = clogp_max,
    tpsa_min = tpsa_min, tpsa_max = tpsa_max,
    qed_min = qed_min, qed_max = qed_max,
    ring_count_min = ring_count_min, ring_count_max = ring_count_max,
    lipinski = collapse_param(lipinski, ","),
    is_drug = is_drug,
    sort = sort
  )

  if (isTRUE(all_pages)) {
    return(unitcm_paginate("/ingredient-explorer", query = query))
  }

  query[["page"]] <- page
  query[["page_size"]] <- page_size
  resp <- unitcm_request("/ingredient-explorer", query = query)

  items <- resp[["items"]] %||% list()
  result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
  attr(result, "total") <- resp[["total"]] %||% 0L
  result
}

#' Get a single compound by ID
#'
#' Retrieve full detail for one compound including cross-references.
#'
#' @param id The UniTCM ingredient ID (e.g. `"UNITCM_I00001"`).
#' @return A named list with 26+ fields including an `xref` sub-list.
#' @export
#' @examples
#' \dontrun{
#' get_compound("UNITCM_I00001")
#' }
get_compound <- function(id) {

  unitcm_request(paste0("/ingredient-explorer/", normalize_id(id)))
}

#' Get ADMET predictions for a compound
#'
#' Returns ~90 ADMET endpoint predictions as a single-row wide tibble.
#'
#' @param id The UniTCM ingredient ID.
#' @return A single-row [tibble::tibble()] with ~90 ADMET columns.
#' @export
#' @examples
#' \dontrun{
#' get_compound_admet("UNITCM_I00001")
#' }
get_compound_admet <- function(id) {
  resp <- unitcm_request(paste0("/ingredient-explorer/", normalize_id(id), "/admet"))
  flatten_response(resp)
}

#' Get predicted targets for a compound
#'
#' Retrieve target predictions from DrugCLIP, ChEMBL similarity search,
#' or both.
#'
#' @param id The UniTCM ingredient ID.
#' @param method One of `"drugclip"`, `"chembl"`, or `"both"` (default
#'   `"drugclip"`).
#' @param page Page number (for ChEMBL targets, default 1).
#' @param page_size Results per page (for ChEMBL targets, default 20).
#' @return A [tibble::tibble()] of targets. When `method = "both"`, a
#'   `source` column is added to distinguish results.
#' @export
#' @examples
#' \dontrun{
#' get_compound_targets("UNITCM_I00001")
#' get_compound_targets("UNITCM_I00001", method = "both")
#' }
get_compound_targets <- function(id, method = c("drugclip", "chembl", "both"),
                                 page = 1L, page_size = 20L) {
  method <- rlang::arg_match(method)
  base_path <- paste0("/ingredient-explorer/", normalize_id(id))

  if (method == "drugclip") {
    resp <- unitcm_request(paste0(base_path, "/targets"))
    if (is.data.frame(resp)) return(tibble::as_tibble(resp))
    if (is.list(resp) && length(resp) > 0L) {
      return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
    }
    return(tibble::tibble())
  }

  if (method == "chembl") {
    resp <- unitcm_request(
      paste0(base_path, "/chembl-targets"),
      query = list(page = page, page_size = page_size)
    )
    items <- resp[["items"]] %||% list()
    result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
    attr(result, "total") <- resp[["total"]] %||% 0L
    return(result)
  }

  # method == "both"
  dc <- get_compound_targets(id, method = "drugclip")
  ch <- get_compound_targets(id, method = "chembl", page = page,
                             page_size = page_size)
  if (nrow(dc) > 0L) dc[["source"]] <- "drugclip"
  if (nrow(ch) > 0L) ch[["source"]] <- "chembl"
  dplyr::bind_rows(dc, ch)
}

#' Get herbs containing a compound
#'
#' List herbs that contain a specific compound.
#'
#' @param id The UniTCM ingredient ID.
#' @param page Page number (default 1).
#' @param page_size Results per page (default 20).
#' @param all_pages If `TRUE`, fetch all pages.
#' @return A [tibble::tibble()] of herbs with attribute `"total"`.
#' @export
#' @examples
#' \dontrun{
#' get_compound_herbs("UNITCM_I00001")
#' }
get_compound_herbs <- function(id, page = 1L, page_size = 20L,
                               all_pages = FALSE) {
  path <- paste0("/ingredient-explorer/", normalize_id(id), "/herbs")

  if (isTRUE(all_pages)) {
    return(unitcm_paginate(path))
  }

  resp <- unitcm_request(path, query = list(page = page, page_size = page_size))
  items <- resp[["items"]] %||% list()
  result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
  attr(result, "total") <- resp[["total"]] %||% 0L
  result
}

#' Get compound facets and statistics
#'
#' Returns summary statistics and filter option counts for the Ingredient
#' Explorer.
#'
#' @return A named list with fields: `total`, `approved_count`,
#'   `lipinski_counts`, `drug_counts`, `mw_range`, `clogp_range`,
#'   `tpsa_range`, `qed_range`.
#' @export
#' @examples
#' \dontrun{
#' fetch_compound_facets()
#' }
fetch_compound_facets <- function() {
  unitcm_request("/ingredient-explorer/facets")
}

#' Export compounds to CSV
#'
#' Download a CSV export of compounds matching the given filters
#' (max 10,000 rows).
#'
#' @inheritParams search_compounds
#' @param file Output file path (default `"compounds_export.csv"`).
#' @return Invisible file path.
#' @export
#' @examples
#' \dontrun{
#' export_compounds(mw_min = 200, file = "filtered_compounds.csv")
#' }
export_compounds <- function(q = NULL, mw_min = NULL, mw_max = NULL,
                             clogp_min = NULL, clogp_max = NULL,
                             tpsa_min = NULL, tpsa_max = NULL,
                             qed_min = NULL, qed_max = NULL,
                             ring_count_min = NULL, ring_count_max = NULL,
                             lipinski = NULL, is_drug = NULL,
                             sort = NULL, file = "compounds_export.csv") {
  query <- list(
    q = q,
    mw_min = mw_min, mw_max = mw_max,
    clogp_min = clogp_min, clogp_max = clogp_max,
    tpsa_min = tpsa_min, tpsa_max = tpsa_max,
    qed_min = qed_min, qed_max = qed_max,
    ring_count_min = ring_count_min, ring_count_max = ring_count_max,
    lipinski = collapse_param(lipinski, ","),
    is_drug = is_drug,
    sort = sort
  )
  unitcm_download("/ingredient-explorer/export", query = query, file = file)
}

#' Export compound data by module
#'
#' Download a CSV of a specific data module for one compound.
#'
#' @param id The UniTCM ingredient ID.
#' @param module One of `"overview"`, `"physicochemical"`, `"admet"`,
#'   or `"targets"`.
#' @param file Output file path (auto-generated if `NULL`).
#' @return Invisible file path.
#' @export
#' @examples
#' \dontrun{
#' export_compound_module("UNITCM_I00001", "admet")
#' }
export_compound_module <- function(id,
                                   module = c("overview", "physicochemical",
                                              "admet", "targets"),
                                   file = NULL) {
  module <- rlang::arg_match(module)
  file <- file %||% paste0(id, "_", module, ".csv")
  unitcm_download(
    paste0("/ingredient-explorer/", normalize_id(id), "/export/", module),
    file = file
  )
}

test_that("search_compounds returns tibble with total", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(
        items = data.frame(
          unitcm_ingredient_id = "I001",
          component_name = "Quercetin",
          mw = 302.24,
          stringsAsFactors = FALSE
        ),
        total = 50L
      )
    }
  )

  result <- search_compounds(q = "quercetin")
  expect_s3_class(result, "tbl_df")
  expect_equal(attr(result, "total"), 50L)
})

test_that("get_compound returns a list with xref", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(
        unitcm_ingredient_id = "I001",
        component_name = "Quercetin",
        xref = list(pubchem_cid = "5280343", cas_number = "117-39-5")
      )
    }
  )

  result <- get_compound("I001")
  expect_type(result, "list")
  expect_equal(result$xref$pubchem_cid, "5280343")
})

test_that("get_compound_admet returns single-row tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(
        caco2_permeability = -5.2,
        hia = "positive",
        bbb_penetration = "negative"
      )
    }
  )

  result <- get_compound_admet("I001")
  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 1L)
})

test_that("get_compound_targets method=drugclip returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(
        rank = 1:2,
        gene_symbol = c("EGFR", "VEGFA"),
        score = c(0.95, 0.88),
        stringsAsFactors = FALSE
      )
    }
  )

  result <- get_compound_targets("I001", method = "drugclip")
  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 2L)
})

test_that("get_compound_targets method=both adds source column", {
  call_count <- 0L
  local_mocked_bindings(
    unitcm_request = function(path, query = list(), ...) {
      call_count <<- call_count + 1L
      if (grepl("chembl", path)) {
        list(
          items = data.frame(gene_symbol = "TP53", stringsAsFactors = FALSE),
          total = 1L
        )
      } else {
        data.frame(gene_symbol = "EGFR", stringsAsFactors = FALSE)
      }
    }
  )

  result <- get_compound_targets("I001", method = "both")
  expect_true("source" %in% names(result))
  expect_setequal(result$source, c("drugclip", "chembl"))
})

test_that("get_compound_herbs returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(
        items = data.frame(
          unitcm_herb_id = "H001",
          herb_english_name = "Ginseng",
          stringsAsFactors = FALSE
        ),
        total = 1L
      )
    }
  )

  result <- get_compound_herbs("I001")
  expect_s3_class(result, "tbl_df")
})

test_that("fetch_compound_facets returns a list", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(total = 5000L, approved_count = 200L, mw_range = list(0, 1000))
    }
  )

  result <- fetch_compound_facets()
  expect_type(result, "list")
  expect_equal(result$total, 5000L)
})

test_that("export_compound_module validates module arg", {
  expect_error(
    export_compound_module("I001", "invalid_module"),
    "must be one of"
  )
})

#' Get latest submissions
#'
#' @return A [tibble::tibble()] of recent submissions with columns:
#'   `submission_id`, `project_title`, `submitted_by`, `updated_at`,
#'   `institution`, `total_file_size`.
#' @export
#' @examples
#' \dontrun{
#' fetch_latest_submissions()
#' }
fetch_latest_submissions <- function() {
  resp <- unitcm_request("/home/latest-submissions")
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }

  tibble::tibble()
}

#' Get homepage statistics
#'
#' @return A named list with fields: `total_datasets`, `total_downloads`,
#'   `total_file_size`, `recent_submissions_count`.
#' @export
#' @examples
#' \dontrun{
#' fetch_home_stats()
#' }
fetch_home_stats <- function() {
  unitcm_request("/home/stats")
}

#' Get TCM classification statistics
#'
#' @return A [tibble::tibble()] with columns: `classification`, `count`,
#'   `percentage`.
#' @export
#' @examples
#' \dontrun{
#' fetch_tcm_classification_stats()
#' }
fetch_tcm_classification_stats <- function() {
  resp <- unitcm_request("/home/tcm-classification-stats")
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }
  tibble::tibble()
}

#' Get omics type statistics
#'
#' @return A [tibble::tibble()] with columns: `omics_type`, `count`,
#'   `percentage`.
#' @export
#' @examples
#' \dontrun{
#' fetch_omics_type_stats()
#' }
fetch_omics_type_stats <- function() {
  resp <- unitcm_request("/home/omics-type-stats")
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }
  tibble::tibble()
}

test_that("fetch_latest_submissions returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(submission_id = "TMA2025001",
                 project_title = "Test project",
                 stringsAsFactors = FALSE)
    }
  )
  result <- fetch_latest_submissions()
  expect_s3_class(result, "tbl_df")
})

test_that("fetch_home_stats returns a list", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(total_datasets = 50L, total_downloads = 1000L)
    }
  )
  result <- fetch_home_stats()
  expect_type(result, "list")
})

test_that("fetch_tcm_classification_stats returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(classification = "Herb", count = 100L, percentage = 0.5,
                 stringsAsFactors = FALSE)
    }
  )
  result <- fetch_tcm_classification_stats()
  expect_s3_class(result, "tbl_df")
})

test_that("fetch_omics_type_stats returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(omics_type = "Transcriptomics", count = 30L, percentage = 0.6,
                 stringsAsFactors = FALSE)
    }
  )
  result <- fetch_omics_type_stats()
  expect_s3_class(result, "tbl_df")
})

#' Build an Herb-Compound-Target network
#'
#' Given herb names or IDs, fetches herb-compound-target data from the API
#' and constructs a typed `igraph` network.
#'
#' @param herbs Character vector of herb names or UniTCM herb IDs.
#' @param target_method Target prediction method passed to
#'   [get_compound_targets()]: `"drugclip"` (default), `"chembl"`, or
#'   `"both"`.
#' @param max_compounds Maximum compounds per herb to include (default 50).
#' @param progress Show progress messages (default `TRUE`).
#' @return An `igraph` graph object with vertex attributes `name`, `type`
#'   (`"herb"`, `"compound"`, `"target"`), and `label`.
#' @export
#' @examples
#' \dontrun{
#' g <- build_hct_network(c("UNITCM_H001", "UNITCM_H002"))
#' igraph::vcount(g)
#' }
build_hct_network <- function(herbs, target_method = "drugclip",
                              max_compounds = 50L, progress = TRUE) {
  check_pkg("igraph", reason = "to build network graphs")

  nodes <- list()
  edges <- list()

  for (herb in herbs) {
    if (progress) cli::cli_inform("Fetching compounds for {.val {herb}}...")

    is_id <- grepl("^UNITCM_H", herb, ignore.case = TRUE)
    if (is_id) {
      herb_id <- herb
      herb_info <- tryCatch(get_herb(herb_id), error = function(e) NULL)
      herb_label <- if (!is.null(herb_info)) {
        herb_info[["herb_english_name"]] %||% herb_id
      } else {
        herb_id
      }
    } else {
      result <- search_herbs(q = herb, page_size = 1L)
      if (nrow(result) == 0L) {
        if (progress) cli::cli_inform("  No herb found for {.val {herb}}, skipping.")
        next
      }
      herb_id <- result$unitcm_herb_id[1L]
      herb_label <- result$herb_english_name[1L] %||% herb_id
    }

    nodes[[herb_id]] <- list(name = herb_id, type = "herb", label = herb_label)

    comps <- get_herb_compounds(herb_id, page_size = max_compounds)
    if (nrow(comps) == 0L) next

    for (i in seq_len(nrow(comps))) {
      cid <- comps$unitcm_ingredient_id[i]
      cname <- comps$component_name[i] %||% cid
      nodes[[cid]] <- list(name = cid, type = "compound", label = cname)
      edges <- c(edges, list(list(from = herb_id, to = cid)))

      if (progress) cli::cli_inform("  Fetching targets for {.val {cname}}...")
      targets <- tryCatch(
        get_compound_targets(cid, method = target_method),
        error = function(e) tibble::tibble()
      )
      if (nrow(targets) > 0L && "gene_symbol" %in% names(targets)) {
        for (j in seq_len(nrow(targets))) {
          tid <- targets$gene_symbol[j]
          if (is.na(tid) || !nzchar(tid)) next
          nodes[[tid]] <- list(name = tid, type = "target", label = tid)
          edges <- c(edges, list(list(from = cid, to = tid)))
        }
      }
    }
  }

  if (length(nodes) == 0L) {
    rlang::abort("No data found for the given herbs.")
  }

  node_df <- do.call(rbind, lapply(nodes, as.data.frame,
    stringsAsFactors = FALSE))
  node_df <- node_df[!duplicated(node_df$name), ]

  edge_df <- do.call(rbind, lapply(edges, as.data.frame,
    stringsAsFactors = FALSE))

  all_endpoints <- unique(c(edge_df$from, edge_df$to))
  missing_nodes <- setdiff(all_endpoints, node_df$name)
  missing_nodes <- missing_nodes[!is.na(missing_nodes) & nzchar(missing_nodes)]
  if (length(missing_nodes) > 0L) {
    extra <- data.frame(name = missing_nodes, type = NA_character_,
      label = missing_nodes, stringsAsFactors = FALSE)
    node_df <- rbind(node_df, extra)
  }

  g <- igraph::graph_from_data_frame(edge_df, directed = FALSE,
    vertices = node_df)

  if (progress) {
    cli::cli_inform("Network built: {igraph::vcount(g)} nodes, {igraph::ecount(g)} edges.")
  }
  g
}

#' Build a Formula-Herb network
#'
#' Given a formula order ID, fetches its herb doses and constructs a
#' star-topology network.
#'
#' @param formula_id The formula order ID (integer or character).
#' @return An `igraph` graph object with vertex attributes `name`, `type`
#'   (`"formula"`, `"herb"`), `label`, and `dose` (for herbs).
#' @export
#' @examples
#' \dontrun{
#' g <- build_formula_herb_network(1)
#' igraph::V(g)$label
#' }
build_formula_herb_network <- function(formula_id) {
  check_pkg("igraph", reason = "to build network graphs")

  formula <- get_formula(formula_id)
  doses <- get_formula_doses(formula_id)

  fname <- formula[["formula_name"]] %||% paste0("Formula_", formula_id)
  fnode_id <- paste0("F:", formula_id)

  nodes <- data.frame(
    name = fnode_id,
    type = "formula",
    label = fname,
    dose = NA_character_,
    stringsAsFactors = FALSE
  )

  edges <- data.frame(from = character(), to = character(),
    stringsAsFactors = FALSE)

  if (nrow(doses) > 0L) {
    for (i in seq_len(nrow(doses))) {
      hname <- doses$herb_name[i]
      hid <- paste0("herb:", hname)
      dose_str <- doses$original_dose[i] %||% NA_character_

      nodes <- rbind(nodes, data.frame(
        name = hid, type = "herb", label = hname,
        dose = dose_str, stringsAsFactors = FALSE
      ))
      edges <- rbind(edges, data.frame(
        from = fnode_id, to = hid, stringsAsFactors = FALSE
      ))
    }
  }

  igraph::graph_from_data_frame(edges, directed = FALSE, vertices = nodes)
}

#' Convert a NetVis graph response to igraph
#'
#' @param graph_response A list with `$nodes` and `$edges` tibbles, as
#'   returned by [get_neighbors()], [get_subgraph()], or [find_path()].
#' @return An `igraph` graph object.
#' @export
#' @examples
#' \dontrun{
#' resp <- get_neighbors("H:UNITCM_H001")
#' g <- as_igraph(resp)
#' }
as_igraph <- function(graph_response) {
  check_pkg("igraph", reason = "to convert graph responses")

  nodes <- graph_response$nodes
  edges <- graph_response$edges

  if (is.null(nodes) || nrow(nodes) == 0L) {
    return(igraph::make_empty_graph(directed = FALSE))
  }

  if (!is.null(edges) && nrow(edges) > 0L) {
    edge_cols <- intersect(c("source", "from"), names(edges))
    target_cols <- intersect(c("target", "to"), names(edges))
    if (length(edge_cols) > 0L && length(target_cols) > 0L) {
      edge_df <- data.frame(
        from = as.character(edges[[edge_cols[1L]]]),
        to = as.character(edges[[target_cols[1L]]]),
        stringsAsFactors = FALSE
      )
      extra_edge_cols <- setdiff(names(edges),
        c("source", "from", "target", "to"))
      for (col in extra_edge_cols) {
        val <- edges[[col]]
        if (is.atomic(val)) edge_df[[col]] <- val
      }
    } else {
      edge_df <- data.frame(from = character(), to = character(),
        stringsAsFactors = FALSE)
    }
  } else {
    edge_df <- data.frame(from = character(), to = character(),
      stringsAsFactors = FALSE)
  }

  node_df <- as.data.frame(nodes, stringsAsFactors = FALSE)

  # Ensure the vertex-name column exists and is FIRST. igraph uses the
  # first column of `vertices` as the vertex name, so simply appending
  # `name <- id` is not enough.
  if (!"name" %in% names(node_df)) {
    if ("id" %in% names(node_df)) {
      node_df$name <- as.character(node_df$id)
    } else {
      rlang::abort("`graph_response$nodes` must have an `id` or `name` column.")
    }
  } else {
    node_df$name <- as.character(node_df$name)
  }
  node_df <- node_df[, c("name", setdiff(names(node_df), "name")), drop = FALSE]

  # Drop list-columns / non-atomic columns — igraph cannot store them as
  # vertex attributes and they trigger "undefined columns selected" errors
  # when set_vertex_attr tries to index them.
  keep_cols <- vapply(node_df, function(col) is.atomic(col) || is.factor(col),
                      logical(1L))
  node_df <- node_df[, keep_cols, drop = FALSE]

  # Add any edge endpoints not present in nodes as orphan vertices so
  # igraph::graph_from_data_frame() does not error.
  all_endpoints <- unique(c(edge_df$from, edge_df$to))
  missing_nodes <- setdiff(all_endpoints, node_df$name)
  missing_nodes <- missing_nodes[!is.na(missing_nodes) & nzchar(missing_nodes)]
  if (length(missing_nodes) > 0L) {
    extra <- data.frame(name = missing_nodes, stringsAsFactors = FALSE)
    for (col in setdiff(names(node_df), "name")) extra[[col]] <- NA
    node_df <- rbind(node_df, extra[, names(node_df), drop = FALSE])
  }

  # Drop duplicate vertex names (keep first).
  node_df <- node_df[!duplicated(node_df$name), , drop = FALSE]

  igraph::graph_from_data_frame(edge_df, directed = FALSE,
    vertices = node_df)
}

#' Convert a NetVis graph response to tidygraph
#'
#' @inheritParams as_igraph
#' @return A `tidygraph::tbl_graph` object.
#' @export
#' @examples
#' \dontrun{
#' resp <- get_neighbors("H:UNITCM_H001")
#' tg <- as_tidygraph(resp)
#' }
as_tidygraph <- function(graph_response) {
  check_pkg("tidygraph", reason = "to convert to tbl_graph")
  g <- as_igraph(graph_response)
  tidygraph::as_tbl_graph(g)
}

test_that("as_igraph converts graph response", {
  skip_if_not_installed("igraph")
  resp <- list(
    nodes = tibble::tibble(id = c("A", "B", "C"), type = c("herb", "compound", "target")),
    edges = tibble::tibble(source = c("A", "B"), target = c("B", "C"))
  )
  g <- as_igraph(resp)
  expect_true(igraph::is.igraph(g))
  expect_equal(igraph::vcount(g), 3L)
  expect_equal(igraph::ecount(g), 2L)
})

test_that("as_igraph handles empty graph", {
  skip_if_not_installed("igraph")
  resp <- list(nodes = tibble::tibble(), edges = tibble::tibble())
  g <- as_igraph(resp)
  expect_true(igraph::is.igraph(g))
  expect_equal(igraph::vcount(g), 0L)
})

test_that("as_tidygraph converts graph response", {
  skip_if_not_installed("igraph")
  skip_if_not_installed("tidygraph")
  resp <- list(
    nodes = tibble::tibble(id = c("A", "B"), type = c("herb", "compound")),
    edges = tibble::tibble(source = "A", target = "B")
  )
  tg <- as_tidygraph(resp)
  expect_s3_class(tg, "tbl_graph")
})

test_that("build_formula_herb_network creates igraph", {
  skip_if_not_installed("igraph")
  local_mocked_bindings(
    get_formula = function(id) list(formula_name = "Test Formula"),
    get_formula_doses = function(id) {
      tibble::tibble(herb_name = c("Ren Shen", "Huang Qi"),
                     original_dose = c("10g", "15g"))
    }
  )
  g <- build_formula_herb_network(1)
  expect_true(igraph::is.igraph(g))
  expect_equal(igraph::vcount(g), 3L)
})

test_that("parse_graph_response handles extra fields", {
  resp <- list(
    nodes = data.frame(id = "A", stringsAsFactors = FALSE),
    edges = data.frame(source = character(), target = character(),
                       stringsAsFactors = FALSE),
    has_more = list(compound = TRUE)
  )
  result <- unitcm:::parse_graph_response(resp)
  expect_true("has_more" %in% names(result))
})

#' Search Target2NP compound-target interactions
#'
#' Query the UniTCM Target2NP database of natural-product to protein-target
#' interactions, combining records from experimental sources such as
#' BindingDB, HERB2, NPASS, BATMAN, and others.
#'
#' @param search Free-text search query.
#' @param search_field Field to restrict the search to. One of `"all"`,
#'   `"gene_symbol"`, `"compound_name"`, `"uniprot_id"`, `"inchikey"`,
#'   `"pubchem_cid"`, or `"chembl_id"`.
#' @param search_mode `"exact"` (case-insensitive equality) or `"fuzzy"`
#'   (substring match).
#' @param source_db Filter by source database (e.g. `"BindingDB"`).
#' @param evidence_level Filter by evidence level (integer 1-4 as string).
#' @param evidence_label Filter by evidence label.
#' @param target_organism Filter by target organism (e.g. `"Homo sapiens"`).
#' @param interaction_type Filter by interaction type.
#' @param activity_type Filter by activity type (e.g. `"IC50"`, `"Ki"`).
#' @param page Page number (default 1).
#' @param page_size Results per page (default 20, max 100).
#' @param all_pages If `TRUE`, fetch all pages.
#' @return A [tibble::tibble()] of interaction records with attribute
#'   `"total"`.
#' @export
#' @examples
#' \dontrun{
#' search_target2np(search = "quercetin")
#' search_target2np(search = "TP53", search_field = "gene_symbol",
#'                  source_db = "BindingDB")
#' }
search_target2np <- function(search = NULL,
                             search_field = c("all", "gene_symbol",
                                              "compound_name", "uniprot_id",
                                              "inchikey", "pubchem_cid",
                                              "chembl_id"),
                             search_mode = c("exact", "fuzzy"),
                             source_db = NULL, evidence_level = NULL,
                             evidence_label = NULL, target_organism = NULL,
                             interaction_type = NULL, activity_type = NULL,
                             page = 1L, page_size = 20L,
                             all_pages = FALSE) {
  search_field <- rlang::arg_match(search_field)
  search_mode <- rlang::arg_match(search_mode)

  query <- list(
    search = search,
    search_field = search_field,
    search_mode = search_mode,
    source_db = source_db,
    evidence_level = evidence_level,
    evidence_label = evidence_label,
    target_organism = target_organism,
    interaction_type = interaction_type,
    activity_type = activity_type
  )

  if (isTRUE(all_pages)) {
    return(unitcm_paginate("/target2np", query = query))
  }

  query[["page"]] <- page
  query[["page_size"]] <- page_size
  resp <- unitcm_request("/target2np", query = query)

  items <- resp[["items"]] %||% list()
  result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
  attr(result, "total") <- resp[["total"]] %||% 0L
  result
}

#' Get a single Target2NP interaction record
#'
#' Retrieve the full detail for one compound-target interaction record.
#'
#' @param record_id Integer record ID.
#' @return A named list of interaction fields.
#' @export
#' @examples
#' \dontrun{
#' get_target2np(1)
#' }
get_target2np <- function(record_id) {
  unitcm_request(paste0("/target2np/", as.integer(record_id)))
}

#' Fetch Target2NP filter options
#'
#' Returns the controlled values used by the Target2NP filter UI
#' (source databases, evidence labels, top target organisms, interaction
#' types, and top activity types).
#'
#' @return A named list with fields `source_db`, `evidence_label`,
#'   `target_organism`, `interaction_type`, `activity_type`.
#' @export
#' @examples
#' \dontrun{
#' opts <- fetch_target2np_filters()
#' opts$source_db
#' }
fetch_target2np_filters <- function() {
  unitcm_request("/target2np/filter-options")
}

#' Fetch Target2NP database statistics
#'
#' Returns counts and distributions across source databases, evidence
#' levels, target organisms, and activity types.
#'
#' @return A named list with `total_records` and four distribution lists.
#' @export
#' @examples
#' \dontrun{
#' stats <- fetch_target2np_stats()
#' stats$total_records
#' }
fetch_target2np_stats <- function() {
  unitcm_request("/target2np/statistics")
}

#' Batch query Target2NP by identifier list
#'
#' Look up interaction records for up to 50 gene symbols, UniProt IDs, or
#' Entrez gene IDs in one call.
#'
#' @param identifiers Character vector of identifiers (max 50).
#' @param id_type One of `"gene_symbol"` (default), `"uniprot_id"`, or
#'   `"entrez_gene_id"`.
#' @return A [tibble::tibble()] of matching records with attributes
#'   `"total"`, `"queries_matched"`, and `"queries_not_found"`.
#' @export
#' @examples
#' \dontrun{
#' batch_target2np(c("TP53", "BRCA1", "EGFR"))
#' batch_target2np(c("P04637", "P38398"), id_type = "uniprot_id")
#' }
batch_target2np <- function(identifiers,
                            id_type = c("gene_symbol", "uniprot_id",
                                        "entrez_gene_id")) {
  id_type <- rlang::arg_match(id_type)
  body <- list(
    identifiers = as.list(identifiers),
    id_type = id_type
  )
  resp <- unitcm_request("/target2np/batch", method = "POST", body = body)

  results <- resp[["results"]] %||% list()
  out <- if (length(results) > 0L) tibble::as_tibble(results) else tibble::tibble()
  attr(out, "total") <- resp[["total"]] %||% 0L
  attr(out, "queries_matched") <- resp[["queries_matched"]] %||% 0L
  attr(out, "queries_not_found") <- resp[["queries_not_found"]] %||% character()
  out
}

#' Aggregated Target2NP view across data sources
#'
#' Return compound-target pairs (keyed by InChIKey + UniProt ID) supported
#' by interaction records in at least `min_sources` source databases.
#' Optionally extends each pair with DrugCLIP / SEA prediction support.
#'
#' @param search Free-text search query.
#' @param target_organism Optional target organism filter.
#' @param min_sources Minimum number of source databases supporting the
#'   pair (1-5, default 2).
#' @param include_predictions If `TRUE`, also count DrugCLIP and SEA
#'   predictions as additional supporting sources.
#' @param page Page number (default 1).
#' @param page_size Results per page (default 20, max 50).
#' @return A [tibble::tibble()] of aggregated pairs with attribute `"total"`.
#' @export
#' @examples
#' \dontrun{
#' aggregated_target2np(search = "quercetin")
#' aggregated_target2np(search = "TP53", min_sources = 3,
#'                      include_predictions = TRUE)
#' }
aggregated_target2np <- function(search = NULL, target_organism = NULL,
                                 min_sources = 2L,
                                 include_predictions = FALSE,
                                 page = 1L, page_size = 20L) {
  query <- list(
    search = search,
    target_organism = target_organism,
    min_sources = min_sources,
    include_predictions = include_predictions,
    page = page,
    page_size = page_size
  )
  resp <- unitcm_request("/target2np/aggregated", query = query)

  items <- resp[["items"]] %||% list()
  result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
  attr(result, "total") <- resp[["total"]] %||% 0L
  result
}

#' Search DrugCLIP predicted compound-target interactions
#'
#' Query DrugCLIP deep-learning predictions, with optional confidence
#' filtering by predicted score.
#'
#' @param search Free-text search.
#' @param search_field One of `"all"`, `"gene_symbol"`, `"compound_name"`,
#'   or `"inchikey"`.
#' @param search_mode `"exact"` or `"fuzzy"`.
#' @param min_score Minimum DrugCLIP score (0-1).
#' @param confidence One of `"high"` (>= 0.8), `"medium"` (0.5-0.8),
#'   `"low"` (< 0.5).
#' @param page,page_size Pagination.
#' @param all_pages If `TRUE`, fetch all pages.
#' @return A [tibble::tibble()] of DrugCLIP predictions with attribute
#'   `"total"`.
#' @export
#' @examples
#' \dontrun{
#' search_target2np_drugclip(search = "quercetin", confidence = "high")
#' }
search_target2np_drugclip <- function(search = NULL,
                                      search_field = c("all", "gene_symbol",
                                                       "compound_name",
                                                       "inchikey"),
                                      search_mode = c("exact", "fuzzy"),
                                      min_score = NULL, confidence = NULL,
                                      page = 1L, page_size = 20L,
                                      all_pages = FALSE) {
  search_field <- rlang::arg_match(search_field)
  search_mode <- rlang::arg_match(search_mode)

  query <- list(
    search = search,
    search_field = search_field,
    search_mode = search_mode,
    min_score = min_score,
    confidence = confidence
  )

  if (isTRUE(all_pages)) {
    return(unitcm_paginate("/target2np/drugclip", query = query))
  }

  query[["page"]] <- page
  query[["page_size"]] <- page_size
  resp <- unitcm_request("/target2np/drugclip", query = query)

  items <- resp[["items"]] %||% list()
  result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
  attr(result, "total") <- resp[["total"]] %||% 0L
  result
}

#' Search SEA (ChEMBL similarity) predicted compound-target interactions
#'
#' Query SEA-style predictions derived from ChEMBL similarity scoring,
#' with optional adjusted p-value filtering.
#'
#' @param search Free-text search.
#' @param search_field One of `"all"`, `"gene_symbol"`, `"compound_name"`,
#'   `"uniprot_id"`, or `"inchikey"`.
#' @param search_mode `"exact"` or `"fuzzy"`.
#' @param max_pvalue Maximum adjusted p-value.
#' @param confidence One of `"high"` (adj. p < 0.01), `"medium"`
#'   (0.01-0.05), `"low"` (>= 0.05).
#' @param page,page_size Pagination.
#' @param all_pages If `TRUE`, fetch all pages.
#' @return A [tibble::tibble()] of SEA predictions with attribute `"total"`.
#' @export
#' @examples
#' \dontrun{
#' search_target2np_sea(search = "quercetin", confidence = "high")
#' }
search_target2np_sea <- function(search = NULL,
                                 search_field = c("all", "gene_symbol",
                                                  "compound_name",
                                                  "uniprot_id", "inchikey"),
                                 search_mode = c("exact", "fuzzy"),
                                 max_pvalue = NULL, confidence = NULL,
                                 page = 1L, page_size = 20L,
                                 all_pages = FALSE) {
  search_field <- rlang::arg_match(search_field)
  search_mode <- rlang::arg_match(search_mode)

  query <- list(
    search = search,
    search_field = search_field,
    search_mode = search_mode,
    max_pvalue = max_pvalue,
    confidence = confidence
  )

  if (isTRUE(all_pages)) {
    return(unitcm_paginate("/target2np/sea", query = query))
  }

  query[["page"]] <- page
  query[["page_size"]] <- page_size
  resp <- unitcm_request("/target2np/sea", query = query)

  items <- resp[["items"]] %||% list()
  result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
  attr(result, "total") <- resp[["total"]] %||% 0L
  result
}

#' Multi-source summary for a Target2NP query
#'
#' Combine experimental records, DrugCLIP predictions, and SEA predictions
#' for one query, returning source counts, target/compound overlap sets,
#' confidence distributions, cross-validated compound-target pairs, and
#' an interpretive suggestion string.
#'
#' @param search Free-text search query.
#' @param sources Character vector of sources to query. Subset of
#'   `c("experimental", "drugclip", "sea")` (default: all three).
#' @param search_field One of `"all"`, `"gene_symbol"`, `"compound_name"`,
#'   `"uniprot_id"`, `"inchikey"`, `"pubchem_cid"`, `"chembl_id"`.
#' @param search_mode `"exact"` or `"fuzzy"`.
#' @return A named list with fields `source_counts`, `target_overlap`,
#'   `compound_overlap`, `confidence_distribution`, `cross_validated`,
#'   and `suggestion_text`.
#' @export
#' @examples
#' \dontrun{
#' summary <- target2np_multi_source_summary(
#'   search = "TP53", search_field = "gene_symbol"
#' )
#' summary$source_counts
#' summary$suggestion_text
#' }
target2np_multi_source_summary <- function(search = NULL,
                                           sources = c("experimental",
                                                       "drugclip", "sea"),
                                           search_field = c("all",
                                                            "gene_symbol",
                                                            "compound_name",
                                                            "uniprot_id",
                                                            "inchikey",
                                                            "pubchem_cid",
                                                            "chembl_id"),
                                           search_mode = c("exact", "fuzzy")) {
  search_field <- rlang::arg_match(search_field)
  search_mode <- rlang::arg_match(search_mode)
  sources <- match.arg(sources, several.ok = TRUE)

  query <- list(
    search = search,
    sources = collapse_param(sources, ","),
    search_field = search_field,
    search_mode = search_mode
  )
  unitcm_request("/target2np/multi-source-summary", query = query)
}

#' Auto-paginate a UniTCM API endpoint
#'
#' Fetches all pages from a paginated endpoint and combines the results
#' into a single tibble. Handles both pagination styles used by the API.
#'
#' @param path API path.
#' @param query Named list of query parameters (excluding `page`/`page_size`).
#' @param max_pages Maximum number of pages to fetch (default 50).
#' @param page_size Number of records per page (default 100).
#' @param progress Show a CLI progress bar (default `TRUE`).
#' @return A [tibble::tibble()] with attribute `"total"` set to the
#'   server-reported total count.
#' @noRd
unitcm_paginate <- function(path, query = list(), max_pages = 50L,
                            page_size = 100L, progress = TRUE) {
  all_items <- list()
  page <- 1L

  if (progress) {
    cli::cli_progress_bar("Fetching pages", clear = FALSE)
  }

  repeat {
    query[["page"]] <- page
    query[["page_size"]] <- page_size
    resp <- unitcm_request(path, query = query)

    items <- resp[["items"]] %||% resp[["results"]] %||% list()
    total <- resp[["total"]] %||% resp[["count"]] %||% 0L

    if (length(items) == 0L) break

    if (is.data.frame(items)) {
      all_items <- c(all_items, list(tibble::as_tibble(items)))
    } else if (is.list(items)) {
      all_items <- c(all_items, list(
        tryCatch(
          tibble::as_tibble(do.call(rbind, lapply(items, as.data.frame,
            stringsAsFactors = FALSE))),
          error = function(e) tibble::as_tibble(items)
        )
      ))
    }

    if (progress) cli::cli_progress_update()

    fetched <- page * page_size
    if (fetched >= total || page >= max_pages) break
    page <- page + 1L
  }

  if (progress) cli::cli_progress_done()

  if (length(all_items) == 0L) {
    result <- tibble::tibble()
    attr(result, "total") <- 0L
    return(result)
  }

  result <- dplyr::bind_rows(all_items)
  attr(result, "total") <- as.integer(total)
  result
}

test_that("unitcm_paginate handles Style A (items + total)", {
  page_count <- 0L
  local_mocked_bindings(
    unitcm_request = function(path, method = "GET", query = list(), ...) {
      page_count <<- page_count + 1L
      if (query$page == 1L) {
        list(items = data.frame(id = c("a", "b"), name = c("A", "B"),
                                stringsAsFactors = FALSE), total = 3L)
      } else {
        list(items = data.frame(id = "c", name = "C",
                                stringsAsFactors = FALSE), total = 3L)
      }
    }
  )

  result <- unitcm_paginate("/herbs", page_size = 2L, progress = FALSE)
  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 3L)
  expect_equal(attr(result, "total"), 3L)
})

test_that("unitcm_paginate handles Style B (results + count)", {
  local_mocked_bindings(
    unitcm_request = function(path, method = "GET", query = list(), ...) {
      list(results = data.frame(id = "x", stringsAsFactors = FALSE), count = 1L)
    }
  )

  result <- unitcm_paginate("/transcriptome-hub", page_size = 10L,
                            progress = FALSE)
  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 1L)
  expect_equal(attr(result, "total"), 1L)
})

test_that("unitcm_paginate returns empty tibble for no results", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) list(items = list(), total = 0L)
  )

  result <- unitcm_paginate("/herbs", progress = FALSE)
  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 0L)
  expect_equal(attr(result, "total"), 0L)
})

test_that("unitcm_paginate respects max_pages", {
  pages_fetched <- 0L
  local_mocked_bindings(
    unitcm_request = function(path, method = "GET", query = list(), ...) {
      pages_fetched <<- pages_fetched + 1L
      list(items = data.frame(id = paste0("item_", pages_fetched),
                              stringsAsFactors = FALSE), total = 1000L)
    }
  )

  result <- unitcm_paginate("/herbs", max_pages = 3L, page_size = 10L,
                            progress = FALSE)
  expect_equal(pages_fetched, 3L)
})

#' Search the TCM Ontology
#'
#' Full-text search across TCM ontology entities.
#'
#' @param q Search query (required).
#' @param limit Maximum results to return (default 20).
#' @param level Filter by ontology level (integer, 1--4).
#' @param category Filter by top-level category.
#' @return A [tibble::tibble()] with columns: `tcm_id`, `name`, `name_cn`,
#'   `level`, `path`, `match_field`, `highlight`.
#' @export
#' @examples
#' \dontrun{
#' search_ontology("Qi stagnation")
#' }
search_ontology <- function(q, limit = 20L, level = NULL, category = NULL) {
  resp <- unitcm_request(
    "/tools/tcm-ontology/search",
    query = list(q = q, limit = limit, level = level, category = category)
  )
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }
  tibble::tibble()
}

#' Get a TCM ontology entity
#'
#' Retrieve full detail for one entity including ancestors, children,
#' external mappings, and relations.
#'
#' @param tcm_id The TCM ontology ID (e.g. `"TCM_0001"`).
#' @return A named list with sub-elements: `ancestors`, `children`,
#'   `external_mappings`, `relations`.
#' @export
#' @examples
#' \dontrun{
#' get_ontology_entity("TCM_0001")
#' }
get_ontology_entity <- function(tcm_id) {
  unitcm_request(paste0("/tools/tcm-ontology/entity/", tcm_id))
}

#' Get children of an ontology entity
#'
#' @param tcm_id The TCM ontology ID.
#' @return A [tibble::tibble()] with columns: `tcm_id`, `name`, `name_cn`,
#'   `level`, `path`, `children_count`, `has_children`.
#' @export
#' @examples
#' \dontrun{
#' get_ontology_children("TCM_0001")
#' }
get_ontology_children <- function(tcm_id) {
  resp <- unitcm_request(paste0("/tools/tcm-ontology/entity/", tcm_id, "/children"))
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }
  tibble::tibble()
}

#' Get all descendants of an ontology entity
#'
#' @param tcm_id The TCM ontology ID.
#' @param max_level Maximum depth to descend (integer or `NULL` for all).
#' @return A [tibble::tibble()].
#' @export
#' @examples
#' \dontrun{
#' get_ontology_descendants("TCM_0001", max_level = 2)
#' }
get_ontology_descendants <- function(tcm_id, max_level = NULL) {
  resp <- unitcm_request(
    paste0("/tools/tcm-ontology/entity/", tcm_id, "/descendants"),
    query = list(max_level = max_level)
  )
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }
  tibble::tibble()
}

#' Get ancestors of an ontology entity
#'
#' @param tcm_id The TCM ontology ID.
#' @return A [tibble::tibble()] with columns: `tcm_id`, `name`, `name_cn`,
#'   `level`.
#' @export
#' @examples
#' \dontrun{
#' get_ontology_ancestors("TCM_0001")
#' }
get_ontology_ancestors <- function(tcm_id) {
  resp <- unitcm_request(paste0("/tools/tcm-ontology/entity/", tcm_id, "/ancestors"))
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }
  tibble::tibble()
}

#' Fetch the TCM ontology tree
#'
#' Returns the full ontology as a recursive nested list.
#'
#' @param depth Tree depth to return (1--10, default 4).
#' @return A recursive nested list: `list(tcm_id, name, name_cn, level,
#'   children = list(...))`.
#' @export
#' @examples
#' \dontrun{
#' tree <- fetch_ontology_tree(depth = 2)
#' }
fetch_ontology_tree <- function(depth = 4L) {
  unitcm_request("/tools/tcm-ontology/tree", query = list(depth = depth))
}

#' Fetch ontology statistics
#'
#' @return A named list with fields: `total_entities`, `total_level1`--
#'   `total_level4`, `total_relations`, `total_mappings`, `categories`.
#' @export
#' @examples
#' \dontrun{
#' fetch_ontology_stats()
#' }
fetch_ontology_stats <- function() {
  unitcm_request("/tools/tcm-ontology/stats")
}

#' List top-level ontology categories
#'
#' @return A [tibble::tibble()] of level-1 entities.
#' @export
#' @examples
#' \dontrun{
#' list_ontology_categories()
#' }
list_ontology_categories <- function() {
  resp <- unitcm_request("/tools/tcm-ontology/categories")
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }
  tibble::tibble()
}

#' Get ontology entities by level
#'
#' @param level Ontology level (integer, 1--4).
#' @param parent_id Optional parent entity ID to filter by.
#' @return A [tibble::tibble()].
#' @export
#' @examples
#' \dontrun{
#' get_ontology_by_level(2)
#' }
get_ontology_by_level <- function(level, parent_id = NULL) {
  resp <- unitcm_request(
    paste0("/tools/tcm-ontology/by-level/", level),
    query = list(parent_id = parent_id)
  )
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }
  tibble::tibble()
}

#' Search ontology external mapping
#'
#' Find TCM entities mapped to an external database identifier.
#'
#' @param database External database name. Must be one of: `"UMLS"`,
#'   `"SNOMED_CT"`, `"ICD11_TM"`, `"MeSH"`.
#' @param external_id The external identifier to look up.
#' @return A [tibble::tibble()] of matched TCM entities.
#' @export
#' @examples
#' \dontrun{
#' search_ontology_mapping("MeSH", "D008516")
#' }
search_ontology_mapping <- function(database, external_id) {
  database <- rlang::arg_match(database,
    values = c("UMLS", "SNOMED_CT", "ICD11_TM", "MeSH"))
  resp <- unitcm_request(
    "/tools/tcm-ontology/mapping/search",
    query = list(database = database, external_id = external_id)
  )
  if (is.data.frame(resp)) return(tibble::as_tibble(resp))
  if (is.list(resp) && length(resp) > 0L) {
    return(tryCatch(tibble::as_tibble(resp), error = function(e) tibble::tibble()))
  }
  tibble::tibble()
}

#' Export the TCM ontology
#'
#' Download the full ontology in JSON, OWL/RDF, or CSV format.
#'
#' @param format Export format: `"json"`, `"owl"`, or `"csv"`.
#' @param file Output file path. If `NULL`, auto-named as
#'   `"unitcm_ontology.{ext}"`.
#' @param depth Tree depth for JSON export (default 4).
#' @return Invisible file path.
#' @export
#' @examples
#' \dontrun{
#' export_ontology("csv")
#' export_ontology("json", depth = 2, file = "ontology_shallow.json")
#' }
export_ontology <- function(format = c("json", "owl", "csv"),
                            file = NULL, depth = 4L) {
  format <- rlang::arg_match(format)

  ext <- switch(format, json = "json", owl = "owl", csv = "csv")
  file <- file %||% paste0("unitcm_ontology.", ext)

  query <- if (format == "json") list(depth = depth) else list()
  unitcm_download(
    paste0("/tools/tcm-ontology/export/", format),
    query = query, file = file
  )
}

test_that("search_ontology returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(
        tcm_id = "TCM_0001",
        name = "Qi stagnation",
        name_cn = "\u6c14\u6ede",
        level = 2L,
        stringsAsFactors = FALSE
      )
    }
  )

  result <- search_ontology("Qi stagnation")
  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 1L)
})

test_that("get_ontology_entity returns a list with sub-elements", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(
        tcm_id = "TCM_0001",
        name = "Qi stagnation",
        ancestors = list(),
        children = list(),
        external_mappings = list(),
        relations = list()
      )
    }
  )

  result <- get_ontology_entity("TCM_0001")
  expect_type(result, "list")
  expect_true("ancestors" %in% names(result))
  expect_true("external_mappings" %in% names(result))
})

test_that("search_ontology_mapping validates database argument", {
  expect_error(
    search_ontology_mapping("InvalidDB", "123"),
    "must be one of"
  )
})

test_that("search_ontology_mapping returns tibble for valid database", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(
        tcm_id = "TCM_0001",
        name = "Qi stagnation",
        stringsAsFactors = FALSE
      )
    }
  )

  result <- search_ontology_mapping("MeSH", "D008516")
  expect_s3_class(result, "tbl_df")
})

test_that("export_ontology validates format argument", {
  expect_error(
    export_ontology("invalid_format"),
    "must be one of"
  )
})

test_that("fetch_ontology_stats returns a list", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(total_entities = 5000L, total_relations = 10000L)
    }
  )

  result <- fetch_ontology_stats()
  expect_type(result, "list")
  expect_equal(result$total_entities, 5000L)
})

test_that("get_ontology_children returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      data.frame(
        tcm_id = c("TCM_0002", "TCM_0003"),
        name = c("Child1", "Child2"),
        has_children = c(TRUE, FALSE),
        stringsAsFactors = FALSE
      )
    }
  )

  result <- get_ontology_children("TCM_0001")
  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 2L)
})

#' Search herbs in the Herb Explorer
#'
#' Query the UniTCM Herb Explorer with optional text search and faceted
#' filters. Multi-value filter parameters accept character vectors and are
#' collapsed to semicolon-separated strings internally.
#'
#' @param q Optional search query string.
#' @param therapeutic_en English therapeutic classification filter (character
#'   vector).
#' @param therapeutic_cn Chinese therapeutic classification filter (character
#'   vector).
#' @param family Botanical family filter (character vector).
#' @param toxicity Toxicity level filter (character vector).
#' @param source Data source filter (character vector).
#' @param flavors Flavor filter (character vector).
#' @param properties Property filter (character vector).
#' @param meridians Meridian tropism filter (character vector).
#' @param medicinal_part Medicinal part filter (character vector).
#' @param page Page number (default 1).
#' @param page_size Results per page (default 20, max 200).
#' @param all_pages If `TRUE`, fetch all pages via auto-pagination.
#' @return A [tibble::tibble()] of herbs with attribute `"total"`.
#' @export
#' @examples
#' \dontrun{
#' search_herbs(q = "ginseng")
#' search_herbs(flavors = c("sweet", "bitter"), page_size = 50)
#' }
search_herbs <- function(q = NULL, therapeutic_en = NULL,
                         therapeutic_cn = NULL, family = NULL,
                         toxicity = NULL, source = NULL,
                         flavors = NULL, properties = NULL,
                         meridians = NULL, medicinal_part = NULL,
                         page = 1L, page_size = 20L,
                         all_pages = FALSE) {
  query <- list(
    q = q,
    therapeutic_en = collapse_param(therapeutic_en, ";"),
    therapeutic_cn = collapse_param(therapeutic_cn, ";"),
    family = collapse_param(family, ";"),
    toxicity = collapse_param(toxicity, ";"),
    source = collapse_param(source, ";"),
    flavors = collapse_param(flavors, ";"),
    properties = collapse_param(properties, ";"),
    meridians = collapse_param(meridians, ";"),
    medicinal_part = collapse_param(medicinal_part, ";")
  )

  if (isTRUE(all_pages)) {
    return(unitcm_paginate("/herbs", query = query))
  }

  query[["page"]] <- page
  query[["page_size"]] <- page_size
  resp <- unitcm_request("/herbs", query = query)

  items <- resp[["items"]] %||% list()
  result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
  attr(result, "total") <- resp[["total"]] %||% 0L
  result
}

#' Get a single herb by ID
#'
#' Retrieve full detail for one herb from the Herb Explorer.
#'
#' @param herb_id The UniTCM herb ID (e.g. `"UNITCM_H001"`).
#' @return A named list with 31 fields including cross-reference IDs.
#' @export
#' @examples
#' \dontrun{
#' get_herb("UNITCM_H001")
#' }
get_herb <- function(herb_id) {
  unitcm_request(paste0("/herbs/", normalize_id(herb_id)))
}

#' Get herb filter facets
#'
#' Returns available filter values and their counts for the Herb Explorer.
#'
#' @return A named list of tibbles, one per facet field
#'   (e.g. `therapeutic_en_class`, `family`, `toxicity`).
#' @export
#' @examples
#' \dontrun{
#' facets <- fetch_herb_facets()
#' facets$toxicity
#' }
fetch_herb_facets <- function() {
  resp <- unitcm_request("/herbs/facets")
  lapply(resp, function(x) {
    if (is.data.frame(x)) {
      tibble::as_tibble(x)
    } else if (is.list(x) && length(x) > 0L) {
      tryCatch(tibble::as_tibble(x), error = function(e) x)
    } else {
      x
    }
  })
}

#' Get compounds for a herb
#'
#' List chemical compounds (ingredients) associated with a specific herb.
#'
#' @param herb_id The UniTCM herb ID.
#' @param page Page number (default 1).
#' @param page_size Results per page (default 20).
#' @param all_pages If `TRUE`, fetch all pages.
#' @return A [tibble::tibble()] of compounds with attribute `"total"`.
#' @export
#' @examples
#' \dontrun{
#' get_herb_compounds("UNITCM_H001")
#' }
get_herb_compounds <- function(herb_id, page = 1L, page_size = 20L,
                               all_pages = FALSE) {
  path <- paste0("/herbs/", normalize_id(herb_id), "/compounds")

  if (isTRUE(all_pages)) {
    return(unitcm_paginate(path))
  }

  resp <- unitcm_request(path, query = list(page = page, page_size = page_size))
  items <- resp[["items"]] %||% list()
  result <- if (length(items) > 0L) tibble::as_tibble(items) else tibble::tibble()
  attr(result, "total") <- resp[["total"]] %||% 0L
  result
}

#' Export herbs to CSV
#'
#' Download a CSV export of herbs matching the given filters.
#'
#' @inheritParams search_herbs
#' @param file Output file path (default `"herbs_export.csv"`).
#' @return Invisible file path.
#' @export
#' @examples
#' \dontrun{
#' export_herbs(q = "ginseng", file = "ginseng_herbs.csv")
#' }
export_herbs <- function(q = NULL, therapeutic_en = NULL,
                         therapeutic_cn = NULL, family = NULL,
                         toxicity = NULL, source = NULL,
                         flavors = NULL, properties = NULL,
                         meridians = NULL, medicinal_part = NULL,
                         file = "herbs_export.csv") {
  query <- list(
    q = q,
    therapeutic_en = collapse_param(therapeutic_en, ";"),
    therapeutic_cn = collapse_param(therapeutic_cn, ";"),
    family = collapse_param(family, ";"),
    toxicity = collapse_param(toxicity, ";"),
    source = collapse_param(source, ";"),
    flavors = collapse_param(flavors, ";"),
    properties = collapse_param(properties, ";"),
    meridians = collapse_param(meridians, ";"),
    medicinal_part = collapse_param(medicinal_part, ";")
  )

  unitcm_download("/herbs/export", query = query, file = file)
}

#' Export herb compounds to CSV
#'
#' Download a CSV export of all compounds for a specific herb.
#'
#' @param herb_id The UniTCM herb ID.
#' @param file Output file path (default `"herb_compounds_export.csv"`).
#' @return Invisible file path.
#' @export
#' @examples
#' \dontrun{
#' export_herb_compounds("UNITCM_H001")
#' }
export_herb_compounds <- function(herb_id,
                                  file = "herb_compounds_export.csv") {
  unitcm_download(
    paste0("/herbs/", normalize_id(herb_id), "/compounds/export"),
    file = file
  )
}

test_that("search_herbs returns tibble with total attr", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(
        items = data.frame(
          unitcm_herb_id = c("H001", "H002"),
          herb_english_name = c("Ginseng", "Astragalus"),
          stringsAsFactors = FALSE
        ),
        total = 100L
      )
    }
  )

  result <- search_herbs(q = "ginseng")
  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 2L)
  expect_equal(attr(result, "total"), 100L)
})

test_that("search_herbs collapses multi-value params with semicolons", {
  captured_query <- NULL
  local_mocked_bindings(
    unitcm_request = function(path, query = list(), ...) {
      captured_query <<- query
      list(items = list(), total = 0L)
    }
  )

  search_herbs(flavors = c("sweet", "bitter"), properties = c("warm"))
  expect_equal(captured_query$flavors, "sweet;bitter")
  expect_equal(captured_query$properties, "warm")
})

test_that("search_herbs handles empty results", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) list(items = list(), total = 0L)
  )

  result <- search_herbs(q = "nonexistent")
  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 0L)
})

test_that("get_herb returns a list", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(unitcm_herb_id = "H001", herb_english_name = "Ginseng",
           flavors = "sweet;slightly bitter")
    }
  )

  result <- get_herb("H001")
  expect_type(result, "list")
  expect_equal(result$unitcm_herb_id, "H001")
})

test_that("fetch_herb_facets returns named list", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(
        toxicity = data.frame(value = c("non-toxic", "toxic"),
                              count = c(100L, 10L),
                              stringsAsFactors = FALSE),
        family = data.frame(value = "Fabaceae", count = 50L,
                            stringsAsFactors = FALSE)
      )
    }
  )

  result <- fetch_herb_facets()
  expect_type(result, "list")
  expect_s3_class(result$toxicity, "tbl_df")
})

test_that("get_herb_compounds returns tibble", {
  local_mocked_bindings(
    unitcm_request = function(path, ...) {
      list(
        items = data.frame(
          unitcm_ingredient_id = "I001",
          component_name = "Ginsenoside",
          stringsAsFactors = FALSE
        ),
        total = 1L
      )
    }
  )

  result <- get_herb_compounds("H001")
  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 1L)
})

#' Extract error message from UniTCM API response
#'
#' Used as the `body` callback in [httr2::req_error()] to provide
#' human-readable error messages from API error responses.
#'
#' @param resp An httr2 response object.
#' @return A single character string describing the error.
#' @noRd
unitcm_error_body <- function(resp) {
  status <- httr2::resp_status(resp)
  body <- tryCatch(
    httr2::resp_body_json(resp),
    error = function(e) list()
  )

  detail <- body[["detail"]] %||% body[["message"]] %||% ""

  switch(as.character(status),
    "400" = glue::glue("Bad request: {detail}"),
    "404" = glue::glue("Resource not found: {detail}"),
    "429" = "Rate limit exceeded. The request will be retried automatically.",
    "500" = "UniTCM server error. Please try again later.",
    glue::glue("HTTP {status}: {detail}")
  )
}

---
title: "Network Pharmacology Workflow"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Network Pharmacology Workflow}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

```{r setup}
library(unitcm)
library(dplyr)
```

This vignette demonstrates a complete network pharmacology workflow: from a
TCM formula to its herb-compound-target (H-C-T) network.

## Step 1: Select a Formula

Start by searching for a formula related to a disease of interest:

```{r select-formula}
formulas <- search_formulas(q = "insomnia", page_size = 5)
formulas

# Get full details for the first formula
formula <- get_formula(formulas$order[1])
formula$formula_name
formula$composition
```

## Step 2: Get Herb Composition

```{r get-doses}
doses <- get_formula_doses(formulas$order[1])
doses

# Extract herb names
herb_names <- doses$herb_name
herb_names
```

## Step 3: Find Herb IDs

Search for each herb to get its UniTCM ID:

```{r find-herbs}
herb_data <- lapply(herb_names, function(name) {
  result <- search_herbs(q = name, page_size = 1)
  if (nrow(result) > 0) result[1, ]
})
herb_data <- bind_rows(herb_data)
herb_data
```

## Step 4: Get Compounds for Each Herb

```{r get-compounds}
all_compounds <- lapply(herb_data$unitcm_herb_id, function(hid) {
  comps <- get_herb_compounds(hid, all_pages = TRUE)
  if (nrow(comps) > 0) comps$herb_id <- hid
  comps
})
all_compounds <- bind_rows(all_compounds)

# Unique compounds
unique_compounds <- distinct(all_compounds, unitcm_ingredient_id,
                             .keep_all = TRUE)
cat(nrow(unique_compounds), "unique compounds found\n")
```

## Step 5: Get Targets for Key Compounds

```{r get-targets}
# Get targets for top compounds (limit to first 10 for speed)
top_compounds <- head(unique_compounds, 10)

all_targets <- lapply(top_compounds$unitcm_ingredient_id, function(cid) {
  targets <- get_compound_targets(cid, method = "drugclip")
  if (nrow(targets) > 0) targets$compound_id <- cid
  targets
})
all_targets <- bind_rows(all_targets)
cat(length(unique(all_targets$gene_symbol)), "unique targets found\n")
```

## Step 6: Build Network with igraph

```{r build-network}
library(igraph)

# Herb-Compound edges
hc_edges <- all_compounds |>
  select(from = herb_id, to = unitcm_ingredient_id) |>
  distinct()

# Compound-Target edges
ct_edges <- all_targets |>
  select(from = compound_id, to = gene_symbol) |>
  distinct()

# Combine edges
edges <- bind_rows(hc_edges, ct_edges)

# Build graph
g <- graph_from_data_frame(edges, directed = FALSE)

# Add node type attribute
V(g)$type <- case_when(
  V(g)$name %in% herb_data$unitcm_herb_id ~ "herb",
  V(g)$name %in% unique_compounds$unitcm_ingredient_id ~ "compound",
  TRUE ~ "target"
)

cat("Network:", vcount(g), "nodes,", ecount(g), "edges\n")
```

## Step 7: Visualize with ggraph

```{r plot-network, fig.width = 10, fig.height = 8}
library(ggraph)

ggraph(g, layout = "fr") +
  geom_edge_link(alpha = 0.3) +
  geom_node_point(aes(color = type, size = type)) +
  scale_color_manual(values = c(herb = "#e74c3c", compound = "#3498db",
                                target = "#2ecc71")) +
  scale_size_manual(values = c(herb = 6, compound = 3, target = 2)) +
  geom_node_text(
    aes(label = ifelse(type == "herb", name, "")),
    repel = TRUE, size = 3
  ) +
  theme_void() +
  labs(title = paste("H-C-T Network:", formula$formula_name))
```

## Step 8: Pathway Enrichment via MIDAS

Use the identified targets for disease enrichment analysis:

```{r enrichment}
target_genes <- unique(all_targets$gene_symbol)
enrichment <- query_disease_enrichment(
  target_genes,
  p_value_cutoff = 0.05,
  correction_method = "fdr"
)
head(enrichment, 10)
```

---
title: "Gene-Disease Analysis with MIDAS"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Gene-Disease Analysis with MIDAS}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

```{r setup}
library(unitcm)
library(dplyr)
```

MIDAS (Multi-source Integrated Disease Association Search) aggregates
gene-disease associations from multiple databases. This vignette demonstrates
common analysis workflows.

## Data Sources

See what databases are available:

```{r sources}
sources <- fetch_midas_sources()
sources

stats <- fetch_midas_stats()
cat(stats$total_associations, "associations across",
    stats$total_genes, "genes and",
    stats$total_diseases, "diseases\n")
```

## Gene-to-Disease Mapping

Find diseases associated with a gene list:

```{r gene-to-disease}
genes <- c("TP53", "BRCA1", "EGFR", "VEGFA", "MYC")

results <- query_gene_diseases(
  genes,
  min_sources = 2,
  scoring_method = "max"
)

head(results, 10)

# Gene ID resolution mapping
attr(results, "gene_mapping")
```

## Disease-to-Gene Mapping

Find genes associated with a disease:

```{r disease-to-gene}
results <- query_disease_genes(
  "breast cancer",
  min_sources = 3,
  page_size = 50
)

head(results, 10)

# Which diseases were matched?
attr(results, "matched_diseases")
```

## Disease Enrichment Analysis

Test whether a gene list is significantly enriched for specific diseases:

```{r enrichment}
gene_list <- c("TP53", "BRCA1", "EGFR", "VEGFA", "MYC", "KRAS",
               "AKT1", "PIK3CA", "PTEN", "RB1")

enrichment <- query_disease_enrichment(
  gene_list,
  p_value_cutoff = 0.05,
  correction_method = "fdr",
  min_hit_count = 3
)

cat(attr(enrichment, "total_significant"), "significant diseases from",
    attr(enrichment, "total_tested"), "tested\n")

head(enrichment, 10)
```

## Gene ID Conversion

Normalize mixed identifiers before analysis:

```{r convert}
mixed_ids <- c("TP53", "7157", "ENSG00000141510", "BRCA1")
converted <- convert_gene_ids(mixed_ids)
converted
```

## Source Comparison

Compare coverage across evidence databases:

```{r source-comparison}
comparison <- query_source_comparison(
  c("TP53", "BRCA1", "EGFR"),
  mode = "union"
)

# Genes covered by each source
lapply(comparison$sets, length)

# Exclusive to each source
comparison$exclusives
```

## Disease Intersection

Find shared genetic targets across diseases:

```{r disease-intersection}
intersection <- query_disease_intersection(
  c("breast cancer", "lung cancer", "colorectal cancer")
)

cat(intersection$total_intersection_genes, "genes shared across all diseases\n")
head(intersection$targets)
```

## Disease Autocomplete

Find disease names interactively:

```{r autocomplete}
autocomplete_disease("diabet")
autocomplete_disease("breast")
```

---
title: "Target2NP: Compound–Target Interactions"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Target2NP: Compound–Target Interactions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

```{r setup}
library(unitcm)
```

The **Target2NP** module provides access to a large-scale compound–target
interaction database covering multiple experimental sources (BindingDB,
HERB2, NPASS, BATMAN, etc.) as well as computational predictions from
DrugCLIP (deep learning) and SEA (ChEMBL similarity). This vignette
walks through the main workflows.

## Experimental interactions

### Search and filter

```{r search-basic}
# Free-text search across all fields
hits <- search_target2np(search = "quercetin")
hits

# Exact-match by gene symbol
tp53 <- search_target2np(
  search = "TP53",
  search_field = "gene_symbol",
  search_mode = "exact"
)
tp53
attr(tp53, "total")
```

```{r search-filters}
# Combine filters
results <- search_target2np(
  search = "curcumin",
  search_field = "compound_name",
  source_db = "BindingDB",
  target_organism = "Homo sapiens",
  activity_type = "IC50"
)
results
```

### Available filters and statistics

```{r filter-options}
# What filter values exist?
opts <- fetch_target2np_filters()
opts$source_db
opts$target_organism
opts$activity_type

# Global database statistics
stats <- fetch_target2np_stats()
stats$total_records
stats$source_db_distribution
```

### Retrieve a single record

```{r detail}
detail <- get_target2np(1)
detail$compound_name
detail$gene_symbol
detail$activity_value
detail$activity_units
detail$pmid
```

### Batch query

Look up interactions for multiple genes in one call (up to 50
identifiers):

```{r batch}
batch <- batch_target2np(c("TP53", "BRCA1", "EGFR", "VEGFA"))
batch
attr(batch, "queries_matched")
attr(batch, "queries_not_found")

# UniProt-based batch
batch_up <- batch_target2np(
  c("P04637", "P38398"),
  id_type = "uniprot_id"
)
```

## Computational predictions

### DrugCLIP deep-learning predictions

```{r drugclip}
# High-confidence predictions for quercetin
dc_high <- search_target2np_drugclip(
  search = "quercetin",
  search_field = "compound_name",
  confidence = "high"
)
dc_high

# Score-based filtering
dc <- search_target2np_drugclip(
  search = "EGFR",
  search_field = "gene_symbol",
  min_score = 0.7
)
dc
```

### SEA similarity-based predictions

```{r sea}
# High-confidence SEA predictions
sea_high <- search_target2np_sea(
  search = "quercetin",
  search_field = "compound_name",
  confidence = "high"
)
sea_high

# Filter by adjusted p-value
sea <- search_target2np_sea(
  search = "TP53",
  search_field = "gene_symbol",
  max_pvalue = 0.01
)
sea
```

## Cross-source analysis

### Multi-source summary

The `target2np_multi_source_summary()` function queries experimental
records, DrugCLIP, and SEA for the same term and returns an integrated
overview: source counts, overlap statistics, confidence distributions,
and cross-validated compound–target pairs.

```{r multi-source}
summary <- target2np_multi_source_summary(
  search = "TP53",
  search_field = "gene_symbol"
)

# How many results per source?
summary$source_counts

# Target overlap across data sources
summary$target_overlap

# Confidence-level distribution for each source
summary$confidence_distribution

# Compound-target pairs found in >= 2 sources
summary$cross_validated

# Natural-language interpretation
cat(summary$suggestion_text)
```

### Aggregated view

The aggregated view groups experimental interaction records by
compound–target pair (InChIKey + UniProt ID) and returns pairs supported
by multiple source databases. This is useful for identifying
well-evidenced interactions.

```{r aggregated}
# Pairs seen in >= 3 databases
agg <- aggregated_target2np(
  search = "quercetin",
  min_sources = 3
)
agg

# Include DrugCLIP/SEA prediction counts as additional sources
agg_pred <- aggregated_target2np(
  search = "quercetin",
  min_sources = 2,
  include_predictions = TRUE
)
agg_pred
```

## Practical example: multi-evidence target prioritisation

A common workflow is to start with a compound of interest, query all
three data sources, and use cross-validation to prioritise targets.

```{r workflow}
# 1. Check experimental evidence
exp <- search_target2np(
  search = "quercetin",
  search_field = "compound_name",
  search_mode = "fuzzy",
  all_pages = TRUE
)
nrow(exp)

# 2. Get multi-source summary in one call
ms <- target2np_multi_source_summary(
  search = "quercetin",
  search_field = "compound_name",
  search_mode = "fuzzy"
)
ms$source_counts
ms$cross_validated

# 3. Batch-check the top cross-validated targets
top_genes <- unique(vapply(
  ms$cross_validated, `[[`, character(1), "gene_symbol"
))
if (length(top_genes) > 0) {
  batch_detail <- batch_target2np(top_genes)
  batch_detail
}
```

---
title: "Database Queries"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Database Queries}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

```{r setup}
library(unitcm)
```

This vignette demonstrates how to query each database module in the
UniTCM platform.

## Herb Explorer

### Search and filter herbs

```{r herbs-search}
# Text search
herbs <- search_herbs(q = "ginseng")

# Multi-value faceted filters
herbs <- search_herbs(
  flavors = c("sweet", "bitter"),
  properties = "warm",
  toxicity = "non-toxic"
)

# View available filter values
facets <- fetch_herb_facets()
facets$toxicity
facets$meridians
```

### Herb details and compounds

UniTCM IDs are flexible: all `get_*` and `export_*` functions accept either
the prefixed display form shown on the website (e.g. `"UNITCM_H001"`,
`"UNITCM_I00001"`) or the bare numeric ID (e.g. `"1"`, `1L`). The package
normalizes them internally before calling the API.

```{r herbs-detail}
herb <- get_herb("UNITCM_H001")
herb$herb_english_name
herb$efficacy

# Get compounds found in this herb
compounds <- get_herb_compounds("UNITCM_H001", all_pages = TRUE)
head(compounds)
```

### Export data

```{r herbs-export}
export_herbs(q = "ginseng", file = "ginseng_herbs.csv")
export_herb_compounds("UNITCM_H001", file = "ginseng_compounds.csv")
```

## Ingredient Explorer

### Search compounds by properties

```{r compounds-search}
# Drug-like compounds with molecular weight 200-500
compounds <- search_compounds(
  mw_min = 200, mw_max = 500,
  lipinski = "pass",
  is_drug = TRUE
)

# Get facet statistics
facets <- fetch_compound_facets()
facets$mw_range
```

### Compound details, ADMET, and targets

```{r compounds-detail}
# Full compound record
compound <- get_compound("UNITCM_I00001")
compound$component_name
compound$xref$pubchem_cid

# ADMET predictions (~90 endpoints)
admet <- get_compound_admet("UNITCM_I00001")
admet$caco2_permeability
admet$hia

# Predicted targets (DrugCLIP deep learning)
targets_dc <- get_compound_targets("UNITCM_I00001", method = "drugclip")

# ChEMBL similarity-based targets
targets_ch <- get_compound_targets("UNITCM_I00001", method = "chembl")

# Both sources combined
targets_all <- get_compound_targets("UNITCM_I00001", method = "both")
table(targets_all$source)
```

## Disease-Formula Atlas

### Search formulas by disease

```{r formulas-search}
# Search by text
formulas <- search_formulas(q = "insomnia")

# Filter by ICD-11 classification
formulas <- search_formulas(
  level1 = "Neoplasms",
  mapping_confidence = c("high", "medium")
)

# Browse the disease classification tree
tree <- fetch_disease_tree()

# Available filter options
list_book_sources()
list_origin_sources()
list_dosage_forms()
```

### Formula details and doses

```{r formulas-detail}
formula <- get_formula(1)
formula$formula_name
formula$efficacy

# Herb composition and dosage
doses <- get_formula_doses(1)
doses
```

## TCM Ontology

### Search and navigate the ontology

```{r ontology-search}
# Full-text search
results <- search_ontology("Qi stagnation")

# Get entity with relations
entity <- get_ontology_entity("TCM_0001")
entity$ancestors
entity$children
entity$external_mappings

# Navigate the hierarchy
children <- get_ontology_children("TCM_0001")
ancestors <- get_ontology_ancestors("TCM_0001")
descendants <- get_ontology_descendants("TCM_0001", max_level = 2)
```

### Ontology tree and statistics

```{r ontology-tree}
# Fetch tree at depth 2
tree <- fetch_ontology_tree(depth = 2)

# Statistics
stats <- fetch_ontology_stats()
stats$total_entities

# Top-level categories
categories <- list_ontology_categories()
```

### Cross-database mapping

```{r ontology-mapping}
# Find TCM entities mapped to a MeSH term
mapped <- search_ontology_mapping("MeSH", "D008516")

# Export ontology
export_ontology("csv", file = "tcm_ontology.csv")
export_ontology("json", depth = 3, file = "tcm_ontology.json")
```

## MIDAS Gene-Disease Analysis

### Gene-to-disease and disease-to-gene

```{r midas-basic}
# Which diseases are associated with these genes?
g2d <- query_gene_diseases(c("TP53", "BRCA1", "EGFR"))
g2d
attr(g2d, "gene_mapping")

# Which genes are associated with breast cancer?
d2g <- query_disease_genes("breast cancer")
d2g
attr(d2g, "matched_diseases")
```

### Disease enrichment analysis

```{r midas-enrichment}
gene_list <- c("TP53", "BRCA1", "EGFR", "VEGFA", "MYC", "KRAS")
enrichment <- query_disease_enrichment(
  gene_list,
  p_value_cutoff = 0.05,
  correction_method = "fdr"
)
enrichment
attr(enrichment, "total_significant")
```

### Utility functions

```{r midas-utils}
# Convert mixed gene identifiers
convert_gene_ids(c("TP53", "7157", "ENSG00000141510"))

# Autocomplete disease names
autocomplete_disease("breast")

# Available data sources
fetch_midas_sources()

# Database statistics
fetch_midas_stats()
```

---
title: "Getting Started with unitcm"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with unitcm}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

## Installation

Install the development version from GitHub:

```{r install}
# install.packages("pak")
pak::pkg_install("zx122ty/UniTCM_R_Package")
```

Or using `remotes`:

```{r install-remotes}
remotes::install_github("zx122ty/UniTCM_R_Package")
```

## Setup

```{r setup}
library(unitcm)
```

By default, `unitcm` connects to the public UniTCM API at
`https://unitcm.qfxulab.com/api/v1`. You can override this if needed:

```{r base-url}
# Only needed for custom/local deployments
set_base_url("https://unitcm.qfxulab.com/api/v1")
```

## Authentication

All data-access endpoints are currently public and require no authentication.
For future authenticated endpoints, you can set a token:
```{r auth}
# Via environment variable (recommended for scripts/CI)
# Set UNITCM_TOKEN in your .Renviron

# Or set in session
set_unitcm_token("your-token-here")

# Or store securely in system keyring
set_unitcm_token("your-token-here", keyring = TRUE)
```

## Your First Query

Search for herbs related to ginseng:

```{r first-query}
herbs <- search_herbs(q = "ginseng")
herbs
```

Get detailed information for a specific herb:

```{r herb-detail}
herb <- get_herb("UNITCM_H001")
herb$herb_english_name
herb$efficacy
```

## Pagination

Most search functions return paginated results. Use `page` and `page_size`
to control pagination manually, or set `all_pages = TRUE` to fetch
everything:

```{r pagination}
# Manual pagination
page1 <- search_herbs(q = "ginseng", page = 1, page_size = 50)
attr(page1, "total") # Total records available

# Auto-pagination: fetches all pages with a progress bar
all_herbs <- search_herbs(q = "ginseng", all_pages = TRUE)
nrow(all_herbs)
```

## Available Modules

`unitcm` provides access to these UniTCM platform modules:

| Module | Key Functions |
|--------|--------------|
| Herb Explorer | `search_herbs()`, `get_herb()`, `get_herb_compounds()` |
| Ingredient Explorer | `search_compounds()`, `get_compound_admet()`, `get_compound_targets()` |
| Disease-Formula Atlas | `search_formulas()`, `get_formula_doses()`, `fetch_disease_tree()` |
| TCM Ontology | `search_ontology()`, `get_ontology_entity()`, `export_ontology()` |
| MIDAS Gene-Disease | `query_gene_diseases()`, `query_disease_enrichment()` |

See `vignette("database-queries")` for detailed examples of each module.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ingredients.R
\name{search_compounds}
\alias{search_compounds}
\title{Search compounds in the Ingredient Explorer}
\usage{
search_compounds(
  q = NULL,
  mw_min = NULL,
  mw_max = NULL,
  clogp_min = NULL,
  clogp_max = NULL,
  tpsa_min = NULL,
  tpsa_max = NULL,
  qed_min = NULL,
  qed_max = NULL,
  ring_count_min = NULL,
  ring_count_max = NULL,
  lipinski = NULL,
  is_drug = NULL,
  sort = NULL,
  page = 1L,
  page_size = 20L,
  all_pages = FALSE
)
}
\arguments{
\item{q}{Search query (name, SMILES, formula, or CAS number).}

\item{mw_min, mw_max}{Molecular weight range.}

\item{clogp_min, clogp_max}{CLogP range.}

\item{tpsa_min, tpsa_max}{Topological polar surface area range.}

\item{qed_min, qed_max}{QED score range.}

\item{ring_count_min, ring_count_max}{Ring count range.}

\item{lipinski}{Lipinski rule filter (character vector, comma-collapsed).}

\item{is_drug}{Approved drug filter (logical or \code{NULL}).}

\item{sort}{Sort field (e.g. \code{"mw"}, \code{"-mw"}).}

\item{page}{Page number (default 1).}

\item{page_size}{Results per page (default 20, max 200).}

\item{all_pages}{If \code{TRUE}, fetch all pages.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of compounds with attribute \code{"total"}.
}
\description{
Query the UniTCM Ingredient Explorer with optional text search and
physicochemical property filters.
}
\examples{
\dontrun{
search_compounds(q = "quercetin")
search_compounds(mw_min = 200, mw_max = 500, lipinski = "pass")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/midas.R
\name{query_disease_genes}
\alias{query_disease_genes}
\title{Query disease-to-gene associations (MIDAS)}
\usage{
query_disease_genes(
  disease_query,
  disease_id_type = "name",
  sources = NULL,
  min_sources = 1L,
  scoring_method = "max",
  page = 1L,
  page_size = 20L
)
}
\arguments{
\item{disease_query}{Disease name or ID.}

\item{disease_id_type}{ID type: \code{"name"} (default) or \code{"icd11"}.}

\item{sources}{Character vector of source databases (or \code{NULL} for all).}

\item{min_sources}{Minimum supporting sources (default 1).}

\item{scoring_method}{Scoring method: \code{"max"} (default) or \code{"mean"}.}

\item{page}{Page number (default 1).}

\item{page_size}{Results per page (default 20).}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of disease-gene associations with attribute
\code{"matched_diseases"}.
}
\description{
Given a disease query, find associated genes across multiple evidence
sources.
}
\examples{
\dontrun{
query_disease_genes("breast cancer")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ingredients.R
\name{get_compound_admet}
\alias{get_compound_admet}
\title{Get ADMET predictions for a compound}
\usage{
get_compound_admet(id)
}
\arguments{
\item{id}{The UniTCM ingredient ID.}
}
\value{
A single-row \code{\link[tibble:tibble]{tibble::tibble()}} with ~90 ADMET columns.
}
\description{
Returns ~90 ADMET endpoint predictions as a single-row wide tibble.
}
\examples{
\dontrun{
get_compound_admet("UNITCM_I00001")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\name{build_formula_herb_network}
\alias{build_formula_herb_network}
\title{Build a Formula-Herb network}
\usage{
build_formula_herb_network(formula_id)
}
\arguments{
\item{formula_id}{The formula order ID (integer or character).}
}
\value{
An \code{igraph} graph object with vertex attributes \code{name}, \code{type}
(\code{"formula"}, \code{"herb"}), \code{label}, and \code{dose} (for herbs).
}
\description{
Given a formula order ID, fetches its herb doses and constructs a
star-topology network.
}
\examples{
\dontrun{
g <- build_formula_herb_network(1)
igraph::V(g)$label
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/browse.R
\name{get_dataset}
\alias{get_dataset}
\title{Get a single dataset by submission ID}
\usage{
get_dataset(submission_id)
}
\arguments{
\item{submission_id}{The submission ID (e.g. \code{"TMA2025001"}).}
}
\value{
A named list with nested sub-lists for \code{persons}, \code{publications},
\code{grants}, and \code{data_files}.
}
\description{
Retrieve full detail including nested persons, publications, grants,
and data files.
}
\examples{
\dontrun{
get_dataset("TMA2025001")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_network}
\alias{plot_network}
\title{Plot a network graph}
\usage{
plot_network(
  graph,
  layout = "fr",
  color_by = "type",
  node_size = 3,
  edge_alpha = 0.3,
  show_labels = NULL
)
}
\arguments{
\item{graph}{An \code{igraph} or \code{tidygraph::tbl_graph} object.}

\item{layout}{Layout algorithm (default \code{"fr"} for Fruchterman-Reingold).
See \code{\link[ggraph:create_layout]{ggraph::create_layout()}} for options.}

\item{color_by}{Vertex attribute name to map to node color (default
\code{"type"}).}

\item{node_size}{Base node size (default 3).}

\item{edge_alpha}{Edge transparency (default 0.3).}

\item{show_labels}{Whether to label nodes (default \code{TRUE} for nodes
with degree >= 3, or all if graph has <= 50 nodes).}
}
\value{
A \code{ggplot} object.
}
\description{
Visualize an \code{igraph} or \code{tbl_graph} object using \code{ggraph}.
}
\examples{
\dontrun{
g <- build_hct_network("UNITCM_H001")
plot_network(g)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netvis.R
\name{search_netvis}
\alias{search_netvis}
\title{Search NetVis nodes}
\usage{
search_netvis(q, type = "all", limit = 20L)
}
\arguments{
\item{q}{Search query.}

\item{type}{Node type filter: \code{"all"} (default), \code{"formula"}, \code{"herb"},
\code{"compound"}, \code{"target"}, or \code{"disease"}.}

\item{limit}{Maximum results (default 20).}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} with columns: \code{id}, \code{type}, \code{label},
\code{label_cn}, \code{degree}.
}
\description{
Search NetVis nodes
}
\examples{
\dontrun{
search_netvis("ginseng", type = "herb")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/target2np.R
\name{fetch_target2np_filters}
\alias{fetch_target2np_filters}
\title{Fetch Target2NP filter options}
\usage{
fetch_target2np_filters()
}
\value{
A named list with fields \code{source_db}, \code{evidence_label},
\code{target_organism}, \code{interaction_type}, \code{activity_type}.
}
\description{
Returns the controlled values used by the Target2NP filter UI
(source databases, evidence labels, top target organisms, interaction
types, and top activity types).
}
\examples{
\dontrun{
opts <- fetch_target2np_filters()
opts$source_db
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/target2np.R
\name{aggregated_target2np}
\alias{aggregated_target2np}
\title{Aggregated Target2NP view across data sources}
\usage{
aggregated_target2np(
  search = NULL,
  target_organism = NULL,
  min_sources = 2L,
  include_predictions = FALSE,
  page = 1L,
  page_size = 20L
)
}
\arguments{
\item{search}{Free-text search query.}

\item{target_organism}{Optional target organism filter.}

\item{min_sources}{Minimum number of source databases supporting the
pair (1-5, default 2).}

\item{include_predictions}{If \code{TRUE}, also count DrugCLIP and SEA
predictions as additional supporting sources.}

\item{page}{Page number (default 1).}

\item{page_size}{Results per page (default 20, max 50).}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of aggregated pairs with attribute \code{"total"}.
}
\description{
Return compound-target pairs (keyed by InChIKey + UniProt ID) supported
by interaction records in at least \code{min_sources} source databases.
Optionally extends each pair with DrugCLIP / SEA prediction support.
}
\examples{
\dontrun{
aggregated_target2np(search = "quercetin")
aggregated_target2np(search = "TP53", min_sources = 3,
                     include_predictions = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netvis.R
\name{fetch_netvis_stats}
\alias{fetch_netvis_stats}
\title{Get NetVis network statistics}
\usage{
fetch_netvis_stats()
}
\value{
A named list with node counts (\code{formula}, \code{herb}, \code{compound},
\code{target}, \code{disease}) and \code{edges} sub-list.
}
\description{
Get NetVis network statistics
}
\examples{
\dontrun{
fetch_netvis_stats()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auth.R
\name{get_unitcm_token}
\alias{get_unitcm_token}
\title{Get the UniTCM API token}
\usage{
get_unitcm_token()
}
\value{
A character string, or \code{NULL} if no token is found.
}
\description{
Checks in order: (1) session value set via \code{\link[=set_unitcm_token]{set_unitcm_token()}},
(2) environment variable \code{UNITCM_TOKEN}, (3) system keyring.
}
\examples{
\dontrun{
get_unitcm_token()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/target2np.R
\name{get_target2np}
\alias{get_target2np}
\title{Get a single Target2NP interaction record}
\usage{
get_target2np(record_id)
}
\arguments{
\item{record_id}{Integer record ID.}
}
\value{
A named list of interaction fields.
}
\description{
Retrieve the full detail for one compound-target interaction record.
}
\examples{
\dontrun{
get_target2np(1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/home.R
\name{fetch_tcm_classification_stats}
\alias{fetch_tcm_classification_stats}
\title{Get TCM classification statistics}
\usage{
fetch_tcm_classification_stats()
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} with columns: \code{classification}, \code{count},
\code{percentage}.
}
\description{
Get TCM classification statistics
}
\examples{
\dontrun{
fetch_tcm_classification_stats()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tcm-term.R
\name{search_terms}
\alias{search_terms}
\title{Search TCM bilingual corpus terms}
\usage{
search_terms(
  q = NULL,
  sources = NULL,
  category = NULL,
  page = 1L,
  page_size = 20L,
  all_pages = FALSE
)
}
\arguments{
\item{q}{Optional search query string.}

\item{sources}{Data source filter (character vector, comma-collapsed).}

\item{category}{Category filter (character vector, comma-collapsed).}

\item{page}{Page number (default 1).}

\item{page_size}{Results per page (default 20, max 100).}

\item{all_pages}{If \code{TRUE}, fetch all pages via auto-pagination.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of terms with attribute \code{"total"}.
}
\description{
Query the TCM Bilingual Corpus with optional text search and filters.
}
\examples{
\dontrun{
search_terms(q = "ginseng")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/target2np.R
\name{target2np_multi_source_summary}
\alias{target2np_multi_source_summary}
\title{Multi-source summary for a Target2NP query}
\usage{
target2np_multi_source_summary(
  search = NULL,
  sources = c("experimental", "drugclip", "sea"),
  search_field = c("all", "gene_symbol", "compound_name", "uniprot_id", "inchikey",
    "pubchem_cid", "chembl_id"),
  search_mode = c("exact", "fuzzy")
)
}
\arguments{
\item{search}{Free-text search query.}

\item{sources}{Character vector of sources to query. Subset of
\code{c("experimental", "drugclip", "sea")} (default: all three).}

\item{search_field}{One of \code{"all"}, \code{"gene_symbol"}, \code{"compound_name"},
\code{"uniprot_id"}, \code{"inchikey"}, \code{"pubchem_cid"}, \code{"chembl_id"}.}

\item{search_mode}{\code{"exact"} or \code{"fuzzy"}.}
}
\value{
A named list with fields \code{source_counts}, \code{target_overlap},
\code{compound_overlap}, \code{confidence_distribution}, \code{cross_validated},
and \code{suggestion_text}.
}
\description{
Combine experimental records, DrugCLIP predictions, and SEA predictions
for one query, returning source counts, target/compound overlap sets,
confidence distributions, cross-validated compound-target pairs, and
an interpretive suggestion string.
}
\examples{
\dontrun{
summary <- target2np_multi_source_summary(
  search = "TP53", search_field = "gene_symbol"
)
summary$source_counts
summary$suggestion_text
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/herbs.R
\name{search_herbs}
\alias{search_herbs}
\title{Search herbs in the Herb Explorer}
\usage{
search_herbs(
  q = NULL,
  therapeutic_en = NULL,
  therapeutic_cn = NULL,
  family = NULL,
  toxicity = NULL,
  source = NULL,
  flavors = NULL,
  properties = NULL,
  meridians = NULL,
  medicinal_part = NULL,
  page = 1L,
  page_size = 20L,
  all_pages = FALSE
)
}
\arguments{
\item{q}{Optional search query string.}

\item{therapeutic_en}{English therapeutic classification filter (character
vector).}

\item{therapeutic_cn}{Chinese therapeutic classification filter (character
vector).}

\item{family}{Botanical family filter (character vector).}

\item{toxicity}{Toxicity level filter (character vector).}

\item{source}{Data source filter (character vector).}

\item{flavors}{Flavor filter (character vector).}

\item{properties}{Property filter (character vector).}

\item{meridians}{Meridian tropism filter (character vector).}

\item{medicinal_part}{Medicinal part filter (character vector).}

\item{page}{Page number (default 1).}

\item{page_size}{Results per page (default 20, max 200).}

\item{all_pages}{If \code{TRUE}, fetch all pages via auto-pagination.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of herbs with attribute \code{"total"}.
}
\description{
Query the UniTCM Herb Explorer with optional text search and faceted
filters. Multi-value filter parameters accept character vectors and are
collapsed to semicolon-separated strings internally.
}
\examples{
\dontrun{
search_herbs(q = "ginseng")
search_herbs(flavors = c("sweet", "bitter"), page_size = 50)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transcriptome-hub.R
\name{get_analysis_modules}
\alias{get_analysis_modules}
\title{List available analysis modules for a dataset}
\usage{
get_analysis_modules(dataset_id)
}
\arguments{
\item{dataset_id}{The dataset ID.}
}
\value{
A character vector of available module names.
}
\description{
List available analysis modules for a dataset
}
\examples{
\dontrun{
get_analysis_modules("TCMtrans00001")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/midas.R
\name{query_gene_diseases}
\alias{query_gene_diseases}
\title{Query gene-to-disease associations (MIDAS)}
\usage{
query_gene_diseases(
  gene_list,
  gene_id_type = "symbol",
  sources = NULL,
  min_sources = 1L,
  min_score = 0,
  evidence_types = NULL,
  scoring_method = "max",
  page = 1L,
  page_size = 20L
)
}
\arguments{
\item{gene_list}{Character vector of gene identifiers.}

\item{gene_id_type}{ID type: \code{"symbol"} (default), \code{"entrez"}, or
\code{"ensembl"}.}

\item{sources}{Character vector of source databases to query (or \code{NULL}
for all).}

\item{min_sources}{Minimum number of sources supporting an association
(default 1).}

\item{min_score}{Minimum association score (default 0).}

\item{evidence_types}{Character vector of evidence types to filter by.}

\item{scoring_method}{Scoring method: \code{"max"} (default) or \code{"mean"}.}

\item{page}{Page number (default 1).}

\item{page_size}{Results per page (default 20).}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of gene-disease associations with attribute
\code{"gene_mapping"} containing the gene ID resolution mapping.
}
\description{
Given a list of gene symbols or IDs, find associated diseases across
multiple evidence sources.
}
\examples{
\dontrun{
query_gene_diseases(c("TP53", "BRCA1"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_compound_radar}
\alias{plot_compound_radar}
\title{Plot compound physicochemical radar chart}
\usage{
plot_compound_radar(
  compound_id,
  properties = c("mw", "clogp", "tpsa", "hbd", "hba", "qed_score")
)
}
\arguments{
\item{compound_id}{The UniTCM ingredient ID.}

\item{properties}{Character vector of property names to plot (default:
key drug-likeness properties).}
}
\value{
A \code{ggplot} object.
}
\description{
Create a radar/spider chart of normalized physicochemical properties
for a compound.
}
\examples{
\dontrun{
plot_compound_radar("UNITCM_I00001")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netvis.R
\name{get_subgraph}
\alias{get_subgraph}
\title{Get subgraph for a set of nodes}
\usage{
get_subgraph(node_ids, limit = 200L)
}
\arguments{
\item{node_ids}{Character vector of node IDs (max 50).}

\item{limit}{Maximum edges (default 200).}
}
\value{
A named list with \verb{$nodes} (\code{\link[tibble:tibble]{tibble::tibble()}}), \verb{$edges}
(\code{\link[tibble:tibble]{tibble::tibble()}}), and \verb{$has_more}.
}
\description{
Get subgraph for a set of nodes
}
\examples{
\dontrun{
get_subgraph(c("H:UNITCM_H001", "C:UNITCM_I00001"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ontology.R
\name{fetch_ontology_stats}
\alias{fetch_ontology_stats}
\title{Fetch ontology statistics}
\usage{
fetch_ontology_stats()
}
\value{
A named list with fields: \code{total_entities}, \code{total_level1}--
\code{total_level4}, \code{total_relations}, \code{total_mappings}, \code{categories}.
}
\description{
Fetch ontology statistics
}
\examples{
\dontrun{
fetch_ontology_stats()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/terms-mechanisms.R
\name{get_mechanism}
\alias{get_mechanism}
\title{Get a single mechanism term by ID}
\usage{
get_mechanism(term_id)
}
\arguments{
\item{term_id}{The mechanism term ID.}
}
\value{
A named list with ~50 fields. Nested arrays (e.g. \code{biomarkers},
\code{signaling_pathways}, \code{gene_targets}) are returned as-is (lists).
}
\description{
Retrieve full detail for one term including nested arrays of biomarkers,
pathways, gene targets, metabolites, etc.
}
\examples{
\dontrun{
get_mechanism("TMM001")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/target2np.R
\name{search_target2np_sea}
\alias{search_target2np_sea}
\title{Search SEA (ChEMBL similarity) predicted compound-target interactions}
\usage{
search_target2np_sea(
  search = NULL,
  search_field = c("all", "gene_symbol", "compound_name", "uniprot_id", "inchikey"),
  search_mode = c("exact", "fuzzy"),
  max_pvalue = NULL,
  confidence = NULL,
  page = 1L,
  page_size = 20L,
  all_pages = FALSE
)
}
\arguments{
\item{search}{Free-text search.}

\item{search_field}{One of \code{"all"}, \code{"gene_symbol"}, \code{"compound_name"},
\code{"uniprot_id"}, or \code{"inchikey"}.}

\item{search_mode}{\code{"exact"} or \code{"fuzzy"}.}

\item{max_pvalue}{Maximum adjusted p-value.}

\item{confidence}{One of \code{"high"} (adj. p < 0.01), \code{"medium"}
(0.01-0.05), \code{"low"} (>= 0.05).}

\item{page, page_size}{Pagination.}

\item{all_pages}{If \code{TRUE}, fetch all pages.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of SEA predictions with attribute \code{"total"}.
}
\description{
Query SEA-style predictions derived from ChEMBL similarity scoring,
with optional adjusted p-value filtering.
}
\examples{
\dontrun{
search_target2np_sea(search = "quercetin", confidence = "high")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auth.R
\name{clear_api_key}
\alias{clear_api_key}
\title{Clear the UniTCM API Key}
\usage{
clear_api_key(keyring = FALSE)
}
\arguments{
\item{keyring}{Logical. If \code{TRUE}, also remove from system keyring.}
}
\value{
Invisible \code{NULL}.
}
\description{
Removes the API key from session memory and optionally from the system
keyring.
}
\examples{
\dontrun{
clear_api_key()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/herbs.R
\name{export_herbs}
\alias{export_herbs}
\title{Export herbs to CSV}
\usage{
export_herbs(
  q = NULL,
  therapeutic_en = NULL,
  therapeutic_cn = NULL,
  family = NULL,
  toxicity = NULL,
  source = NULL,
  flavors = NULL,
  properties = NULL,
  meridians = NULL,
  medicinal_part = NULL,
  file = "herbs_export.csv"
)
}
\arguments{
\item{q}{Optional search query string.}

\item{therapeutic_en}{English therapeutic classification filter (character
vector).}

\item{therapeutic_cn}{Chinese therapeutic classification filter (character
vector).}

\item{family}{Botanical family filter (character vector).}

\item{toxicity}{Toxicity level filter (character vector).}

\item{source}{Data source filter (character vector).}

\item{flavors}{Flavor filter (character vector).}

\item{properties}{Property filter (character vector).}

\item{meridians}{Meridian tropism filter (character vector).}

\item{medicinal_part}{Medicinal part filter (character vector).}

\item{file}{Output file path (default \code{"herbs_export.csv"}).}
}
\value{
Invisible file path.
}
\description{
Download a CSV export of herbs matching the given filters.
}
\examples{
\dontrun{
export_herbs(q = "ginseng", file = "ginseng_herbs.csv")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ontology.R
\name{get_ontology_by_level}
\alias{get_ontology_by_level}
\title{Get ontology entities by level}
\usage{
get_ontology_by_level(level, parent_id = NULL)
}
\arguments{
\item{level}{Ontology level (integer, 1--4).}

\item{parent_id}{Optional parent entity ID to filter by.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}}.
}
\description{
Get ontology entities by level
}
\examples{
\dontrun{
get_ontology_by_level(2)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/midas.R
\name{query_source_comparison}
\alias{query_source_comparison}
\title{Compare gene-disease sources (MIDAS)}
\usage{
query_source_comparison(
  gene_list,
  sources = NULL,
  mode = c("union", "intersection")
)
}
\arguments{
\item{gene_list}{Character vector of gene identifiers.}

\item{sources}{Character vector of source databases (or \code{NULL}).}

\item{mode}{Comparison mode: \code{"union"} (default) or \code{"intersection"}.}
}
\value{
A named list with elements: \verb{$mode}, \verb{$sources}, \verb{$sets}
(named list of gene vectors), \verb{$intersections}, \verb{$exclusives},
\verb{$genes_used}.
}
\description{
Compare how different evidence sources cover a gene list, producing
Venn-diagram-ready set data.
}
\examples{
\dontrun{
query_source_comparison(c("TP53", "BRCA1"), mode = "union")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/terms-mechanisms.R
\name{fetch_mechanism_filters}
\alias{fetch_mechanism_filters}
\title{Get mechanism filter options}
\usage{
fetch_mechanism_filters()
}
\value{
A named list with elements: \code{categories}, \code{omics_types},
\code{evidence_levels}, \code{confidence_levels}, \code{study_types}, \code{species}.
Each is a \code{\link[tibble:tibble]{tibble::tibble()}} with columns \code{value}, \code{label}, \code{count}.
}
\description{
Fetches all 6 filter option endpoints and returns them as a named list
of tibbles.
}
\examples{
\dontrun{
filters <- fetch_mechanism_filters()
filters$categories
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ontology.R
\name{export_ontology}
\alias{export_ontology}
\title{Export the TCM ontology}
\usage{
export_ontology(format = c("json", "owl", "csv"), file = NULL, depth = 4L)
}
\arguments{
\item{format}{Export format: \code{"json"}, \code{"owl"}, or \code{"csv"}.}

\item{file}{Output file path. If \code{NULL}, auto-named as
\code{"unitcm_ontology.{ext}"}.}

\item{depth}{Tree depth for JSON export (default 4).}
}
\value{
Invisible file path.
}
\description{
Download the full ontology in JSON, OWL/RDF, or CSV format.
}
\examples{
\dontrun{
export_ontology("csv")
export_ontology("json", depth = 2, file = "ontology_shallow.json")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netvis.R
\name{detect_communities}
\alias{detect_communities}
\title{Detect communities in a graph}
\usage{
detect_communities(nodes, edges)
}
\arguments{
\item{nodes}{Character vector of node IDs.}

\item{edges}{A data frame or list of edges, each with \code{source} and
\code{target} fields.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} with columns \code{node_id} and \code{community_id}.
}
\description{
POST a graph (nodes + edges) to the server for community detection.
}
\examples{
\dontrun{
detect_communities(
  nodes = c("A", "B", "C"),
  edges = data.frame(source = c("A", "B"), target = c("B", "C"))
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auth.R
\name{get_base_url}
\alias{get_base_url}
\title{Get the UniTCM API base URL}
\usage{
get_base_url()
}
\value{
A character string.
}
\description{
Checks in order: (1) session value set via \code{\link[=set_base_url]{set_base_url()}},
(2) option \code{unitcm.base_url}, (3) environment variable \code{UNITCM_BASE_URL},
(4) hardcoded default.
}
\examples{
\dontrun{
get_base_url()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/midas.R
\name{query_disease_intersection}
\alias{query_disease_intersection}
\title{Find disease intersection (MIDAS)}
\usage{
query_disease_intersection(disease_queries, sources = NULL)
}
\arguments{
\item{disease_queries}{Character vector of disease names/IDs.}

\item{sources}{Character vector of source databases (or \code{NULL}).}
}
\value{
A named list with elements: \verb{$diseases}, \verb{$per_source},
\verb{$targets}, \verb{$total_intersection_genes}.
}
\description{
Find genes shared across multiple diseases.
}
\examples{
\dontrun{
query_disease_intersection(c("breast cancer", "lung cancer"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transcriptome-hub.R
\name{search_transcriptomes}
\alias{search_transcriptomes}
\title{Search transcriptome datasets}
\usage{
search_transcriptomes(
  search = NULL,
  tcm_classification = NULL,
  organism = NULL,
  model_type = NULL,
  experiment_type = NULL,
  disease_classification = NULL,
  cell_line = NULL,
  comparison_type = NULL,
  confidence = NULL,
  sequence_type = NULL,
  page = 1L,
  page_size = 20L,
  all_pages = FALSE
)
}
\arguments{
\item{search}{Optional text search query.}

\item{tcm_classification}{TCM classification filter.}

\item{organism}{Organism filter.}

\item{model_type}{Model type filter.}

\item{experiment_type}{Experiment type filter.}

\item{disease_classification}{Disease classification filter.}

\item{cell_line}{Cell line filter.}

\item{comparison_type}{Comparison type filter.}

\item{confidence}{Confidence filter.}

\item{sequence_type}{Sequence type filter.}

\item{page}{Page number (default 1).}

\item{page_size}{Results per page (default 20).}

\item{all_pages}{If \code{TRUE}, fetch all pages via auto-pagination.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of datasets with attribute \code{"total"}.
}
\description{
Query the TCM Transcriptome Hub. This endpoint uses Style-B pagination
(\code{count}/\code{results} instead of \code{total}/\code{items}).
}
\examples{
\dontrun{
search_transcriptomes(search = "ginseng")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ingredients.R
\name{export_compound_module}
\alias{export_compound_module}
\title{Export compound data by module}
\usage{
export_compound_module(
  id,
  module = c("overview", "physicochemical", "admet", "targets"),
  file = NULL
)
}
\arguments{
\item{id}{The UniTCM ingredient ID.}

\item{module}{One of \code{"overview"}, \code{"physicochemical"}, \code{"admet"},
or \code{"targets"}.}

\item{file}{Output file path (auto-generated if \code{NULL}).}
}
\value{
Invisible file path.
}
\description{
Download a CSV of a specific data module for one compound.
}
\examples{
\dontrun{
export_compound_module("UNITCM_I00001", "admet")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/home.R
\name{fetch_home_stats}
\alias{fetch_home_stats}
\title{Get homepage statistics}
\usage{
fetch_home_stats()
}
\value{
A named list with fields: \code{total_datasets}, \code{total_downloads},
\code{total_file_size}, \code{recent_submissions_count}.
}
\description{
Get homepage statistics
}
\examples{
\dontrun{
fetch_home_stats()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auth.R
\name{set_base_url}
\alias{set_base_url}
\title{Set the UniTCM API base URL}
\usage{
set_base_url(url)
}
\arguments{
\item{url}{A character string. The base URL for the UniTCM API,
e.g. \code{"https://unitcm.qfxulab.com/api/v1"}.}
}
\value{
Invisible previous URL value.
}
\description{
Set the UniTCM API base URL
}
\examples{
\dontrun{
set_base_url("https://unitcm.qfxulab.com/api/v1")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ontology.R
\name{search_ontology_mapping}
\alias{search_ontology_mapping}
\title{Search ontology external mapping}
\usage{
search_ontology_mapping(database, external_id)
}
\arguments{
\item{database}{External database name. Must be one of: \code{"UMLS"},
\code{"SNOMED_CT"}, \code{"ICD11_TM"}, \code{"MeSH"}.}

\item{external_id}{The external identifier to look up.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of matched TCM entities.
}
\description{
Find TCM entities mapped to an external database identifier.
}
\examples{
\dontrun{
search_ontology_mapping("MeSH", "D008516")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transcriptome-hub.R
\name{get_transcriptome}
\alias{get_transcriptome}
\title{Get a single transcriptome dataset}
\usage{
get_transcriptome(dataset_id)
}
\arguments{
\item{dataset_id}{The dataset ID (e.g. \code{"TCMtrans00001"}).}
}
\value{
A named list with 35+ fields.
}
\description{
Get a single transcriptome dataset
}
\examples{
\dontrun{
get_transcriptome("TCMtrans00001")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ontology.R
\name{get_ontology_entity}
\alias{get_ontology_entity}
\title{Get a TCM ontology entity}
\usage{
get_ontology_entity(tcm_id)
}
\arguments{
\item{tcm_id}{The TCM ontology ID (e.g. \code{"TCM_0001"}).}
}
\value{
A named list with sub-elements: \code{ancestors}, \code{children},
\code{external_mappings}, \code{relations}.
}
\description{
Retrieve full detail for one entity including ancestors, children,
external mappings, and relations.
}
\examples{
\dontrun{
get_ontology_entity("TCM_0001")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ingredients.R
\name{fetch_compound_facets}
\alias{fetch_compound_facets}
\title{Get compound facets and statistics}
\usage{
fetch_compound_facets()
}
\value{
A named list with fields: \code{total}, \code{approved_count},
\code{lipinski_counts}, \code{drug_counts}, \code{mw_range}, \code{clogp_range},
\code{tpsa_range}, \code{qed_range}.
}
\description{
Returns summary statistics and filter option counts for the Ingredient
Explorer.
}
\examples{
\dontrun{
fetch_compound_facets()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ontology.R
\name{get_ontology_ancestors}
\alias{get_ontology_ancestors}
\title{Get ancestors of an ontology entity}
\usage{
get_ontology_ancestors(tcm_id)
}
\arguments{
\item{tcm_id}{The TCM ontology ID.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} with columns: \code{tcm_id}, \code{name}, \code{name_cn},
\code{level}.
}
\description{
Get ancestors of an ontology entity
}
\examples{
\dontrun{
get_ontology_ancestors("TCM_0001")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_enrichment}
\alias{plot_enrichment}
\title{Plot enrichment results}
\usage{
plot_enrichment(
  enrichment_result,
  type = c("dotplot", "barplot"),
  top_n = 20L,
  name_col = NULL,
  pvalue_col = NULL,
  count_col = NULL
)
}
\arguments{
\item{enrichment_result}{A \code{\link[tibble:tibble]{tibble::tibble()}} from an enrichment
analysis. Must contain columns mappable to term name and p-value.}

\item{type}{Plot type: \code{"dotplot"} (default) or \code{"barplot"}.}

\item{top_n}{Number of top terms to show (default 20).}

\item{name_col}{Column name for term labels (auto-detected if \code{NULL}).}

\item{pvalue_col}{Column name for p-values (auto-detected if \code{NULL}).}

\item{count_col}{Column name for gene counts (auto-detected if \code{NULL}).}
}
\value{
A \code{ggplot} object.
}
\description{
Create a dot plot or bar plot from enrichment analysis results
(e.g. from \code{\link[=query_disease_enrichment]{query_disease_enrichment()}}).
}
\examples{
\dontrun{
enrich <- query_disease_enrichment(c("TP53", "BRCA1", "EGFR"))
plot_enrichment(enrich)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/target2np.R
\name{batch_target2np}
\alias{batch_target2np}
\title{Batch query Target2NP by identifier list}
\usage{
batch_target2np(
  identifiers,
  id_type = c("gene_symbol", "uniprot_id", "entrez_gene_id")
)
}
\arguments{
\item{identifiers}{Character vector of identifiers (max 50).}

\item{id_type}{One of \code{"gene_symbol"} (default), \code{"uniprot_id"}, or
\code{"entrez_gene_id"}.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of matching records with attributes
\code{"total"}, \code{"queries_matched"}, and \code{"queries_not_found"}.
}
\description{
Look up interaction records for up to 50 gene symbols, UniProt IDs, or
Entrez gene IDs in one call.
}
\examples{
\dontrun{
batch_target2np(c("TP53", "BRCA1", "EGFR"))
batch_target2np(c("P04637", "P38398"), id_type = "uniprot_id")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/browse.R
\name{fetch_dataset_stats}
\alias{fetch_dataset_stats}
\title{Get TCMomics database statistics}
\usage{
fetch_dataset_stats()
}
\value{
A named list with fields: \code{total_datasets}, \code{total_downloads},
\code{omics_types_count}, \code{unique_organisms}.
}
\description{
Get TCMomics database statistics
}
\examples{
\dontrun{
fetch_dataset_stats()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/midas.R
\name{autocomplete_disease}
\alias{autocomplete_disease}
\title{Autocomplete disease names (MIDAS)}
\usage{
autocomplete_disease(q)
}
\arguments{
\item{q}{Search query (minimum 2 characters).}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} with columns: \code{disease_name}, \code{disease_id},
\code{gene_count}.
}
\description{
Search for disease names with autocomplete. Query must be at least 2
characters.
}
\examples{
\dontrun{
autocomplete_disease("breast")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/home.R
\name{fetch_latest_submissions}
\alias{fetch_latest_submissions}
\title{Get latest submissions}
\usage{
fetch_latest_submissions()
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of recent submissions with columns:
\code{submission_id}, \code{project_title}, \code{submitted_by}, \code{updated_at},
\code{institution}, \code{total_file_size}.
}
\description{
Get latest submissions
}
\examples{
\dontrun{
fetch_latest_submissions()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ontology.R
\name{get_ontology_descendants}
\alias{get_ontology_descendants}
\title{Get all descendants of an ontology entity}
\usage{
get_ontology_descendants(tcm_id, max_level = NULL)
}
\arguments{
\item{tcm_id}{The TCM ontology ID.}

\item{max_level}{Maximum depth to descend (integer or \code{NULL} for all).}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}}.
}
\description{
Get all descendants of an ontology entity
}
\examples{
\dontrun{
get_ontology_descendants("TCM_0001", max_level = 2)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ontology.R
\name{search_ontology}
\alias{search_ontology}
\title{Search the TCM Ontology}
\usage{
search_ontology(q, limit = 20L, level = NULL, category = NULL)
}
\arguments{
\item{q}{Search query (required).}

\item{limit}{Maximum results to return (default 20).}

\item{level}{Filter by ontology level (integer, 1--4).}

\item{category}{Filter by top-level category.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} with columns: \code{tcm_id}, \code{name}, \code{name_cn},
\code{level}, \code{path}, \code{match_field}, \code{highlight}.
}
\description{
Full-text search across TCM ontology entities.
}
\examples{
\dontrun{
search_ontology("Qi stagnation")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/target2np.R
\name{fetch_target2np_stats}
\alias{fetch_target2np_stats}
\title{Fetch Target2NP database statistics}
\usage{
fetch_target2np_stats()
}
\value{
A named list with \code{total_records} and four distribution lists.
}
\description{
Returns counts and distributions across source databases, evidence
levels, target organisms, and activity types.
}
\examples{
\dontrun{
stats <- fetch_target2np_stats()
stats$total_records
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/browse.R
\name{fetch_dataset_facets}
\alias{fetch_dataset_facets}
\title{Get dataset facets}
\usage{
fetch_dataset_facets()
}
\value{
A named list of tibbles for each facet field.
}
\description{
Returns available filter values and their counts for the TCMomics
database.
}
\examples{
\dontrun{
facets <- fetch_dataset_facets()
facets$omics_type
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/midas.R
\name{fetch_midas_sources}
\alias{fetch_midas_sources}
\title{Get MIDAS data sources}
\usage{
fetch_midas_sources()
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} with columns: \code{key}, \code{label}, \code{has_score},
\code{weight}, \code{row_count}.
}
\description{
List all available gene-disease association databases.
}
\examples{
\dontrun{
fetch_midas_sources()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/herbs.R
\name{fetch_herb_facets}
\alias{fetch_herb_facets}
\title{Get herb filter facets}
\usage{
fetch_herb_facets()
}
\value{
A named list of tibbles, one per facet field
(e.g. \code{therapeutic_en_class}, \code{family}, \code{toxicity}).
}
\description{
Returns available filter values and their counts for the Herb Explorer.
}
\examples{
\dontrun{
facets <- fetch_herb_facets()
facets$toxicity
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/disease-formula.R
\name{fetch_disease_tree}
\alias{fetch_disease_tree}
\title{Get the ICD-11 disease classification tree}
\usage{
fetch_disease_tree()
}
\value{
A recursive nested list with structure
\code{list(label, count, children = list(...))}.
}
\description{
Returns the full 4-level ICD-11 disease classification tree used by the
Disease-Formula Atlas.
}
\examples{
\dontrun{
tree <- fetch_disease_tree()
names(tree[[1]])
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transcriptome-hub.R
\name{get_analysis_data}
\alias{get_analysis_data}
\title{Get analysis data for a transcriptome dataset}
\usage{
get_analysis_data(dataset_id, module, gene = NULL)
}
\arguments{
\item{dataset_id}{The dataset ID.}

\item{module}{Analysis module name. One of: \code{"meta"}, \code{"expression"},
\code{"deg"}, \code{"go"}, \code{"kegg"}, \code{"gsea"}, \code{"ppi"}, \code{"immune"}, \code{"tf"},
\code{"pca"}, \code{"qc"}.}

\item{gene}{Optional gene filter (for expression module only).}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} for tabular modules (deg, go, kegg, gsea,
immune, tf), or a named list for structured modules (meta, expression,
ppi, pca, qc).
}
\description{
Retrieve data for a specific analysis module. Return type varies by
module.
}
\examples{
\dontrun{
get_analysis_data("TCMtrans00001", "deg")
get_analysis_data("TCMtrans00001", "expression", gene = "TP53")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netvis.R
\name{find_path}
\alias{find_path}
\title{Find shortest path between two nodes}
\usage{
find_path(source, target, max_depth = 4L)
}
\arguments{
\item{source}{Source node ID.}

\item{target}{Target node ID.}

\item{max_depth}{Maximum path depth (max 8, default 4).}
}
\value{
A named list with \verb{$nodes} (\code{\link[tibble:tibble]{tibble::tibble()}}) and \verb{$edges}
(\code{\link[tibble:tibble]{tibble::tibble()}}).
}
\description{
Find shortest path between two nodes
}
\examples{
\dontrun{
find_path("H:UNITCM_H001", "T:TP53")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netvis.R
\name{get_neighbors}
\alias{get_neighbors}
\title{Get neighbors of a node}
\usage{
get_neighbors(node_id, depth = 1L, limit = 50L, node_types = NULL)
}
\arguments{
\item{node_id}{Node ID (e.g. \code{"H:UNITCM_H001"}).}

\item{depth}{Neighbor depth (1--3, default 1).}

\item{limit}{Maximum neighbors (max 200, default 50).}

\item{node_types}{Comma-separated node types to include.}
}
\value{
A named list with \verb{$nodes} (\code{\link[tibble:tibble]{tibble::tibble()}}), \verb{$edges}
(\code{\link[tibble:tibble]{tibble::tibble()}}), and \verb{$has_more}.
}
\description{
Get neighbors of a node
}
\examples{
\dontrun{
get_neighbors("H:UNITCM_H001", depth = 1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ingredients.R
\name{get_compound_targets}
\alias{get_compound_targets}
\title{Get predicted targets for a compound}
\usage{
get_compound_targets(
  id,
  method = c("drugclip", "chembl", "both"),
  page = 1L,
  page_size = 20L
)
}
\arguments{
\item{id}{The UniTCM ingredient ID.}

\item{method}{One of \code{"drugclip"}, \code{"chembl"}, or \code{"both"} (default
\code{"drugclip"}).}

\item{page}{Page number (for ChEMBL targets, default 1).}

\item{page_size}{Results per page (for ChEMBL targets, default 20).}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of targets. When \code{method = "both"}, a
\code{source} column is added to distinguish results.
}
\description{
Retrieve target predictions from DrugCLIP, ChEMBL similarity search,
or both.
}
\examples{
\dontrun{
get_compound_targets("UNITCM_I00001")
get_compound_targets("UNITCM_I00001", method = "both")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ontology.R
\name{list_ontology_categories}
\alias{list_ontology_categories}
\title{List top-level ontology categories}
\usage{
list_ontology_categories()
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of level-1 entities.
}
\description{
List top-level ontology categories
}
\examples{
\dontrun{
list_ontology_categories()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ontology.R
\name{fetch_ontology_tree}
\alias{fetch_ontology_tree}
\title{Fetch the TCM ontology tree}
\usage{
fetch_ontology_tree(depth = 4L)
}
\arguments{
\item{depth}{Tree depth to return (1--10, default 4).}
}
\value{
A recursive nested list: \code{list(tcm_id, name, name_cn, level, children = list(...))}.
}
\description{
Returns the full ontology as a recursive nested list.
}
\examples{
\dontrun{
tree <- fetch_ontology_tree(depth = 2)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/midas.R
\name{query_disease_enrichment}
\alias{query_disease_enrichment}
\title{Disease enrichment analysis (MIDAS)}
\usage{
query_disease_enrichment(
  gene_list,
  gene_id_type = "symbol",
  sources = NULL,
  min_sources = 1L,
  background_gene_count = 20000L,
  p_value_cutoff = 0.05,
  correction_method = "fdr",
  min_hit_count = 2L,
  page = 1L,
  page_size = 20L
)
}
\arguments{
\item{gene_list}{Character vector of gene identifiers.}

\item{gene_id_type}{ID type: \code{"symbol"} (default), \code{"entrez"}, or
\code{"ensembl"}.}

\item{sources}{Character vector of source databases (or \code{NULL}).}

\item{min_sources}{Minimum supporting sources (default 1).}

\item{background_gene_count}{Background gene count (default 20000).}

\item{p_value_cutoff}{P-value significance cutoff (default 0.05).}

\item{correction_method}{P-value correction: \code{"fdr"} (default),
\code{"bonferroni"}, \code{"holm"}, or \code{"none"}.}

\item{min_hit_count}{Minimum gene hits per disease (default 2).}

\item{page}{Page number (default 1).}

\item{page_size}{Results per page (default 20).}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of enrichment results with attributes
\code{"total_significant"}, \code{"total_tested"}, and \code{"input_gene_count"}.
}
\description{
Perform Fisher's exact test enrichment analysis to identify diseases
significantly associated with a gene list.
}
\examples{
\dontrun{
query_disease_enrichment(c("TP53", "BRCA1", "EGFR", "VEGFA"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/terms-mechanisms.R
\name{search_mechanisms}
\alias{search_mechanisms}
\title{Search terms molecular mechanisms}
\usage{
search_mechanisms(
  search = NULL,
  category = NULL,
  omics_type = NULL,
  evidence_level = NULL,
  confidence_level = NULL,
  study_type = NULL,
  species = NULL,
  page = 1L,
  page_size = 20L,
  all_pages = FALSE
)
}
\arguments{
\item{search}{Optional text search query.}

\item{category}{Category filter.}

\item{omics_type}{Omics type filter.}

\item{evidence_level}{Evidence level filter.}

\item{confidence_level}{Confidence level filter.}

\item{study_type}{Study type filter.}

\item{species}{Species filter.}

\item{page}{Page number (default 1).}

\item{page_size}{Results per page (default 20).}

\item{all_pages}{If \code{TRUE}, fetch all pages via auto-pagination.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of mechanism terms with attribute \code{"total"}.
}
\description{
Query the Terms Molecular Mechanisms database with optional filters.
}
\examples{
\dontrun{
search_mechanisms(search = "Qi deficiency")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netvis.R
\name{get_node_detail}
\alias{get_node_detail}
\title{Get node detail}
\usage{
get_node_detail(node_id)
}
\arguments{
\item{node_id}{Node ID.}
}
\value{
A named list with fields: \code{id}, \code{type}, \code{label}, \code{label_cn},
\code{properties}, \code{detail_url}.
}
\description{
Get node detail
}
\examples{
\dontrun{
get_node_detail("H:UNITCM_H001")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auth.R
\name{set_api_key}
\alias{set_api_key}
\title{Set a UniTCM API Key}
\usage{
set_api_key(api_key, keyring = FALSE)
}
\arguments{
\item{api_key}{A character string. The API key (starts with \code{unitcm_}).}

\item{keyring}{Logical. If \code{TRUE}, also store in system keyring.}
}
\value{
Invisible \code{NULL}.
}
\description{
Stores the API key in session memory. Optionally also stores it in the
system keyring (requires the \pkg{keyring} package).
}
\examples{
\dontrun{
set_api_key("unitcm_your_key_here")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\name{as_igraph}
\alias{as_igraph}
\title{Convert a NetVis graph response to igraph}
\usage{
as_igraph(graph_response)
}
\arguments{
\item{graph_response}{A list with \verb{$nodes} and \verb{$edges} tibbles, as
returned by \code{\link[=get_neighbors]{get_neighbors()}}, \code{\link[=get_subgraph]{get_subgraph()}}, or \code{\link[=find_path]{find_path()}}.}
}
\value{
An \code{igraph} graph object.
}
\description{
Convert a NetVis graph response to igraph
}
\examples{
\dontrun{
resp <- get_neighbors("H:UNITCM_H001")
g <- as_igraph(resp)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transcriptome-hub.R
\name{fetch_transcriptome_filters}
\alias{fetch_transcriptome_filters}
\title{Get transcriptome filter options}
\usage{
fetch_transcriptome_filters()
}
\value{
A named list of character vectors for each filter field.
}
\description{
Get transcriptome filter options
}
\examples{
\dontrun{
filters <- fetch_transcriptome_filters()
filters$organism
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auth.R
\name{set_unitcm_token}
\alias{set_unitcm_token}
\title{Set a UniTCM API token}
\usage{
set_unitcm_token(token, keyring = FALSE)
}
\arguments{
\item{token}{A character string. The bearer token.}

\item{keyring}{Logical. If \code{TRUE}, also store in system keyring.}
}
\value{
Invisible \code{NULL}.
}
\description{
Stores the token in session memory. Optionally also stores it in the
system keyring (requires the \pkg{keyring} package).
}
\examples{
\dontrun{
set_unitcm_token("my-secret-token")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/browse.R
\name{export_datasets}
\alias{export_datasets}
\title{Export datasets to CSV}
\usage{
export_datasets(
  q = NULL,
  tcm = NULL,
  omics = NULL,
  source = NULL,
  organism = NULL,
  tissue = NULL,
  disease = NULL,
  repo = NULL,
  year_min = NULL,
  year_max = NULL,
  sort = NULL,
  file = "datasets_export.csv"
)
}
\arguments{
\item{q}{Optional search query string.}

\item{tcm}{TCM classification filter.}

\item{omics}{Omics type filter.}

\item{source}{Source type filter.}

\item{organism}{Organism filter.}

\item{tissue}{Tissue filter.}

\item{disease}{Disease filter.}

\item{repo}{Repository filter.}

\item{year_min}{Minimum publication year.}

\item{year_max}{Maximum publication year.}

\item{sort}{Sort field: \code{"relevance"}, \code{"date_desc"}, \code{"views_desc"}, or
\code{"downloads_desc"}.}

\item{file}{Output file path (default \code{"datasets_export.csv"}).}
}
\value{
Invisible file path.
}
\description{
Download a CSV export of datasets matching the given filters.
}
\examples{
\dontrun{
export_datasets(omics = "Transcriptomics", file = "transcriptomics.csv")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/target2np.R
\name{search_target2np_drugclip}
\alias{search_target2np_drugclip}
\title{Search DrugCLIP predicted compound-target interactions}
\usage{
search_target2np_drugclip(
  search = NULL,
  search_field = c("all", "gene_symbol", "compound_name", "inchikey"),
  search_mode = c("exact", "fuzzy"),
  min_score = NULL,
  confidence = NULL,
  page = 1L,
  page_size = 20L,
  all_pages = FALSE
)
}
\arguments{
\item{search}{Free-text search.}

\item{search_field}{One of \code{"all"}, \code{"gene_symbol"}, \code{"compound_name"},
or \code{"inchikey"}.}

\item{search_mode}{\code{"exact"} or \code{"fuzzy"}.}

\item{min_score}{Minimum DrugCLIP score (0-1).}

\item{confidence}{One of \code{"high"} (>= 0.8), \code{"medium"} (0.5-0.8),
\code{"low"} (< 0.5).}

\item{page, page_size}{Pagination.}

\item{all_pages}{If \code{TRUE}, fetch all pages.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of DrugCLIP predictions with attribute
\code{"total"}.
}
\description{
Query DrugCLIP deep-learning predictions, with optional confidence
filtering by predicted score.
}
\examples{
\dontrun{
search_target2np_drugclip(search = "quercetin", confidence = "high")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\name{build_hct_network}
\alias{build_hct_network}
\title{Build an Herb-Compound-Target network}
\usage{
build_hct_network(
  herbs,
  target_method = "drugclip",
  max_compounds = 50L,
  progress = TRUE
)
}
\arguments{
\item{herbs}{Character vector of herb names or UniTCM herb IDs.}

\item{target_method}{Target prediction method passed to
\code{\link[=get_compound_targets]{get_compound_targets()}}: \code{"drugclip"} (default), \code{"chembl"}, or
\code{"both"}.}

\item{max_compounds}{Maximum compounds per herb to include (default 50).}

\item{progress}{Show progress messages (default \code{TRUE}).}
}
\value{
An \code{igraph} graph object with vertex attributes \code{name}, \code{type}
(\code{"herb"}, \code{"compound"}, \code{"target"}), and \code{label}.
}
\description{
Given herb names or IDs, fetches herb-compound-target data from the API
and constructs a typed \code{igraph} network.
}
\examples{
\dontrun{
g <- build_hct_network(c("UNITCM_H001", "UNITCM_H002"))
igraph::vcount(g)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unitcm-package.R
\docType{package}
\name{unitcm-package}
\alias{unitcm}
\alias{unitcm-package}
\title{unitcm: Client for the 'UniTCM' Traditional Chinese Medicine Platform}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Provides functions to query the 'UniTCM' API (\url{https://unitcm.qfxulab.com}), covering herb exploration, compound/ADMET (Absorption, Distribution, Metabolism, Excretion, and Toxicity) data, disease-formula associations, Traditional Chinese Medicine (TCM) ontology, transcriptomics, and gene-disease analysis (MIDAS, Mining Integrated Disease Association Sources).
}
\seealso{
Useful links:
\itemize{
  \item \url{https://zx122ty.github.io/UniTCM_R_Package/}
  \item \url{https://github.com/zx122ty/UniTCM_R_Package}
  \item Report bugs at \url{https://github.com/zx122ty/UniTCM_R_Package/issues}
}

}
\author{
\strong{Maintainer}: Xiao Zheng \email{12519088@zju.edu.cn} (\href{https://orcid.org/0009-0000-3918-5725}{ORCID})

Authors:
\itemize{
  \item Xiao Zheng \email{12519088@zju.edu.cn} (\href{https://orcid.org/0009-0000-3918-5725}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/disease-formula.R
\name{list_book_sources}
\alias{list_book_sources}
\title{List book sources}
\usage{
list_book_sources()
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} with columns \code{value}, \code{label}, \code{count}.
}
\description{
List book sources
}
\examples{
\dontrun{
list_book_sources()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/browse.R
\name{get_similar_datasets}
\alias{get_similar_datasets}
\title{Get similar datasets}
\usage{
get_similar_datasets(submission_id)
}
\arguments{
\item{submission_id}{The submission ID.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} with columns: \code{submission_id},
\code{project_title}, \code{TCM_classification}, \code{similarity_score}.
}
\description{
Find datasets similar to a given submission based on content similarity.
}
\examples{
\dontrun{
get_similar_datasets("TMA2025001")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/herbs.R
\name{get_herb_compounds}
\alias{get_herb_compounds}
\title{Get compounds for a herb}
\usage{
get_herb_compounds(herb_id, page = 1L, page_size = 20L, all_pages = FALSE)
}
\arguments{
\item{herb_id}{The UniTCM herb ID.}

\item{page}{Page number (default 1).}

\item{page_size}{Results per page (default 20).}

\item{all_pages}{If \code{TRUE}, fetch all pages.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of compounds with attribute \code{"total"}.
}
\description{
List chemical compounds (ingredients) associated with a specific herb.
}
\examples{
\dontrun{
get_herb_compounds("UNITCM_H001")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{unitcm_cache_clear}
\alias{unitcm_cache_clear}
\title{Clear unitcm cache}
\usage{
unitcm_cache_clear()
}
\value{
Invisible \code{NULL}.
}
\description{
Clears any memoized API results. Currently a no-op placeholder for
future caching support.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tcm-term.R
\name{get_term}
\alias{get_term}
\title{Get a single term by ID}
\usage{
get_term(term_id)
}
\arguments{
\item{term_id}{The term ID.}
}
\value{
A named list with fields including \code{chinese_name}, \code{pinyin},
\code{english_name}, \code{latin_name}, \code{description_english}, etc.
}
\description{
Retrieve full detail for one term from the TCM Bilingual Corpus.
}
\examples{
\dontrun{
get_term("TCM_T001")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netvis.R
\name{get_node_metrics}
\alias{get_node_metrics}
\title{Get node metrics}
\usage{
get_node_metrics(node_id)
}
\arguments{
\item{node_id}{Node ID.}
}
\value{
A named list with fields: \code{node_id}, \code{degree}, \code{neighbor_types}.
}
\description{
Get node metrics
}
\examples{
\dontrun{
get_node_metrics("H:UNITCM_H001")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/target2np.R
\name{search_target2np}
\alias{search_target2np}
\title{Search Target2NP compound-target interactions}
\usage{
search_target2np(
  search = NULL,
  search_field = c("all", "gene_symbol", "compound_name", "uniprot_id", "inchikey",
    "pubchem_cid", "chembl_id"),
  search_mode = c("exact", "fuzzy"),
  source_db = NULL,
  evidence_level = NULL,
  evidence_label = NULL,
  target_organism = NULL,
  interaction_type = NULL,
  activity_type = NULL,
  page = 1L,
  page_size = 20L,
  all_pages = FALSE
)
}
\arguments{
\item{search}{Free-text search query.}

\item{search_field}{Field to restrict the search to. One of \code{"all"},
\code{"gene_symbol"}, \code{"compound_name"}, \code{"uniprot_id"}, \code{"inchikey"},
\code{"pubchem_cid"}, or \code{"chembl_id"}.}

\item{search_mode}{\code{"exact"} (case-insensitive equality) or \code{"fuzzy"}
(substring match).}

\item{source_db}{Filter by source database (e.g. \code{"BindingDB"}).}

\item{evidence_level}{Filter by evidence level (integer 1-4 as string).}

\item{evidence_label}{Filter by evidence label.}

\item{target_organism}{Filter by target organism (e.g. \code{"Homo sapiens"}).}

\item{interaction_type}{Filter by interaction type.}

\item{activity_type}{Filter by activity type (e.g. \code{"IC50"}, \code{"Ki"}).}

\item{page}{Page number (default 1).}

\item{page_size}{Results per page (default 20, max 100).}

\item{all_pages}{If \code{TRUE}, fetch all pages.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of interaction records with attribute
\code{"total"}.
}
\description{
Query the UniTCM Target2NP database of natural-product to protein-target
interactions, combining records from experimental sources such as
BindingDB, HERB2, NPASS, BATMAN, and others.
}
\examples{
\dontrun{
search_target2np(search = "quercetin")
search_target2np(search = "TP53", search_field = "gene_symbol",
                 source_db = "BindingDB")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transcriptome-hub.R
\name{fetch_transcriptome_stats}
\alias{fetch_transcriptome_stats}
\title{Get Transcriptome Hub statistics}
\usage{
fetch_transcriptome_stats()
}
\value{
A named list with fields: \code{total_datasets}, \code{total_organisms},
\code{total_tcm_entities}, \code{total_analysis_modules}, plus distribution data.
}
\description{
Get Transcriptome Hub statistics
}
\examples{
\dontrun{
fetch_transcriptome_stats()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/midas.R
\name{fetch_midas_stats}
\alias{fetch_midas_stats}
\title{Get MIDAS statistics}
\usage{
fetch_midas_stats()
}
\value{
A named list with fields: \code{total_associations}, \code{total_genes},
\code{total_diseases}, \code{sources}.
}
\description{
Get MIDAS statistics
}
\examples{
\dontrun{
fetch_midas_stats()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/midas.R
\name{convert_gene_ids}
\alias{convert_gene_ids}
\title{Convert gene identifiers (MIDAS)}
\usage{
convert_gene_ids(identifiers)
}
\arguments{
\item{identifiers}{Character vector of gene identifiers.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} with columns including match status.
}
\description{
Convert a mixed list of gene identifiers (symbols, Entrez IDs, Ensembl
IDs) to a standardized mapping.
}
\examples{
\dontrun{
convert_gene_ids(c("TP53", "7157", "ENSG00000141510"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ingredients.R
\name{export_compounds}
\alias{export_compounds}
\title{Export compounds to CSV}
\usage{
export_compounds(
  q = NULL,
  mw_min = NULL,
  mw_max = NULL,
  clogp_min = NULL,
  clogp_max = NULL,
  tpsa_min = NULL,
  tpsa_max = NULL,
  qed_min = NULL,
  qed_max = NULL,
  ring_count_min = NULL,
  ring_count_max = NULL,
  lipinski = NULL,
  is_drug = NULL,
  sort = NULL,
  file = "compounds_export.csv"
)
}
\arguments{
\item{q}{Search query (name, SMILES, formula, or CAS number).}

\item{mw_min, mw_max}{Molecular weight range.}

\item{clogp_min, clogp_max}{CLogP range.}

\item{tpsa_min, tpsa_max}{Topological polar surface area range.}

\item{qed_min, qed_max}{QED score range.}

\item{ring_count_min, ring_count_max}{Ring count range.}

\item{lipinski}{Lipinski rule filter (character vector, comma-collapsed).}

\item{is_drug}{Approved drug filter (logical or \code{NULL}).}

\item{sort}{Sort field (e.g. \code{"mw"}, \code{"-mw"}).}

\item{file}{Output file path (default \code{"compounds_export.csv"}).}
}
\value{
Invisible file path.
}
\description{
Download a CSV export of compounds matching the given filters
(max 10,000 rows).
}
\examples{
\dontrun{
export_compounds(mw_min = 200, file = "filtered_compounds.csv")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/disease-formula.R
\name{get_formula_doses}
\alias{get_formula_doses}
\title{Get herb doses for a formula}
\usage{
get_formula_doses(order_id)
}
\arguments{
\item{order_id}{The formula order ID.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} with columns: \code{id}, \code{herb_name},
\code{original_dose}, \code{composition_ratio}, \code{modern_dose_g},
\code{clinical_ref_dose_g}, \code{dynasty}, \code{notes}, \code{herb_ids}.
}
\description{
Retrieve the composition and dosage information for a specific formula.
}
\examples{
\dontrun{
get_formula_doses(1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{flatten_response}
\alias{flatten_response}
\title{Flatten a nested API response to a tibble}
\usage{
flatten_response(x)
}
\arguments{
\item{x}{A named list from an API response.}
}
\value{
A single-row \code{\link[tibble:tibble]{tibble::tibble()}}.
}
\description{
Recursively flattens a nested list into a single-row tibble. Nested lists
that cannot be further flattened are kept as list-columns.
}
\examples{
\dontrun{
herb <- get_herb("UNITCM_H001")
flatten_response(herb)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/herbs.R
\name{get_herb}
\alias{get_herb}
\title{Get a single herb by ID}
\usage{
get_herb(herb_id)
}
\arguments{
\item{herb_id}{The UniTCM herb ID (e.g. \code{"UNITCM_H001"}).}
}
\value{
A named list with 31 fields including cross-reference IDs.
}
\description{
Retrieve full detail for one herb from the Herb Explorer.
}
\examples{
\dontrun{
get_herb("UNITCM_H001")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/herbs.R
\name{export_herb_compounds}
\alias{export_herb_compounds}
\title{Export herb compounds to CSV}
\usage{
export_herb_compounds(herb_id, file = "herb_compounds_export.csv")
}
\arguments{
\item{herb_id}{The UniTCM herb ID.}

\item{file}{Output file path (default \code{"herb_compounds_export.csv"}).}
}
\value{
Invisible file path.
}
\description{
Download a CSV export of all compounds for a specific herb.
}
\examples{
\dontrun{
export_herb_compounds("UNITCM_H001")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ingredients.R
\name{get_compound}
\alias{get_compound}
\title{Get a single compound by ID}
\usage{
get_compound(id)
}
\arguments{
\item{id}{The UniTCM ingredient ID (e.g. \code{"UNITCM_I00001"}).}
}
\value{
A named list with 26+ fields including an \code{xref} sub-list.
}
\description{
Retrieve full detail for one compound including cross-references.
}
\examples{
\dontrun{
get_compound("UNITCM_I00001")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/disease-formula.R
\name{search_formulas}
\alias{search_formulas}
\title{Search formulas in the Disease-Formula Atlas}
\usage{
search_formulas(
  q = NULL,
  level1 = NULL,
  level2 = NULL,
  level3 = NULL,
  level4 = NULL,
  book_sources = NULL,
  origin_sources = NULL,
  dosage_forms = NULL,
  mapping_confidence = NULL,
  page = 1L,
  page_size = 20L,
  all_pages = FALSE
)
}
\arguments{
\item{q}{Optional search query string.}

\item{level1, level2, level3, level4}{ICD-11 disease classification levels.}

\item{book_sources}{Book source filter (character vector).}

\item{origin_sources}{Origin source filter (character vector).}

\item{dosage_forms}{Dosage form filter (character vector).}

\item{mapping_confidence}{Mapping confidence filter (character vector).}

\item{page}{Page number (default 1).}

\item{page_size}{Results per page (default 20, max 100).}

\item{all_pages}{If \code{TRUE}, fetch all pages via auto-pagination.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of formulas with attribute \code{"total"}.
}
\description{
Query the Disease-Formula Atlas with optional text search and ICD-11
disease classification filters. Multi-value parameters accept character
vectors and are collapsed to comma-separated strings internally.
}
\examples{
\dontrun{
search_formulas(q = "insomnia")
search_formulas(level1 = "Neoplasms", mapping_confidence = "high")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/browse.R
\name{search_datasets}
\alias{search_datasets}
\title{Search TCMomics datasets}
\usage{
search_datasets(
  q = NULL,
  tcm = NULL,
  omics = NULL,
  source = NULL,
  organism = NULL,
  tissue = NULL,
  disease = NULL,
  repo = NULL,
  year_min = NULL,
  year_max = NULL,
  sort = NULL,
  search_mode = NULL,
  page = 1L,
  page_size = 20L,
  all_pages = FALSE
)
}
\arguments{
\item{q}{Optional search query string.}

\item{tcm}{TCM classification filter.}

\item{omics}{Omics type filter.}

\item{source}{Source type filter.}

\item{organism}{Organism filter.}

\item{tissue}{Tissue filter.}

\item{disease}{Disease filter.}

\item{repo}{Repository filter.}

\item{year_min}{Minimum publication year.}

\item{year_max}{Maximum publication year.}

\item{sort}{Sort field: \code{"relevance"}, \code{"date_desc"}, \code{"views_desc"}, or
\code{"downloads_desc"}.}

\item{search_mode}{Search mode: \code{"fuzzy"} (default) or \code{"exact"}.}

\item{page}{Page number (default 1).}

\item{page_size}{Results per page (default 20).}

\item{all_pages}{If \code{TRUE}, fetch all pages via auto-pagination.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} of datasets with attribute \code{"total"}.
}
\description{
Query the TCMomics multi-omics database with optional text search and
faceted filters.
}
\examples{
\dontrun{
search_datasets(q = "ginseng", omics = "Transcriptomics")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/disease-formula.R
\name{get_formula}
\alias{get_formula}
\title{Get a single formula by order ID}
\usage{
get_formula(order_id)
}
\arguments{
\item{order_id}{The formula order ID (integer or character).}
}
\value{
A named list with 30+ fields.
}
\description{
Retrieve full detail for one formula from the Disease-Formula Atlas.
}
\examples{
\dontrun{
get_formula(1)
}
}
